@@ -8,6 +8,7 @@
 
 #include "Audacity.h"
 #include "FileException.h"
+
 #include "Internat.h"
 #include "Prefs.h"
 
@@ -15,12 +16,6 @@ FileException::~FileException()
 {
 }
 
-std::unique_ptr< AudacityException > FileException::Move()
-{
-   return std::unique_ptr< AudacityException >
-      { safenew FileException{ std::move( *this ) } };
-}
-
 wxString FileException::ErrorMessage() const
 {
    wxString format;
