@@ -45,17 +45,22 @@ and in the spectrogram spectral selection.
 
 #include <algorithm>
 
+#include <wx/setup.h> // for wxUSE_* macros
+
 #include <wx/brush.h>
 #include <wx/button.h>
+#include <wx/checkbox.h>
 #include <wx/choice.h>
+#include <wx/dcclient.h>
 #include <wx/font.h>
 #include <wx/image.h>
 #include <wx/dcmemory.h>
-#include <wx/msgdlg.h>
 #include <wx/file.h>
 #include <wx/filedlg.h>
 #include <wx/intl.h>
+#include <wx/scrolbar.h>
 #include <wx/sizer.h>
+#include <wx/slider.h>
 #include <wx/statbmp.h>
 #include <wx/stattext.h>
 #include <wx/statusbr.h>
@@ -80,10 +85,14 @@ and in the spectrogram spectral selection.
 
 #include "WaveTrack.h"
 
-#include "Experimental.h"
-
 #include "./widgets/LinkingHtmlWindow.h"
 #include "./widgets/HelpSystem.h"
+#include "widgets/ErrorDialog.h"
+#include "widgets/Ruler.h"
+
+#if wxUSE_ACCESSIBILITY
+#include "widgets/WindowAccessible.h"
+#endif
 
 DEFINE_EVENT_TYPE(EVT_FREQWINDOW_RECALC);
 
@@ -206,39 +215,43 @@ FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
    if (!p)
       return;
 
-   wxArrayString algChoices;
-   algChoices.Add(_("Spectrum"));
-   algChoices.Add(_("Standard Autocorrelation"));
-   algChoices.Add(_("Cuberoot Autocorrelation"));
-   algChoices.Add(_("Enhanced Autocorrelation"));
-     /* i18n-hint: This is a technical term, derived from the word
-      * "spectrum".  Do not translate it unless you are sure you
-      * know the correct technical word in your language. */
-   algChoices.Add(_("Cepstrum"));
+   wxArrayStringEx algChoices{
+      _("Spectrum") ,
+      _("Standard Autocorrelation") ,
+      _("Cuberoot Autocorrelation") ,
+      _("Enhanced Autocorrelation") ,
+        /* i18n-hint: This is a technical term, derived from the word
+         * "spectrum".  Do not translate it unless you are sure you
+         * know the correct technical word in your language. */
+      _("Cepstrum") ,
+   };
 
-   wxArrayString sizeChoices;
-   sizeChoices.Add(wxT("128"));
-   sizeChoices.Add(wxT("256"));
-   sizeChoices.Add(wxT("512"));
-   sizeChoices.Add(wxT("1024"));
-   sizeChoices.Add(wxT("2048"));
-   sizeChoices.Add(wxT("4096"));
-   sizeChoices.Add(wxT("8192"));
-   sizeChoices.Add(wxT("16384"));
-   sizeChoices.Add(wxT("32768"));
-   sizeChoices.Add(wxT("65536"));
+   wxArrayStringEx sizeChoices{
+      wxT("128") ,
+      wxT("256") ,
+      wxT("512") ,
+      wxT("1024") ,
+      wxT("2048") ,
+      wxT("4096") ,
+      wxT("8192") ,
+      wxT("16384") ,
+      wxT("32768") ,
+      wxT("65536") ,
+   };
 
-   wxArrayString funcChoices;
+   wxArrayStringEx funcChoices;
    for (int i = 0, cnt = NumWindowFuncs(); i < cnt; i++)
    {
-      /* i18n-hint: This refers to a "window function", used in the
+      /* i18n-hint: This refers to a "window function",
+       * such as Hann or Rectangular, used in the
        * Frequency analyze dialog box. */
-      funcChoices.Add(wxString(WindowFuncName(i)) + wxT(" ") + _("window"));
+      funcChoices.push_back(wxString::Format("%s window",  WindowFuncName(i) ) );
    }
 
-   wxArrayString axisChoices;
-   axisChoices.Add(_("Linear frequency"));
-   axisChoices.Add(_("Log frequency"));
+   wxArrayStringEx axisChoices{
+      _("Linear frequency") ,
+      _("Log frequency") ,
+   };
 
    mFreqFont = wxFont(fontSize, wxFONTFAMILY_SWISS, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
    mArrowCursor = std::make_unique<wxCursor>(wxCURSOR_ARROW);
@@ -279,17 +292,16 @@ FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
 
       S.StartVerticalLay(2);
       {
-         vRuler = safenew RulerPanel(this, wxID_ANY);
-         vRuler->ruler.SetBounds(0, 0, 100, 100); // Ruler can't handle small sizes
-         vRuler->ruler.SetOrientation(wxVERTICAL);
-         vRuler->ruler.SetRange(0.0, -dBRange);
-         vRuler->ruler.SetFormat(Ruler::LinearDBFormat);
-         vRuler->ruler.SetUnits(_("dB"));
-         vRuler->ruler.SetLabelEdges(true);
-         int w;
-         vRuler->ruler.GetMaxSize(&w, NULL);
-         vRuler->SetMinSize(wxSize(w, 150));  // height needed for wxGTK
-         vRuler->SetTickColour( theTheme.Colour( clrGraphLabels ));
+         vRuler = safenew RulerPanel(
+            this, wxID_ANY, wxVERTICAL,
+            wxSize{ 100, 100 }, // Ruler can't handle small sizes
+            RulerPanel::Range{ 0.0, -dBRange },
+            Ruler::LinearDBFormat,
+            _("dB"),
+            RulerPanel::Options{}
+               .LabelEdges(true)
+               .TickColour( theTheme.Colour( clrGraphLabels ) )
+         );
 
          S.AddSpace(wxDefaultCoord, 1);
          S.Prop(1);
@@ -298,7 +310,7 @@ FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
       }
       S.EndVerticalLay();
 
-      mFreqPlot = safenew FreqPlot(this);
+      mFreqPlot = safenew FreqPlot(this, wxID_ANY);
       mFreqPlot->SetMinSize(wxSize(wxDefaultCoord, FREQ_WINDOW_HEIGHT));
       S.Prop(1);
       S.AddWindow(mFreqPlot, wxEXPAND);
@@ -309,6 +321,10 @@ FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
          {
             mPanScroller = safenew wxScrollBar(this, FreqPanScrollerID,
                wxDefaultPosition, wxDefaultSize, wxSB_VERTICAL);
+#if wxUSE_ACCESSIBILITY
+            // so that name can be set on a standard control
+            mPanScroller->SetAccessible(safenew WindowAccessible(mPanScroller));
+#endif
             mPanScroller->SetName(_("Scroll"));
             S.Prop(1);
             S.AddWindow(mPanScroller, wxALIGN_LEFT | wxTOP);
@@ -322,10 +338,14 @@ FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
 
             S.AddSpace(5);
 
-            mZoomSlider = safenew wxSlider(this, FreqZoomSliderID, 100, 1, 100,
+            mZoomSlider = safenew wxSliderWrapper(this, FreqZoomSliderID, 100, 1, 100,
                wxDefaultPosition, wxDefaultSize, wxSL_VERTICAL);
             S.Prop(1);
             S.AddWindow(mZoomSlider, wxALIGN_CENTER_HORIZONTAL);
+#if wxUSE_ACCESSIBILITY
+            // so that name can be set on a standard control
+            mZoomSlider->SetAccessible(safenew WindowAccessible(mZoomSlider));
+#endif
             mZoomSlider->SetName(_("Zoom"));
 
             S.AddSpace(5);
@@ -347,19 +367,18 @@ FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
 
       S.StartHorizontalLay(wxEXPAND, 0);
       {
-         hRuler  = safenew RulerPanel(this, wxID_ANY);
-         hRuler->ruler.SetBounds(0, 0, 100, 100); // Ruler can't handle small sizes
-         hRuler->ruler.SetOrientation(wxHORIZONTAL);
-         hRuler->ruler.SetLog(true);
-         hRuler->ruler.SetRange(10, 20000);
-         hRuler->ruler.SetFormat(Ruler::RealFormat);
-         hRuler->ruler.SetUnits(_("Hz"));
-         hRuler->ruler.SetFlip(true);
-         hRuler->ruler.SetLabelEdges(true);
-         int h;
-         hRuler->ruler.GetMaxSize(NULL, &h);
-         hRuler->SetMinSize(wxSize(wxDefaultCoord, h));
-         hRuler->SetTickColour( theTheme.Colour( clrGraphLabels ));
+         hRuler  = safenew RulerPanel(
+            this, wxID_ANY, wxHORIZONTAL,
+            wxSize{ 100, 100 }, // Ruler can't handle small sizes
+            RulerPanel::Range{ 10, 20000 },
+            Ruler::RealFormat,
+            _("Hz"),
+            RulerPanel::Options{}
+               .Log(true)
+               .Flip(true)
+               .LabelEdges(true)
+               .TickColour( theTheme.Colour( clrGraphLabels ) )
+         );
 
          S.AddSpace(1, wxDefaultCoord);
          S.Prop(1);
@@ -393,16 +412,15 @@ FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
             S.AddPrompt(_("Cursor:"));
 
             S.SetStyle(wxTE_READONLY);
-            mCursorText = S.AddTextBox(wxT(""), wxT(""), 10);
+            mCursorText = S.AddTextBox( {}, wxT(""), 10);
 
             S.AddPrompt(_("Peak:"));
 
             S.SetStyle(wxTE_READONLY);
-            mPeakText = S.AddTextBox(wxT(""), wxT(""), 10);
+            mPeakText = S.AddTextBox( {}, wxT(""), 10);
             S.AddSpace(5);
 
-            mGridOnOff = S.Id(GridOnOffID).AddCheckBox(_("&Grids"), wxT("false"));
-            mGridOnOff->SetValue(mDrawGrid);
+            mGridOnOff = S.Id(GridOnOffID).AddCheckBox(_("&Grids"), mDrawGrid);
          }
          S.EndMultiColumn();
       }
@@ -428,14 +446,14 @@ FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
 
       S.AddSpace(5);
 
-      mAlgChoice = S.Id(FreqAlgChoiceID).AddChoice(_("&Algorithm:"), wxT(""), &algChoices);
-      mAlgChoice->SetSelection(mAlg);
+      mAlgChoice = S.Id(FreqAlgChoiceID)
+         .AddChoice(_("&Algorithm:"), algChoices, mAlg);
       S.SetSizeHints(wxDefaultCoord, wxDefaultCoord);
 
       S.AddSpace(5);
 
-      mSizeChoice = S.Id(FreqSizeChoiceID).AddChoice(_("&Size:"), wxT(""), &sizeChoices);
-      mSizeChoice->SetSelection(mSize);
+      mSizeChoice = S.Id(FreqSizeChoiceID)
+         .AddChoice(_("&Size:"), sizeChoices, mSize);
       S.SetSizeHints(wxDefaultCoord, wxDefaultCoord);
 
       S.AddSpace(5);
@@ -451,15 +469,15 @@ FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
 
       S.AddSpace(5);
 
-      mFuncChoice = S.Id(FreqFuncChoiceID).AddChoice(_("&Function:"), wxT(""), &funcChoices);
-      mFuncChoice->SetSelection(mFunc);
+      mFuncChoice = S.Id(FreqFuncChoiceID)
+         .AddChoice(_("&Function:"), funcChoices, mFunc);
       S.SetSizeHints(wxDefaultCoord, wxDefaultCoord);
       mFuncChoice->MoveAfterInTabOrder(mSizeChoice);
 
       S.AddSpace(5);
 
-      mAxisChoice = S.Id(FreqAxisChoiceID).AddChoice(_("&Axis:"), wxT(""), &axisChoices);
-      mAxisChoice->SetSelection(mAxis);
+      mAxisChoice = S.Id(FreqAxisChoiceID)
+         .AddChoice(_("&Axis:"), axisChoices, mAxis);
       S.SetSizeHints(wxDefaultCoord, wxDefaultCoord);
       mAxisChoice->MoveAfterInTabOrder(mFuncChoice);
 
@@ -482,7 +500,7 @@ FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
 
    S.AddSpace(5);
 
-   mProgress = safenew FreqGauge(this); //, wxID_ANY, wxST_SIZEGRIP);
+   mProgress = safenew FreqGauge(this, wxID_ANY); //, wxST_SIZEGRIP);
    S.AddWindow(mProgress, wxEXPAND);
 
    // Log-frequency axis works for spectrum plots only.
@@ -493,7 +511,7 @@ FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
    }
    mLogAxis = mAxis != 0;
 
-   mCloseButton = reinterpret_cast<wxButton*>(FindWindowById( wxID_CANCEL ));
+   mCloseButton = static_cast<wxButton*>(FindWindowById( wxID_CANCEL ));
    mCloseButton->SetDefault();
    mCloseButton->SetFocus();
 
@@ -566,46 +584,40 @@ void FreqWindow::GetAudio()
 
    int selcount = 0;
    bool warning = false;
-   TrackListIterator iter(p->GetTracks());
-   Track *t = iter.First();
-   while (t) {
-      if (t->GetSelected() && t->GetKind() == Track::Wave) {
-         WaveTrack *track = (WaveTrack *)t;
-         if (selcount==0) {
-            mRate = track->GetRate();
-            auto start = track->TimeToLongSamples(p->mViewInfo.selectedRegion.t0());
-            auto end = track->TimeToLongSamples(p->mViewInfo.selectedRegion.t1());
-            auto dataLen = end - start;
-            if (dataLen > 10485760) {
-               warning = true;
-               mDataLen = 10485760;
-            }
-            else
-               // dataLen is not more than 10 * 2 ^ 20
-               mDataLen = dataLen.as_size_t();
-            mData = Floats{ mDataLen };
-            // Don't allow throw for bad reads
-            track->Get((samplePtr)mData.get(), floatSample, start, mDataLen,
-                       fillZero, false);
+   for (auto track : p->GetTracks()->Selected< const WaveTrack >()) {
+      if (selcount==0) {
+         mRate = track->GetRate();
+         auto start = track->TimeToLongSamples(p->mViewInfo.selectedRegion.t0());
+         auto end = track->TimeToLongSamples(p->mViewInfo.selectedRegion.t1());
+         auto dataLen = end - start;
+         if (dataLen > 10485760) {
+            warning = true;
+            mDataLen = 10485760;
          }
-         else {
-            if (track->GetRate() != mRate) {
-               wxMessageBox(_("To plot the spectrum, all selected tracks must be the same sample rate."));
-               mData.reset();
-               mDataLen = 0;
-               return;
-            }
-            auto start = track->TimeToLongSamples(p->mViewInfo.selectedRegion.t0());
-            Floats buffer2{ mDataLen };
-            // Again, stop exceptions
-            track->Get((samplePtr)buffer2.get(), floatSample, start, mDataLen,
-                       fillZero, false);
-            for (size_t i = 0; i < mDataLen; i++)
-               mData[i] += buffer2[i];
-         }
-         selcount++;
+         else
+            // dataLen is not more than 10 * 2 ^ 20
+            mDataLen = dataLen.as_size_t();
+         mData = Floats{ mDataLen };
+         // Don't allow throw for bad reads
+         track->Get((samplePtr)mData.get(), floatSample, start, mDataLen,
+                    fillZero, false);
       }
-      t = iter.Next();
+      else {
+         if (track->GetRate() != mRate) {
+            AudacityMessageBox(_("To plot the spectrum, all selected tracks must be the same sample rate."));
+            mData.reset();
+            mDataLen = 0;
+            return;
+         }
+         auto start = track->TimeToLongSamples(p->mViewInfo.selectedRegion.t0());
+         Floats buffer2{ mDataLen };
+         // Again, stop exceptions
+         track->Get((samplePtr)buffer2.get(), floatSample, start, mDataLen,
+                    fillZero, false);
+         for (size_t i = 0; i < mDataLen; i++)
+            mData[i] += buffer2[i];
+      }
+      selcount++;
    }
 
    if (selcount == 0)
@@ -613,9 +625,9 @@ void FreqWindow::GetAudio()
 
    if (warning) {
       wxString msg;
-      msg.Printf(_("Too much audio was selected.  Only the first %.1f seconds of audio will be analyzed."),
+      msg.Printf(_("Too much audio was selected. Only the first %.1f seconds of audio will be analyzed."),
                           (mDataLen / mRate));
-      wxMessageBox(msg);
+      AudacityMessageBox(msg);
    }
 }
 
@@ -636,7 +648,7 @@ void FreqWindow::DrawBackground(wxMemoryDC & dc)
 
    mPlotRect = mFreqPlot->GetClientRect();
 
-   mBitmap = std::make_unique<wxBitmap>(mPlotRect.width, mPlotRect.height);
+   mBitmap = std::make_unique<wxBitmap>(mPlotRect.width, mPlotRect.height,24);
 
    dc.SelectObject(*mBitmap);
 
@@ -753,9 +765,9 @@ void FreqWindow::DrawPlot()
 
    // Draw the plot
    if (mAlg == SpectrumAnalyst::Spectrum)
-      memDC.SetPen(wxPen(theTheme.Colour( clrHzPlot ), 1, wxSOLID));
+      memDC.SetPen(wxPen(theTheme.Colour( clrHzPlot ), 1, wxPENSTYLE_SOLID));
    else
-      memDC.SetPen(wxPen(theTheme.Colour( clrWavelengthPlot), 1, wxSOLID));
+      memDC.SetPen(wxPen(theTheme.Colour( clrWavelengthPlot), 1, wxPENSTYLE_SOLID));
 
    float xPos = xMin;
 
@@ -912,7 +924,7 @@ void FreqWindow::PlotPaint(wxPaintEvent & event)
       else
          px = (int)((bestpeak - xMin) * width / (xMax - xMin));
 
-      dc.SetPen(wxPen(wxColour(160,160,160), 1, wxSOLID));
+      dc.SetPen(wxPen(wxColour(160,160,160), 1, wxPENSTYLE_SOLID));
       AColor::Line(dc, r.x + 1 + px, r.y, r.x + 1 + px, r.y + r.height);
 
        // print out info about the cursor location
@@ -937,16 +949,16 @@ void FreqWindow::PlotPaint(wxPaintEvent & event)
       if (mAlg == SpectrumAnalyst::Spectrum) {
          xpitch = PitchName_Absolute(FreqToMIDInote(xPos));
          peakpitch = PitchName_Absolute(FreqToMIDInote(bestpeak));
-         xp = xpitch.c_str();
-         pp = peakpitch.c_str();
+         xp = xpitch;
+         pp = peakpitch;
          /* i18n-hint: The %d's are replaced by numbers, the %s by musical notes, e.g. A#*/
          cursor.Printf(_("%d Hz (%s) = %d dB"), (int)(xPos + 0.5), xp, (int)(value + 0.5));
          peak.Printf(_("%d Hz (%s) = %.1f dB"), (int)(bestpeak + 0.5), pp, bestValue);
       } else if (xPos > 0.0 && bestpeak > 0.0) {
          xpitch = PitchName_Absolute(FreqToMIDInote(1.0 / xPos));
          peakpitch = PitchName_Absolute(FreqToMIDInote(1.0 / bestpeak));
-         xp = xpitch.c_str();
-         pp = peakpitch.c_str();
+         xp = xpitch;
+         pp = peakpitch;
          /* i18n-hint: The %d's are replaced by numbers, the %s by musical notes, e.g. A#
           * the %.4f are numbers, and 'sec' should be an abbreviation for seconds */
          cursor.Printf(_("%.4f sec (%d Hz) (%s) = %f"),
@@ -1044,7 +1056,7 @@ void FreqWindow::OnExport(wxCommandEvent & WXUNUSED(event))
       _("Export Spectral Data As:"),
       wxEmptyString, fName, wxT("txt"), wxT("*.txt"), wxFD_SAVE | wxRESIZE_BORDER, this);
 
-   if (fName == wxT(""))
+   if (fName.empty())
       return;
 
    wxTextFile f(fName);
@@ -1055,7 +1067,8 @@ void FreqWindow::OnExport(wxCommandEvent & WXUNUSED(event))
 #endif
    f.Open();
    if (!f.IsOpened()) {
-      wxMessageBox(_("Couldn't write to file: ") + fName);
+      AudacityMessageBox( wxString::Format(
+         _("Couldn't write to file: %s"), fName ) );
       return;
    }
 
@@ -1109,8 +1122,8 @@ BEGIN_EVENT_TABLE(FreqPlot, wxWindow)
    EVT_MOUSE_EVENTS(FreqPlot::OnMouseEvent)
 END_EVENT_TABLE()
 
-FreqPlot::FreqPlot(wxWindow *parent)
-:  wxWindow(parent, wxID_ANY)
+FreqPlot::FreqPlot(wxWindow *parent, wxWindowID winid)
+:  wxWindow(parent, winid)
 {
    freqWindow = (FreqWindow *) parent;
 }
@@ -1135,8 +1148,8 @@ void FreqPlot::OnMouseEvent(wxMouseEvent & event)
    freqWindow->PlotMouseEvent(event);
 }
 
-FreqGauge::FreqGauge(wxWindow * parent)
-:  wxStatusBar(parent, wxID_ANY, wxST_SIZEGRIP)
+FreqGauge::FreqGauge(wxWindow * parent, wxWindowID winid)
+:  wxStatusBar(parent, winid, wxST_SIZEGRIP)
 {
    mRange = 0;
 }
