@@ -9,16 +9,12 @@
 #include "Audacity.h"
 #include "InconsistencyException.h"
 
+#include "Internat.h"
+
 InconsistencyException::~InconsistencyException()
 {
 }
 
-std::unique_ptr< AudacityException > InconsistencyException::Move()
-{
-   return std::unique_ptr< AudacityException >
-   { safenew InconsistencyException{ std::move( *this ) } };
-}
-
 wxString InconsistencyException::ErrorMessage() const
 {
    // Shorten the path
