@@ -31,26 +31,23 @@
 
 
 #include "Audacity.h"
+#include "Languages.h"
 
 #include <wx/defs.h>
-#include <wx/hashmap.h>
 #include <wx/intl.h>
 #include <wx/textfile.h>
 
-#include "Languages.h"
 #include "FileNames.h"
 
 #include "AudacityApp.h"
 
-#ifndef __AUDACITY_OLD_STD__
 #include <unordered_map>
-#endif
 
 using LangHash = std::unordered_map<wxString, wxString>;
 
-static bool TranslationExists(wxArrayString &audacityPathList, wxString code)
+static bool TranslationExists(const FilePaths &audacityPathList, wxString code)
 {
-   wxArrayString results;
+   FilePaths results;
    wxGetApp().FindFilesInPathList(wxString::Format(wxT("%s/audacity.mo"),
                                                    code),
                                   audacityPathList,
@@ -67,12 +64,12 @@ static bool TranslationExists(wxArrayString &audacityPathList, wxString code)
                                   audacityPathList,
                                   results);
 
-   return (results.GetCount() > 0);
+   return (results.size() > 0);
 }
 
 #ifdef __WXMAC__
 #include <CoreFoundation/CFLocale.h>
-#include "wx/osx/core/cfstring.h"
+#include <wx/osx/core/cfstring.h>
 #endif
 
 wxString GetSystemLanguageCode()
@@ -111,13 +108,13 @@ wxString GetSystemLanguageCode()
 
    if (info) {
       wxString fullCode = info->CanonicalName;
-      if (fullCode.Length() < 2)
+      if (fullCode.length() < 2)
          return wxT("en");
 
       wxString code = fullCode.Left(2);
       unsigned int i;
 
-      for(i=0; i<langCodes.GetCount(); i++) {
+      for(i=0; i<langCodes.size(); i++) {
          if (langCodes[i] == fullCode)
             return fullCode;
 
@@ -204,7 +201,7 @@ void GetLanguages(wxArrayString &langCodes, wxArrayString &langNames)
       localLanguageName[code] = name;
    }
 
-   wxArrayString audacityPathList = wxGetApp().audacityPathList;
+   auto audacityPathList = wxGetApp().audacityPathList;
 
 #if defined(__WXGTK__)
    wxGetApp().AddUniquePathToPathList(wxString::Format(wxT("%s/share/locale"),
@@ -240,13 +237,13 @@ void GetLanguages(wxArrayString &langCodes, wxArrayString &langNames)
       // name but associate it with the full code.  This allows someone
       // to drop in a NEW language and still get reasonable behavior.
 
-      if (fullCode.Length() < 2)
+      if (fullCode.length() < 2)
          continue;
 
-      if (localLanguageName[code] != wxT("")) {
+      if (!localLanguageName[code].empty()) {
          name = localLanguageName[code];
       }
-      if (localLanguageName[fullCode] != wxT("")) {
+      if (!localLanguageName[fullCode].empty()) {
          name = localLanguageName[fullCode];
       }
 
@@ -254,12 +251,12 @@ void GetLanguages(wxArrayString &langCodes, wxArrayString &langNames)
          code = fullCode;
       }
 
-      if (tempHash[code] != wxT(""))
+      if (!tempHash[code].empty())
          continue;
 
       if (TranslationExists(audacityPathList, code) || code==wxT("en")) {
-         tempCodes.Add(code);
-         tempNames.Add(name);
+         tempCodes.push_back(code);
+         tempNames.push_back(name);
          tempHash[code] = name;
 
 /*         wxLogDebug(wxT("code=%s name=%s fullCode=%s name=%s -> %s"),
@@ -276,8 +273,8 @@ void GetLanguages(wxArrayString &langCodes, wxArrayString &langNames)
       code = wxT("en-simple");
       name = wxT("Simplified");
       if (TranslationExists(audacityPathList, code) ) {
-         tempCodes.Add(code);
-         tempNames.Add(name);
+         tempCodes.push_back(code);
+         tempNames.push_back(name);
          tempHash[code] = name;
       }
    }
@@ -285,18 +282,18 @@ void GetLanguages(wxArrayString &langCodes, wxArrayString &langNames)
 
    // Sort
    unsigned int j;
-   for(j=0; j<tempNames.GetCount(); j++){
+   for(j=0; j<tempNames.size(); j++){
       reverseHash[tempNames[j]] = tempCodes[j];
    }
 
-   tempNames.Sort();
+   std::sort( tempNames.begin(), tempNames.end() );
 
    // Add system language
-   langNames.Add(wxT("System"));
-   langCodes.Add(wxT(""));
+   langNames.push_back(wxT("System"));
+   langCodes.push_back(wxT(""));
 
-   for(j=0; j<tempNames.GetCount(); j++) {
-      langNames.Add(tempNames[j]);
-      langCodes.Add(reverseHash[tempNames[j]]);
+   for(j=0; j<tempNames.size(); j++) {
+      langNames.push_back(tempNames[j]);
+      langCodes.push_back(reverseHash[tempNames[j]]);
    }
 }
