@@ -9,32 +9,42 @@
 
 **********************************************************************/
 
-#include "Audacity.h"
+#include "Audacity.h" // for USE_* macros
+#include "MixerBoard.h"
+
+#include "Experimental.h"
+
+#include "Menus.h"
 
 #include <cfloat>
 #include <math.h>
 
+#include <wx/setup.h> // for wxUSE_* macros
+
+#include <wx/bmpbuttn.h>
+#include <wx/dcclient.h>
 #include <wx/dcmemory.h>
-#include <wx/arrimpl.cpp>
 #include <wx/icon.h>
 #include <wx/settings.h> // for wxSystemSettings::GetColour and wxSystemSettings::GetMetric
 
 #include "Theme.h"
-#include "Experimental.h"
-#include "MixerBoard.h"
 #include "AColor.h"
 #include "AllThemeResources.h"
 #include "AudioIO.h"
 
+#include "Menus.h"
+
 #ifdef USE_MIDI
 #include "NoteTrack.h"
 #endif
 
+#include "Prefs.h" // for RTL_WORKAROUND
 #include "Project.h"
 #include "TrackPanel.h" // for EVT_TRACK_PANEL_TIMER
 #include "UndoManager.h"
 #include "WaveTrack.h"
 
+#include "widgets/AButton.h"
 #include "widgets/Meter.h"
 
 
@@ -45,6 +55,7 @@
    #include "../images/AudacityLogo48x48.xpm"
 #endif
 
+#include "commands/CommandManager.h"
 
 // class MixerTrackSlider
 
@@ -62,13 +73,8 @@ MixerTrackSlider::MixerTrackSlider(wxWindow * parent,
                                     const wxString &name,
                                     const wxPoint & pos,
                                     const wxSize & size,
-                                    int style /*= FRAC_SLIDER*/,
-                                    bool popup /*= true*/,
-                                    bool canUseShift /*= true*/,
-                                    float stepValue /*= STEP_CONTINUOUS*/,
-                                    int orientation /*= wxHORIZONTAL*/)
-: ASlider(parent, id, name, pos, size,
-            style, popup, canUseShift, stepValue, orientation)
+                                    const ASlider::Options &options)
+: ASlider(parent, id, name, pos, size, options)
 {
 }
 
@@ -123,19 +129,19 @@ void MixerTrackSlider::OnCaptureKey(wxCommandEvent &event)
 
 // class MixerTrackCluster
 
-#define kInset             4
-#define kDoubleInset       (2 * kInset)
-#define kTripleInset       (3 * kInset)
-#define kQuadrupleInset    (4 * kInset)
+const int kInset = 4;
+const int kDoubleInset    = (2 * kInset);
+const int kTripleInset    = (3 * kInset);
+const int kQuadrupleInset = (4 * kInset);
 
-#define TRACK_NAME_HEIGHT                    18
-#define MUSICAL_INSTRUMENT_HEIGHT_AND_WIDTH  48
-#define MUTE_SOLO_HEIGHT                     19
-#define PAN_HEIGHT                           24
+const int TRACK_NAME_HEIGHT                   = 18;
+const int MUSICAL_INSTRUMENT_HEIGHT_AND_WIDTH = 48;
+const int MUTE_SOLO_HEIGHT                    = 19;
+const int  PAN_HEIGHT                         = 24;
 
-#define kLeftSideStackWidth         MUSICAL_INSTRUMENT_HEIGHT_AND_WIDTH - kDoubleInset //vvv Change when numbers shown on slider scale.
-#define kRightSideStackWidth        MUSICAL_INSTRUMENT_HEIGHT_AND_WIDTH + kDoubleInset
-#define kMixerTrackClusterWidth     kLeftSideStackWidth + kRightSideStackWidth + kQuadrupleInset // kDoubleInset margin on both sides
+const int kLeftSideStackWidth     = MUSICAL_INSTRUMENT_HEIGHT_AND_WIDTH - kDoubleInset; //vvv Change when numbers shown on slider scale.
+const int kRightSideStackWidth    = MUSICAL_INSTRUMENT_HEIGHT_AND_WIDTH + kDoubleInset;
+const int kMixerTrackClusterWidth = kLeftSideStackWidth + kRightSideStackWidth + kQuadrupleInset; // kDoubleInset margin on both sides
 
 enum {
    ID_BITMAPBUTTON_MUSICAL_INSTRUMENT = 13000,
@@ -209,20 +215,23 @@ MixerTrackCluster::MixerTrackCluster(wxWindow* parent,
             this, ID_SLIDER_GAIN,
             /* i18n-hint: title of the Gain slider, used to adjust the volume */
             _("Gain"),
-            ctrlPos, ctrlSize, DB_SLIDER, true,
-            true, 0.0, wxVERTICAL);
+            ctrlPos, ctrlSize,
+            ASlider::Options{}
+               .Style( DB_SLIDER )
+               .Orientation( wxVERTICAL ));
    mSlider_Gain->SetName(_("Gain"));
-   this->UpdateGain();
+
 #ifdef EXPERIMENTAL_MIDI_OUT
    mSlider_Velocity =
       safenew MixerTrackSlider(
             this, ID_SLIDER_VELOCITY,
             /* i18n-hint: title of the MIDI Velocity slider */
             _("Velocity"),
-            ctrlPos, ctrlSize, VEL_SLIDER, true,
-            true, 0.0, wxVERTICAL);
+            ctrlPos, ctrlSize,
+            ASlider::Options{}
+               .Style( VEL_SLIDER )
+               .Orientation( wxVERTICAL ));
    mSlider_Velocity->SetName(_("Velocity"));
-   this->UpdateVelocity();
 #endif
 
    // other controls and meter at right
@@ -254,11 +263,10 @@ MixerTrackCluster::MixerTrackCluster(wxWindow* parent,
             this, ID_SLIDER_PAN,
             /* i18n-hint: Title of the Pan slider, used to move the sound left or right */
             _("Pan"),
-            ctrlPos, ctrlSize, PAN_SLIDER, true);
+            ctrlPos, ctrlSize,
+            ASlider::Options{}.Style( PAN_SLIDER ));
    mSlider_Pan->SetName(_("Pan"));
 
-   this->UpdatePan();
-
    // mute/solo buttons stacked below Pan slider
    ctrlPos.y += PAN_HEIGHT + kDoubleInset;
    ctrlSize.Set(mMixerBoard->mMuteSoloWidth, MUTE_SOLO_HEIGHT);
@@ -275,7 +283,6 @@ MixerTrackCluster::MixerTrackCluster(wxWindow* parent,
       *(mMixerBoard->mImageMuteUp), *(mMixerBoard->mImageMuteOver),
       *(mMixerBoard->mImageMuteDown), *(mMixerBoard->mImageMuteDown), 
       *(mMixerBoard->mImageMuteDisabled));
-   this->UpdateMute();
 
    ctrlPos.y += MUTE_SOLO_HEIGHT;
    mToggleButton_Solo =
@@ -286,7 +293,6 @@ MixerTrackCluster::MixerTrackCluster(wxWindow* parent,
                   *(mMixerBoard->mImageSoloDisabled),
                   true); // toggle button
    mToggleButton_Solo->SetName(_("Solo"));
-   this->UpdateSolo();
    bool bSoloNone = mProject->IsSoloNone();
    mToggleButton_Solo->Show(!bSoloNone);
 
@@ -303,11 +309,11 @@ MixerTrackCluster::MixerTrackCluster(wxWindow* parent,
    mMeter = NULL;
    if (GetWave()) {
       mMeter =
-         safenew Meter(GetActiveProject(), // AudacityProject* project,
+         safenew MeterPanel(GetActiveProject(), // AudacityProject* project,
                    this, -1, // wxWindow* parent, wxWindowID id,
                    false, // bool isInput
                    ctrlPos, ctrlSize, // const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize,
-                   Meter::MixerTrackCluster); // Style style = HorizontalStereo,
+                   MeterPanel::MixerTrackCluster); // Style style = HorizontalStereo,
       mMeter->SetName(_("Signal Level Meter"));
    }
 
@@ -319,11 +325,12 @@ MixerTrackCluster::MixerTrackCluster(wxWindow* parent,
          mMeter->SetToolTip(_("Signal Level Meter"));
    #endif // wxUSE_TOOLTIPS
 
+   UpdateForStateChange();
+
    #ifdef __WXMAC__
       wxSizeEvent event(GetSize(), GetId());
       event.SetEventObject(this);
       GetEventHandler()->ProcessEvent(event);
-      UpdateGain();
    #endif
 }
 
@@ -334,11 +341,14 @@ WaveTrack *MixerTrackCluster::GetWave() const
 
 WaveTrack *MixerTrackCluster::GetRight() const
 {
+  // TODO: more-than-two-channels
    auto left = GetWave();
-   if (left)
-      return static_cast<WaveTrack*>(left->GetLink());
-   else
-      return nullptr;
+   if (left) {
+      auto channels = TrackList::Channels(left);
+      if ( channels.size() > 1 )
+         return * ++ channels.first;
+   }
+   return nullptr;
 }
 
 #ifdef EXPERIMENTAL_MIDI_OUT
@@ -348,6 +358,10 @@ NoteTrack *MixerTrackCluster::GetNote() const
 }
 #endif
 
+// Old approach modified things in situ.
+// However with a theme change there is so much to modify, it is easier
+// to recreate.
+#if 0
 void MixerTrackCluster::UpdatePrefs()
 {
    this->SetBackgroundColour( theTheme.Colour( clrMedium ) );
@@ -356,6 +370,7 @@ void MixerTrackCluster::UpdatePrefs()
       mMeter->UpdatePrefs(); // in case meter range has changed
    HandleResize(); // in case prefs "/GUI/Solo" changed
 }
+#endif
 
 void MixerTrackCluster::HandleResize() // For wxSizeEvents, update gain slider and meter.
 {
@@ -405,9 +420,9 @@ void MixerTrackCluster::HandleSliderGain(const bool bWantPushState /*= false*/)
       GetRight()->SetGain(fValue);
 
    // Update the TrackPanel correspondingly.
-   mProject->RefreshTPTrack(mTrack.get());
+   mProject->GetTrackPanel()->RefreshTrack(mTrack.get());
    if (bWantPushState)
-      mProject->TP_PushState(_("Moved gain slider"), _("Gain"), UndoPush::CONSOLIDATE );
+      mProject->PushState(_("Moved gain slider"), _("Gain"), UndoPush::CONSOLIDATE );
 }
 
 #ifdef EXPERIMENTAL_MIDI_OUT
@@ -418,9 +433,9 @@ void MixerTrackCluster::HandleSliderVelocity(const bool bWantPushState /*= false
       GetNote()->SetVelocity(fValue);
 
    // Update the TrackPanel correspondingly.
-   mProject->RefreshTPTrack(mTrack.get());
+   mProject->GetTrackPanel()->RefreshTrack(mTrack.get());
    if (bWantPushState)
-      mProject->TP_PushState(_("Moved velocity slider"), _("Velocity"), UndoPush::CONSOLIDATE);
+      mProject->PushState(_("Moved velocity slider"), _("Velocity"), UndoPush::CONSOLIDATE);
 }
 #endif
 
@@ -433,10 +448,10 @@ void MixerTrackCluster::HandleSliderPan(const bool bWantPushState /*= false*/)
       GetRight()->SetPan(fValue);
 
    // Update the TrackPanel correspondingly.
-   mProject->RefreshTPTrack(mTrack.get());
+   mProject->GetTrackPanel()->RefreshTrack(mTrack.get());
 
    if (bWantPushState)
-      mProject->TP_PushState(_("Moved pan slider"), _("Pan"), UndoPush::CONSOLIDATE );
+      mProject->PushState(_("Moved pan slider"), _("Pan"), UndoPush::CONSOLIDATE );
 }
 
 void MixerTrackCluster::ResetMeter(const bool bResetClipping)
@@ -446,76 +461,51 @@ void MixerTrackCluster::ResetMeter(const bool bResetClipping)
 }
 
 
-// These are used by TrackPanel for synchronizing control states, etc.
-
-// Update the controls that can be affected by state change.
+// Update appearance to match the state of the track
 void MixerTrackCluster::UpdateForStateChange()
-{
-   this->UpdateName();
-   this->UpdatePan();
-   this->UpdateGain();
-}
-
-void MixerTrackCluster::UpdateName()
 {
    const wxString newName = mTrack->GetName();
-   SetName(newName);
-   mStaticText_TrackName->SetLabel(newName);
-   mStaticText_TrackName->SetName(newName);
-   #if wxUSE_TOOLTIPS
-      mStaticText_TrackName->SetToolTip(newName);
-   #endif
-   mBitmapButton_MusicalInstrument->SetBitmapLabel(
-      *(mMixerBoard->GetMusicalInstrumentBitmap(mTrack.get())));
-}
+   if (newName != GetName()) {
+      SetName(newName);
+      mStaticText_TrackName->SetLabel(newName);
+      mStaticText_TrackName->SetName(newName);
+#if wxUSE_TOOLTIPS
+         mStaticText_TrackName->SetToolTip(newName);
+#endif
+      mBitmapButton_MusicalInstrument->SetBitmapLabel(
+         *(mMixerBoard->GetMusicalInstrumentBitmap(mTrack.get())));
+   }
 
-void MixerTrackCluster::UpdateMute()
-{
    mToggleButton_Mute->SetAlternateIdx(mTrack->GetSolo() ? 1 : 0);
    if (mTrack->GetMute())
       mToggleButton_Mute->PushDown();
    else
       mToggleButton_Mute->PopUp();
-}
 
-void MixerTrackCluster::UpdateSolo()
-{
    bool bIsSolo = mTrack->GetSolo();
    if (bIsSolo)
       mToggleButton_Solo->PushDown();
    else
       mToggleButton_Solo->PopUp();
    mToggleButton_Mute->SetAlternateIdx(bIsSolo ? 1 : 0);
-}
 
-void MixerTrackCluster::UpdatePan()
-{
-   if (!GetWave()) {
+   if (!GetWave())
       mSlider_Pan->Hide();
-      return;
-   }
-   mSlider_Pan->Set(GetWave()->GetPan());
-}
+   else
+      mSlider_Pan->Set(GetWave()->GetPan());
 
-void MixerTrackCluster::UpdateGain()
-{
-   if (!GetWave()) {
+   if (!GetWave())
       mSlider_Gain->Hide();
-      return;
-   }
-   mSlider_Gain->Set(GetWave()->GetGain());
-}
+   else
+      mSlider_Gain->Set(GetWave()->GetGain());
 
 #ifdef EXPERIMENTAL_MIDI_OUT
-void MixerTrackCluster::UpdateVelocity()
-{
-   if (!GetNote()) {
+   if (!GetNote())
       mSlider_Velocity->Hide();
-      return;
-   }
-   mSlider_Velocity->Set(GetNote()->GetVelocity());
-}
+   else
+      mSlider_Velocity->Set(GetNote()->GetVelocity());
 #endif
+}
 
 void MixerTrackCluster::UpdateMeter(const double t0, const double t1)
 {
@@ -540,12 +530,12 @@ void MixerTrackCluster::UpdateMeter(const double t0, const double t1)
 
    // Vaughan, 2010-11-27:
    // This commented out code is flawed. Mistaken understanding of "frame" vs "window".
-   // Caused me to override Meter::UpdateDisplay().
+   // Caused me to override MeterPanel::UpdateDisplay().
    // But I think it's got a good idea, of calling WaveTracks' GetMinMax and GetRMS
    // instead of passing in all the data and asking the meter to derive peak and rms.
    // May be worth revisiting as I think it should perform better, because it uses the min/max/rms
    // stored in blockfiles, rather than calculating them, but for now, changing it to use the
-   // original Meter::UpdateDisplay(). New code is below the previous (now commented out).
+   // original MeterPanel::UpdateDisplay(). New code is below the previous (now commented out).
    //
    //const size_t kFramesPerBuffer = 4;
    //float min; // dummy, since it's not shown in meters
@@ -600,7 +590,7 @@ void MixerTrackCluster::UpdateMeter(const double t0, const double t1)
    //      rmsRight[i] *= gain;
    //   }
    //   if ( mMeter ) mMeter->UpdateDisplay(
-   //      2, // If mono, show left track values in both meters, as in MeterToolBar, rather than kNumChannels.
+   //      2, // If mono, show left track values in both meters, as in MeterToolBar, rather than nChannels.
    //      kFramesPerBuffer,
    //      maxLeft, rmsLeft,
    //      maxRight, rmsRight,
@@ -666,7 +656,8 @@ void MixerTrackCluster::UpdateMeter(const double t0, const double t1)
          else if (meterFloatsArray[index] > 1.0)
             meterFloatsArray[index] = 1.0;
 
-      mMeter->UpdateDisplay(2, nFrames, meterFloatsArray.get());
+      if (mMeter)
+         mMeter->UpdateDisplay(2, nFrames, meterFloatsArray.get());
    }
    else
       this->ResetMeter(false);
@@ -684,8 +675,8 @@ wxColour MixerTrackCluster::GetTrackColor()
 
 void MixerTrackCluster::HandleSelect(bool bShiftDown, bool bControlDown)
 {
-   mProject->HandleListSelection( mTrack.get(), bShiftDown, bControlDown, true
-                                 );
+   SelectActions::DoListSelection(*mProject,
+      mTrack.get(), bShiftDown, bControlDown, true);
 }
 
 void MixerTrackCluster::OnMouseEvent(wxMouseEvent& event)
@@ -698,24 +689,28 @@ void MixerTrackCluster::OnMouseEvent(wxMouseEvent& event)
 
 void MixerTrackCluster::OnPaint(wxPaintEvent & WXUNUSED(event))
 {
+   UpdateForStateChange();
+
+   auto selected = mTrack->GetSelected();
+
+   wxColour col = theTheme.Colour(selected ? clrTrackInfoSelected : clrTrackInfo) ;
+   SetBackgroundColour( col );
+   if (mMeter)
+      mMeter->SetBackgroundColour( col );
+   mStaticText_TrackName->SetBackgroundColour( col );
+   mSlider_Gain->SetBackgroundColour( col );
+   mSlider_Pan->SetBackgroundColour( col );
+
    wxPaintDC dc(this);
 
-   #ifdef __WXMAC__
-      // Fill with correct color, not scroller background. Done automatically on Windows.
-      AColor::Medium(&dc, false);
-      dc.DrawRectangle(this->GetClientRect());
-   #endif
+   AColor::MediumTrackInfo(&dc, selected);
+   dc.DrawRectangle(this->GetClientRect());
 
    wxSize clusterSize = this->GetSize();
    wxRect bev(0, 0, clusterSize.GetWidth() - 1, clusterSize.GetHeight() - 1);
 
-   auto selected = mTrack->GetSelected();
-
-   for (unsigned int i = 0; i < 4; i++) // 4 gives a big bevel, but there were complaints about visibility otherwise.
-   {
-      bev.Inflate(-1, -1);
-      AColor::Bevel(dc, !selected, bev);
-   }
+   //bev.Inflate(-1, -1);
+   AColor::Bevel(dc, true, bev);// same bevel whether selected or not.
 }
 
 
@@ -759,34 +754,28 @@ void MixerTrackCluster::OnSlider_Pan(wxCommandEvent& WXUNUSED(event))
 
 void MixerTrackCluster::OnButton_Mute(wxCommandEvent& WXUNUSED(event))
 {
-   mProject->HandleTrackMute(mTrack.get(), mToggleButton_Mute->WasShiftDown());
+   TrackActions::DoTrackMute(
+      *mProject, mTrack.get(), mToggleButton_Mute->WasShiftDown());
    mToggleButton_Mute->SetAlternateIdx(mTrack->GetSolo() ? 1 : 0);
 
    // Update the TrackPanel correspondingly.
    if (mProject->IsSoloSimple())
    {
-      // Have to refresh all tracks.
-      mMixerBoard->UpdateSolo();
       mProject->RedrawProject();
    }
    else
       // Update only the changed track.
-      mProject->RefreshTPTrack(mTrack.get());
+      mProject->GetTrackPanel()->RefreshTrack(mTrack.get());
 }
 
 void MixerTrackCluster::OnButton_Solo(wxCommandEvent& WXUNUSED(event))
 {
-   mProject->HandleTrackSolo(mTrack.get(), mToggleButton_Solo->WasShiftDown());
+   TrackActions::DoTrackSolo(
+      *mProject, mTrack.get(), mToggleButton_Solo->WasShiftDown());
    bool bIsSolo = mTrack->GetSolo();
    mToggleButton_Mute->SetAlternateIdx(bIsSolo ? 1 : 0);
 
    // Update the TrackPanel correspondingly.
-   if (mProject->IsSoloSimple())
-   {
-      // Have to refresh all tracks.
-      mMixerBoard->UpdateMute();
-      mMixerBoard->UpdateSolo();
-   }
    // Bug 509: Must repaint all, as many tracks can change with one Solo change.
    mProject->RedrawProject();
 }
@@ -805,16 +794,16 @@ MusicalInstrument::MusicalInstrument(std::unique_ptr<wxBitmap> &&pBitmap, const
    while ((nUnderscoreIndex = strFilename.Find(wxT('_'))) != -1)
    {
       strKeyword = strFilename.Left(nUnderscoreIndex);
-      mKeywords.Add(strKeyword);
+      mKeywords.push_back(strKeyword);
       strFilename = strFilename.Mid(nUnderscoreIndex + 1);
    }
-   if (!strFilename.IsEmpty()) // Skip trailing underscores.
-      mKeywords.Add(strFilename); // Add the last one.
+   if (!strFilename.empty()) // Skip trailing underscores.
+      mKeywords.push_back(strFilename); // Add the last one.
 }
 
 MusicalInstrument::~MusicalInstrument()
 {
-   mKeywords.Clear();
+   mKeywords.clear();
 }
 
 
@@ -862,10 +851,11 @@ void MixerBoardScrolledWindow::OnMouseEvent(wxMouseEvent& event)
 #define MIXER_BOARD_MIN_HEIGHT      460
 
 // Min width is one cluster wide, plus margins.
-#define MIXER_BOARD_MIN_WIDTH       kTripleInset + kMixerTrackClusterWidth + kTripleInset
+#define MIXER_BOARD_MIN_WIDTH       kTripleInset + kMixerTrackClusterWidth*2 + kTripleInset
 
 
 BEGIN_EVENT_TABLE(MixerBoard, wxWindow)
+   EVT_PAINT(MixerBoard::OnPaint)
    EVT_SIZE(MixerBoard::OnSize)
 END_EVENT_TABLE()
 
@@ -911,13 +901,14 @@ MixerBoard::MixerBoard(AudacityProject* pProject,
 //      mScrolledWindow->SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DSHADOW));
 //   #endif
    mScrolledWindow->SetBackgroundColour( theTheme.Colour( clrMedium ) );
+   RTL_WORKAROUND(mScrolledWindow);
 
    mScrolledWindow->SetScrollRate(10, 0); // no vertical scroll
    mScrolledWindow->SetVirtualSize(size);
 
    /* This doesn't work to make the mScrolledWindow automatically resize, so do it explicitly in OnSize.
          auto pBoxSizer = std::make_unique<wxBoxSizer>(wxVERTICAL);
-         pBoxSizer->Add(mScrolledWindow, 0, wxExpand, 0);
+         pBoxSizer->push_back(mScrolledWindow, 0, wxExpand, 0);
          this->SetAutoLayout(true);
          this->SetSizer(pBoxSizer);
          pBoxSizer->Fit(this);
@@ -928,19 +919,32 @@ MixerBoard::MixerBoard(AudacityProject* pProject,
    mTracks = mProject->GetTracks();
 
    // Events from the project don't propagate directly to this other frame, so...
-   mProject->Connect(EVT_TRACK_PANEL_TIMER,
-      wxCommandEventHandler(MixerBoard::OnTimer),
-      NULL,
+   mProject->Bind(EVT_TRACK_PANEL_TIMER,
+      &MixerBoard::OnTimer,
       this);
-}
 
-MixerBoard::~MixerBoard()
-{
-   // private data members
-   mMusicalInstruments.clear();
+   mProject->GetTracks()->Bind(EVT_TRACKLIST_SELECTION_CHANGE,
+      &MixerBoard::OnTrackChanged,
+      this);
 
-   mProject->Disconnect(EVT_TRACK_PANEL_TIMER,
-      wxCommandEventHandler(MixerBoard::OnTimer),
+   mProject->GetTracks()->Bind(EVT_TRACKLIST_PERMUTED,
+      &MixerBoard::OnTrackSetChanged,
+      this);
+
+   mProject->GetTracks()->Bind(EVT_TRACKLIST_ADDITION,
+      &MixerBoard::OnTrackSetChanged,
+      this);
+
+   mProject->GetTracks()->Bind(EVT_TRACKLIST_DELETION,
+      &MixerBoard::OnTrackSetChanged,
+      this);
+
+   mProject->GetTracks()->Bind(EVT_TRACKLIST_TRACK_DATA_CHANGE,
+      &MixerBoard::OnTrackChanged,
+      this);
+
+   wxTheApp->Connect(EVT_AUDIOIO_PLAYBACK,
+      wxCommandEventHandler(MixerBoard::OnStartStop),
       NULL,
       this);
 }
@@ -950,7 +954,8 @@ MixerBoard::~MixerBoard()
 
 void MixerBoard::UpdatePrefs()
 {
-   mProject->RecreateMixerBoard();
+   // Destroys this:
+   static_cast<MixerBoardFrame*>(GetParent())->Recreate( mProject );
 
 // Old approach modified things in situ.
 // However with a theme change there is so much to modify, it is easier
@@ -968,7 +973,7 @@ void MixerBoard::UpdatePrefs()
       mImageSoloDown.reset();
       mImageSoloDisabled.reset();
    }
-   for (unsigned int nClusterIndex = 0; nClusterIndex < mMixerTrackClusters.GetCount(); nClusterIndex++)
+   for (unsigned int nClusterIndex = 0; nClusterIndex < mMixerTrackClusters.size(); nClusterIndex++)
       mMixerTrackClusters[nClusterIndex]->UpdatePrefs();
    Refresh();
 #endif
@@ -987,47 +992,38 @@ void MixerBoard::UpdateTrackClusters()
       this->CreateMuteSoloImages();
 
    const int nClusterHeight = mScrolledWindow->GetClientSize().GetHeight() - kDoubleInset;
-   size_t nClusterCount = mMixerTrackClusters.GetCount();
+   size_t nClusterCount = mMixerTrackClusters.size();
    unsigned int nClusterIndex = 0;
-   TrackListIterator iterTracks(mTracks);
    MixerTrackCluster* pMixerTrackCluster = NULL;
-   Track* pTrack;
 
-   pTrack = iterTracks.First();
-   while (pTrack) {
-      if (auto pPlayableTrack = dynamic_cast<PlayableTrack*>(pTrack))
+   for (auto pPlayableTrack: mTracks->Leaders<PlayableTrack>()) {
+      // TODO: more-than-two-channels
+      auto spTrack = pPlayableTrack->SharedPointer<PlayableTrack>();
+      if (nClusterIndex < nClusterCount)
       {
-         auto spTrack = Track::Pointer<PlayableTrack>( pPlayableTrack );
-         if (nClusterIndex < nClusterCount)
-         {
-            // Already showing it.
-            // Track clusters are maintained in the same order as the WaveTracks.
-            // Track pointers can change for the "same" track for different states
-            // on the undo stack, so update the pointers and display name.
-            mMixerTrackClusters[nClusterIndex]->mTrack = spTrack;
-            // Assume linked track is wave or null
-            mMixerTrackClusters[nClusterIndex]->UpdateForStateChange();
-         }
-         else
-         {
-            // Not already showing it. Add a NEW MixerTrackCluster.
-            wxPoint clusterPos(
-               (kInset +                                       // extra inset to left for first one, so it's double
-                  (nClusterIndex *
-                     (kInset + kMixerTrackClusterWidth)) +     // left margin and width for each to its left
-                  kInset),                                     // plus left margin for NEW cluster
-               kInset);
-            wxSize clusterSize(kMixerTrackClusterWidth, nClusterHeight);
-            pMixerTrackCluster =
-               safenew MixerTrackCluster(mScrolledWindow, this, mProject,
-                                       spTrack,
-                                       clusterPos, clusterSize);
-            if (pMixerTrackCluster)
-               mMixerTrackClusters.Add(pMixerTrackCluster);
-         }
-         nClusterIndex++;
+         // Already showing it.
+         // Track clusters are maintained in the same order as the WaveTracks.
+         // Track pointers can change for the "same" track for different states
+         // on the undo stack, so update the pointers and display name.
+         mMixerTrackClusters[nClusterIndex]->mTrack = spTrack;
+         // Assume linked track is wave or null
+         mMixerTrackClusters[nClusterIndex]->UpdateForStateChange();
       }
-      pTrack = iterTracks.Next(true);
+      else
+      {
+         // Not already showing it. Add a NEW MixerTrackCluster.
+         wxPoint clusterPos(
+            kInset + nClusterIndex * kMixerTrackClusterWidth,
+            kInset);
+         wxSize clusterSize(kMixerTrackClusterWidth, nClusterHeight);
+         pMixerTrackCluster =
+            safenew MixerTrackCluster(mScrolledWindow, this, mProject,
+                                    spTrack,
+                                    clusterPos, clusterSize);
+         if (pMixerTrackCluster)
+            mMixerTrackClusters.push_back(pMixerTrackCluster);
+      }
+      nClusterIndex++;
    }
 
    if (pMixerTrackCluster)
@@ -1051,68 +1047,21 @@ int MixerBoard::GetTrackClustersWidth()
 {
    return
       kInset +                                     // extra margin at left for first one
-      (mMixerTrackClusters.GetCount() *            // number of tracks times
+      (mMixerTrackClusters.size() *            // number of tracks times
          (kInset + kMixerTrackClusterWidth)) +     // left margin and width for each
       kDoubleInset;                                // plus final right margin
 }
 
-void MixerBoard::MoveTrackCluster(const PlayableTrack* pTrack,
-                                  bool bUp) // Up in TrackPanel is left in MixerBoard.
-{
-   MixerTrackCluster* pMixerTrackCluster;
-   int nIndex = FindMixerTrackCluster(pTrack, &pMixerTrackCluster);
-   if (pMixerTrackCluster == NULL)
-      return; // Couldn't find it.
-
-   wxPoint pos;
-   if (bUp)
-   {  // Move it up (left).
-      if (nIndex <= 0)
-         return; // It's already first (0), or not found (-1).
-
-      pos = pMixerTrackCluster->GetPosition();
-      mMixerTrackClusters[nIndex] = mMixerTrackClusters[nIndex - 1];
-      mMixerTrackClusters[nIndex]->Move(pos);
-
-      mMixerTrackClusters[nIndex - 1] = pMixerTrackCluster;
-      pMixerTrackCluster->Move(pos.x - (kInset + kMixerTrackClusterWidth), pos.y);
-   }
-   else
-   {  // Move it down (right).
-      if (((unsigned int)nIndex + 1) >= mMixerTrackClusters.GetCount())
-         return; // It's already last.
-
-      pos = pMixerTrackCluster->GetPosition();
-      mMixerTrackClusters[nIndex] = mMixerTrackClusters[nIndex + 1];
-      mMixerTrackClusters[nIndex]->Move(pos);
-
-      mMixerTrackClusters[nIndex + 1] = pMixerTrackCluster;
-      pMixerTrackCluster->Move(pos.x + (kInset + kMixerTrackClusterWidth), pos.y);
-   }
-}
-
-void MixerBoard::RemoveTrackCluster(const PlayableTrack* pTrack)
-{
-   // Find and destroy.
-   MixerTrackCluster* pMixerTrackCluster;
-   int nIndex = this->FindMixerTrackCluster(pTrack, &pMixerTrackCluster);
-
-   if (pMixerTrackCluster == NULL)
-      return; // Couldn't find it.
-
-   RemoveTrackCluster(nIndex);
-}
-
 void MixerBoard::RemoveTrackCluster(size_t nIndex)
 {
    auto pMixerTrackCluster = mMixerTrackClusters[nIndex];
-   mMixerTrackClusters.RemoveAt(nIndex);
+   mMixerTrackClusters.erase(mMixerTrackClusters.begin() + nIndex);
    pMixerTrackCluster->Destroy(); // DELETE is unsafe on wxWindow.
 
    // Close the gap, if any.
    wxPoint pos;
    int targetX;
-   for (unsigned int i = nIndex; i < mMixerTrackClusters.GetCount(); i++)
+   for (unsigned int i = nIndex; i < mMixerTrackClusters.size(); i++)
    {
       pos = mMixerTrackClusters[i]->GetPosition();
       targetX =
@@ -1132,7 +1081,7 @@ wxBitmap* MixerBoard::GetMusicalInstrumentBitmap(const Track* pTrack)
    if (mMusicalInstruments.empty())
       return NULL;
 
-   // random choice:    return mMusicalInstruments[(int)pTrack % mMusicalInstruments.GetCount()].mBitmap;
+   // random choice:    return mMusicalInstruments[(int)pTrack % mMusicalInstruments.size()].mBitmap;
 
    const wxString strTrackName(pTrack->GetName().MakeLower());
    size_t nBestItemIndex = 0;
@@ -1146,7 +1095,7 @@ wxBitmap* MixerBoard::GetMusicalInstrumentBitmap(const Track* pTrack)
    {
       nScore = 0;
 
-      nNumKeywords = mMusicalInstruments[nInstrIndex]->mKeywords.GetCount();
+      nNumKeywords = mMusicalInstruments[nInstrIndex]->mKeywords.size();
       if (nNumKeywords > 0)
       {
          nPointsPerMatch = 10 / nNumKeywords;
@@ -1156,7 +1105,7 @@ wxBitmap* MixerBoard::GetMusicalInstrumentBitmap(const Track* pTrack)
                nScore +=
                   nPointsPerMatch +
                   // Longer keywords get more points.
-                  (2 * mMusicalInstruments[nInstrIndex]->mKeywords[nKeywordIndex].Length());
+                  (2 * mMusicalInstruments[nInstrIndex]->mKeywords[nKeywordIndex].length());
             }
       }
 
@@ -1173,33 +1122,18 @@ wxBitmap* MixerBoard::GetMusicalInstrumentBitmap(const Track* pTrack)
 
 bool MixerBoard::HasSolo()
 {
-   TrackListIterator iterTracks(mTracks);
-   Track* pTrack;
-   for (pTrack = iterTracks.First(); pTrack; pTrack = iterTracks.Next()) {
-      auto pPlayable = dynamic_cast<PlayableTrack *>( pTrack );
-      if (pPlayable && pPlayable->GetSolo())
-         return true;
-   }
-   return false;
-}
-
-void MixerBoard::RefreshTrackCluster(const PlayableTrack* pTrack, bool bEraseBackground /*= true*/)
-{
-   MixerTrackCluster* pMixerTrackCluster;
-   this->FindMixerTrackCluster(pTrack, &pMixerTrackCluster);
-   if (pMixerTrackCluster)
-      pMixerTrackCluster->Refresh(bEraseBackground);
+   return !(( mTracks->Any<PlayableTrack>() + &PlayableTrack::GetSolo ).empty());
 }
 
 void MixerBoard::RefreshTrackClusters(bool bEraseBackground /*= true*/)
 {
-   for (unsigned int i = 0; i < mMixerTrackClusters.GetCount(); i++)
+   for (unsigned int i = 0; i < mMixerTrackClusters.size(); i++)
       mMixerTrackClusters[i]->Refresh(bEraseBackground);
 }
 
 void MixerBoard::ResizeTrackClusters()
 {
-   for (unsigned int nClusterIndex = 0; nClusterIndex < mMixerTrackClusters.GetCount(); nClusterIndex++)
+   for (unsigned int nClusterIndex = 0; nClusterIndex < mMixerTrackClusters.size(); nClusterIndex++)
       mMixerTrackClusters[nClusterIndex]->HandleResize();
 }
 
@@ -1210,84 +1144,10 @@ void MixerBoard::ResetMeters(const bool bResetClipping)
    if (!this->IsShown())
       return;
 
-   for (unsigned int i = 0; i < mMixerTrackClusters.GetCount(); i++)
+   for (unsigned int i = 0; i < mMixerTrackClusters.size(); i++)
       mMixerTrackClusters[i]->ResetMeter(bResetClipping);
 }
 
-void MixerBoard::UpdateName(const PlayableTrack* pTrack)
-{
-   MixerTrackCluster* pMixerTrackCluster;
-   this->FindMixerTrackCluster(pTrack, &pMixerTrackCluster);
-   if (pMixerTrackCluster)
-      pMixerTrackCluster->UpdateName();
-}
-
-void MixerBoard::UpdateMute(const PlayableTrack* pTrack /*= NULL*/) // NULL means update for all tracks.
-{
-   if (pTrack == NULL)
-   {
-      for (unsigned int i = 0; i < mMixerTrackClusters.GetCount(); i++)
-         mMixerTrackClusters[i]->UpdateMute();
-   }
-   else
-   {
-      MixerTrackCluster* pMixerTrackCluster;
-      FindMixerTrackCluster(pTrack, &pMixerTrackCluster);
-      if (pMixerTrackCluster)
-         pMixerTrackCluster->UpdateMute();
-   }
-}
-
-void MixerBoard::UpdateSolo(const PlayableTrack* pTrack /*= NULL*/) // NULL means update for all tracks.
-{
-   if (pTrack == NULL)
-   {
-      for (unsigned int i = 0; i < mMixerTrackClusters.GetCount(); i++)
-         mMixerTrackClusters[i]->UpdateSolo();
-   }
-   else
-   {
-      MixerTrackCluster* pMixerTrackCluster;
-      FindMixerTrackCluster(pTrack, &pMixerTrackCluster);
-      if (pMixerTrackCluster)
-         pMixerTrackCluster->UpdateSolo();
-   }
-}
-
-void MixerBoard::UpdatePan(const PlayableTrack* pTrack)
-{
-   if (pTrack == NULL)
-   {
-      for (unsigned int i = 0; i < mMixerTrackClusters.GetCount(); i++)
-         mMixerTrackClusters[i]->UpdatePan();
-   }
-   else
-   {
-      MixerTrackCluster* pMixerTrackCluster;
-      FindMixerTrackCluster(pTrack, &pMixerTrackCluster);
-      if (pMixerTrackCluster)
-         pMixerTrackCluster->UpdatePan();
-   }
-}
-
-void MixerBoard::UpdateGain(const PlayableTrack* pTrack)
-{
-   MixerTrackCluster* pMixerTrackCluster;
-   FindMixerTrackCluster(pTrack, &pMixerTrackCluster);
-   if (pMixerTrackCluster)
-      pMixerTrackCluster->UpdateGain();
-}
-
-#ifdef EXPERIMENTAL_MIDI_OUT
-void MixerBoard::UpdateVelocity(const PlayableTrack* pTrack)
-{
-   MixerTrackCluster* pMixerTrackCluster;
-   FindMixerTrackCluster(pTrack, &pMixerTrackCluster);
-   if (pMixerTrackCluster)
-      pMixerTrackCluster->UpdateVelocity();
-}
-#endif
-
 void MixerBoard::UpdateMeters(const double t1, const bool bLoopedPlay)
 {
    if (!this->IsShown() || (t1 == BAD_STREAM_TIME))
@@ -1309,7 +1169,7 @@ void MixerBoard::UpdateMeters(const double t1, const bool bLoopedPlay)
       return;
    }
 
-   for (unsigned int i = 0; i < mMixerTrackClusters.GetCount(); i++)
+   for (unsigned int i = 0; i < mMixerTrackClusters.size(); i++)
       mMixerTrackClusters[i]->UpdateMeter(mPrevT1, t1);
 
    mPrevT1 = t1;
@@ -1342,7 +1202,7 @@ void MixerBoard::MakeButtonBitmap( wxMemoryDC & dc, wxBitmap & WXUNUSED(bitmap),
    #ifdef __WXMSW__
       fontSize = 8;
    #endif
-   wxFont font(fontSize, wxSWISS, wxNORMAL, wxNORMAL);
+   wxFont font(fontSize, wxFONTFAMILY_SWISS, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
    GetTextExtent(str, &textWidth, &textHeight, NULL, NULL, &font);
 
    AColor::UseThemeColour( &dc, clrMedium );
@@ -1404,7 +1264,7 @@ int MixerBoard::FindMixerTrackCluster(const PlayableTrack* pTrack,
                                         MixerTrackCluster** hMixerTrackCluster) const
 {
    *hMixerTrackCluster = NULL;
-   for (unsigned int i = 0; i < mMixerTrackClusters.GetCount(); i++)
+   for (unsigned int i = 0; i < mMixerTrackClusters.size(); i++)
    {
       if (mMixerTrackClusters[i]->mTrack.get() == pTrack)
       {
@@ -1417,7 +1277,7 @@ int MixerBoard::FindMixerTrackCluster(const PlayableTrack* pTrack,
 
 void MixerBoard::LoadMusicalInstruments()
 {
-   const struct Data { const char **bitmap; wxString name; } table[] = {
+   const struct Data { const char * const *bitmap; wxString name; } table[] = {
       {acoustic_guitar_gtr_xpm, wxT("acoustic_guitar_gtr")},
       {acoustic_piano_pno_xpm, wxT("acoustic_piano_pno")},
       {back_vocal_bg_vox_xpm, wxT("back_vocal_bg_vox")},
@@ -1448,10 +1308,10 @@ void MixerBoard::LoadMusicalInstruments()
    wxMemoryDC dc;
 
    for (const auto &data : table) {
-      auto bmp = std::make_unique<wxBitmap>(data.bitmap);
+      auto bmp = std::make_unique<wxBitmap>(data.bitmap,24);
       dc.SelectObject(*bmp);
       AColor::Bevel(dc, false, bev);
-      mMusicalInstruments.push_back(make_movable<MusicalInstrument>(
+      mMusicalInstruments.push_back(std::make_unique<MusicalInstrument>(
          std::move(bmp), data.name
       ));
    };
@@ -1459,6 +1319,17 @@ void MixerBoard::LoadMusicalInstruments()
 
 // event handlers
 
+void MixerBoard::OnPaint(wxPaintEvent& evt)
+{
+   if (!mUpToDate) {
+      mUpToDate = true;
+      UpdateTrackClusters();
+      Refresh();
+   }
+   // Does the base class do anything for repainting?
+   evt.Skip();
+}
+
 void MixerBoard::OnSize(wxSizeEvent &evt)
 {
    // this->FitInside() doesn't work, and it doesn't happen automatically. Is wxScrolledWindow wrong?
@@ -1492,6 +1363,34 @@ void MixerBoard::OnTimer(wxCommandEvent &event)
    event.Skip();
 }
 
+void MixerBoard::OnTrackChanged(TrackListEvent &evt)
+{
+   evt.Skip();
+
+   auto pTrack = evt.mpTrack.lock();
+   auto pPlayable = dynamic_cast<PlayableTrack*>( pTrack.get() );
+   if ( pPlayable ) {
+      MixerTrackCluster *pMixerTrackCluster;
+      FindMixerTrackCluster( pPlayable, &pMixerTrackCluster );
+      if ( pMixerTrackCluster )
+         pMixerTrackCluster->Refresh();
+   }
+}
+
+void MixerBoard::OnTrackSetChanged(wxEvent &evt)
+{
+   evt.Skip();
+   mUpToDate = false;
+   UpdateTrackClusters();
+   Refresh();
+}
+
+void MixerBoard::OnStartStop(wxCommandEvent &evt)
+{
+   evt.Skip();
+   bool start = evt.GetInt();
+   ResetMeters( start );
+}
 
 // class MixerBoardFrame
 
@@ -1509,10 +1408,10 @@ const wxSize kDefaultSize =
 MixerBoardFrame::MixerBoardFrame(AudacityProject* parent)
 : wxFrame(parent, -1,
           wxString::Format(_("Audacity Mixer Board%s"),
-                           ((parent->GetName() == wxEmptyString) ?
+                           ((parent->GetProjectName().empty()) ?
                               wxT("") :
                               wxString::Format(wxT(" - %s"),
-                                             parent->GetName()))),
+                                             parent->GetProjectName()))),
             wxDefaultPosition, kDefaultSize,
             //vvv Bug in wxFRAME_FLOAT_ON_PARENT:
             // If both the project frame and MixerBoardFrame are minimized and you restore MixerBoardFrame,
@@ -1570,4 +1469,22 @@ void MixerBoardFrame::OnKeyEvent(wxKeyEvent & event)
    project->GetCommandManager()->FilterKeyEvent(project, event, true);
 }
 
+void MixerBoardFrame::Recreate( AudacityProject *pProject )
+{
+   wxPoint  pos = mMixerBoard->GetPosition();
+   wxSize siz = mMixerBoard->GetSize();
+   wxSize siz2 = this->GetSize();
+
+   //wxLogDebug("Got rid of board %p", mMixerBoard );
+   mMixerBoard->Destroy();
+   mMixerBoard = NULL;
+   mMixerBoard = safenew MixerBoard(pProject, this, pos, siz);
+   //wxLogDebug("Created NEW board %p", mMixerBoard );
+   mMixerBoard->UpdateTrackClusters();
+   mMixerBoard->SetSize( siz );
+
+   this->SetSize( siz2 );
+}
+
+
 
