@@ -14,9 +14,6 @@
 #ifndef __AUDACITY_SILENT_BLOCKFILE__
 #define __AUDACITY_SILENT_BLOCKFILE__
 
-#include <wx/string.h>
-#include <wx/filename.h>
-
 #include "../BlockFile.h"
 #include "../DirManager.h"
 
