@@ -5,6 +5,7 @@
    License: wxwidgets
 
    Dan Horgan
+   James Crook
 
 ******************************************************************//**
 
@@ -22,32 +23,35 @@ classes
 
 class WaveTrack;
 
-class CompareAudioCommandType final : public CommandType
+#define COMPARE_AUDIO_PLUGIN_SYMBOL ComponentInterfaceSymbol{ XO("Compare Audio") }
+
+class CompareAudioCommand final : public AudacityCommand
 {
 public:
-   wxString BuildName() override;
-   void BuildSignature(CommandSignature &signature) override;
-   CommandHolder Create(std::unique_ptr<CommandOutputTarget> &&target) override;
-};
+   // ComponentInterface overrides
+   ComponentInterfaceSymbol GetSymbol() override {return XO("Compare Audio");}
+   wxString GetDescription() override {return _("Compares a range on two tracks.");};
+   bool DefineParams( ShuttleParams & S ) override;
+   void PopulateOrExchange(ShuttleGui & S) override;
+   bool Apply() override;
+
+   // AudacityCommand overrides
+   wxString ManualPage() override {return wxT("Extra_Menu:_Scriptables_II#compare_Audio");};
+   bool Apply(const CommandContext &context) override;
+
 
-class CompareAudioCommand final : public CommandImplementation
-{
 private:
+   double errorThreshold;
    double mT0, mT1;
    const WaveTrack *mTrack0;
    const WaveTrack *mTrack1;
 
    // Update member variables with project selection data (and validate)
-   bool GetSelection(AudacityProject &proj);
+   bool GetSelection(const CommandContext &context, AudacityProject &proj);
 
 protected:
    double CompareSample(double value1, double value2) /* not override */;
 
-public:
-   CompareAudioCommand(CommandType &type, std::unique_ptr<CommandOutputTarget> &&target)
-      : CommandImplementation(type, std::move(target))
-   { }
-   bool Apply(CommandExecutionContext context) override;
 };
 
 #endif /* End of include guard: __COMPAREAUDIOCOMMAND__ */
