@@ -30,9 +30,11 @@
 #include <math.h>
 
 #include <wx/intl.h>
+#include <wx/slider.h>
 #include <wx/valgen.h>
 
 #include "../Prefs.h"
+#include "../Shuttle.h"
 #include "../ShuttleGui.h"
 #include "../widgets/ErrorDialog.h"
 #include "../widgets/valnum.h"
@@ -73,9 +75,9 @@ EffectClickRemoval::~EffectClickRemoval()
 {
 }
 
-// IdentInterface implementation
+// ComponentInterface implementation
 
-wxString EffectClickRemoval::GetSymbol()
+ComponentInterfaceSymbol EffectClickRemoval::GetSymbol()
 {
    return CLICKREMOVAL_PLUGIN_SYMBOL;
 }
@@ -90,7 +92,7 @@ wxString EffectClickRemoval::ManualPage()
    return wxT("Click_Removal");
 }
 
-// EffectIdentInterface implementation
+// EffectDefinitionInterface implementation
 
 EffectType EffectClickRemoval::GetType()
 {
@@ -98,8 +100,13 @@ EffectType EffectClickRemoval::GetType()
 }
 
 // EffectClientInterface implementation
+bool EffectClickRemoval::DefineParams( ShuttleParams & S ){
+   S.SHUTTLE_PARAM( mThresholdLevel, Threshold );
+   S.SHUTTLE_PARAM( mClickWidth, Width );
+   return true;
+}
 
-bool EffectClickRemoval::GetAutomationParameters(EffectAutomationParameters & parms)
+bool EffectClickRemoval::GetAutomationParameters(CommandParameters & parms)
 {
    parms.Write(KEY_Threshold, mThresholdLevel);
    parms.Write(KEY_Width, mClickWidth);
@@ -107,7 +114,7 @@ bool EffectClickRemoval::GetAutomationParameters(EffectAutomationParameters & pa
    return true;
 }
 
-bool EffectClickRemoval::SetAutomationParameters(EffectAutomationParameters & parms)
+bool EffectClickRemoval::SetAutomationParameters(CommandParameters & parms)
 {
    ReadAndVerifyInt(Threshold);
    ReadAndVerifyInt(Width);
@@ -167,10 +174,8 @@ bool EffectClickRemoval::Process()
    bool bGoodResult = true;
    mbDidSomething = false;
 
-   SelectedTrackListOfKindIterator iter(Track::Wave, mOutputTracks.get());
-   WaveTrack *track = (WaveTrack *) iter.First();
    int count = 0;
-   while (track) {
+   for( auto track : mOutputTracks->Selected< WaveTrack >() ) {
       double trackStart = track->GetStartTime();
       double trackEnd = track->GetEndTime();
       double t0 = mT0 < trackStart? trackStart: mT0;
@@ -188,7 +193,6 @@ bool EffectClickRemoval::Process()
          }
       }
 
-      track = (WaveTrack *) iter.Next();
       count++;
    }
    if (bGoodResult && !mbDidSomething) // Processing successful, but ineffective.
