@@ -20,6 +20,7 @@
 #include "../widgets/LinkingHtmlWindow.h"
 #include "../widgets/HelpSystem.h"
 #include "../widgets/NumericTextCtrl.h"
+#include "../widgets/ErrorDialog.h"
 
 #include <cmath>
 #include <limits>
@@ -29,6 +30,7 @@
 #define finite(x) _finite(x)
 #endif
 
+#include <wx/button.h>
 #include <wx/filedlg.h>
 #include <wx/valtext.h>
 #include <wx/log.h>
@@ -38,27 +40,31 @@
 #define DB_MAX_LIMIT 0.0   // Audio is massively distorted.
 #define WCAG2_PASS 20.0    // dB difference required to pass WCAG2 test.
 
+
 bool ContrastDialog::GetDB(float &dB)
 {
    float rms = float(0.0);
-   int numberSelecteTracks = 0;
 
    // For stereo tracks: sqrt((mean(L)+mean(R))/2)
-   bool isStereo = false;
    double meanSq = 0.0;
 
    AudacityProject *p = GetActiveProject();
-   SelectedTrackListOfKindIterator iter(Track::Wave, p->GetTracks());
-   WaveTrack *t = (WaveTrack *) iter.First();
-   while (t) {
-      numberSelecteTracks++;
-      if (numberSelecteTracks > 1 && !isStereo) {
-         wxMessageDialog m(NULL, _("You can only measure one track at a time."), _("Error"), wxOK);
-         m.ShowModal();
-         return false;
-      }
-      isStereo = t->GetLinked();
+   auto range =
+      p->GetTracks()->SelectedLeaders< const WaveTrack >();
+   auto numberSelectedTracks = range.size();
+   if (numberSelectedTracks > 1) {
+      AudacityMessageDialog m(NULL, _("You can only measure one track at a time."), _("Error"), wxOK);
+      m.ShowModal();
+      return false;
+   }
+   if(numberSelectedTracks == 0) {
+      AudacityMessageDialog m(NULL, _("Please select an audio track."), _("Error"), wxOK);
+      m.ShowModal();
+      return false;
+   }
 
+   const auto channels = TrackList::Channels( *range.begin() );
+   for ( auto t : channels ) {
       wxASSERT(mT0 <= mT1);
 
       // Ignore whitespace beyond ends of track.
@@ -72,32 +78,33 @@ bool ContrastDialog::GetDB(float &dB)
 
       if(SelT0 > SelT1)
       {
-         wxMessageDialog m(NULL, _("Invalid audio selection.\nPlease ensure that audio is selected."), _("Error"), wxOK);
+         AudacityMessageDialog m(NULL, _("Invalid audio selection.\nPlease ensure that audio is selected."), _("Error"), wxOK);
          m.ShowModal();
          return false;
       }
 
       if(SelT0 == SelT1)
       {
-         wxMessageDialog m(NULL, _("Nothing to measure.\nPlease select a section of a track."), _("Error"), wxOK);
+         AudacityMessageDialog m(NULL, _("Nothing to measure.\nPlease select a section of a track."), _("Error"), wxOK);
          m.ShowModal();
          return false;
       }
 
       // Don't throw in this analysis dialog
-      rms = ((WaveTrack *)t)->GetRMS(mT0, mT1, false);
+      rms = t->GetRMS(mT0, mT1, false);
       meanSq += rms * rms;
-      t = (WaveTrack *) iter.Next();
    }
    // TODO: This works for stereo, provided the audio clips are in both channels.
    // We should really count gaps between clips as silence.
-   rms = (meanSq > 0.0)? sqrt(meanSq/(double)numberSelecteTracks) : 0.0;
+   rms = (meanSq > 0.0)
+      ? sqrt( meanSq/static_cast<double>( channels.size() ) )
+      : 0.0;
 
-   if(numberSelecteTracks == 0) {
-      wxMessageDialog m(NULL, _("Please select an audio track."), _("Error"), wxOK);
-      m.ShowModal();
-      return false;
-   }
+   // Gives warning C4056, Overflow in floating-point constant arithmetic
+   // -INFINITY is intentional here.
+   // Looks like we are stuck with this warning, as 
+   // #pragma warning( disable : 4056)
+   // even around the whole function does not disable it successfully.
 
    dB = (rms == 0.0)? -INFINITY : LINEAR_TO_DB(rms);
    return true;
@@ -145,6 +152,20 @@ BEGIN_EVENT_TABLE(ContrastDialog,wxDialogWrapper)
    EVT_BUTTON(wxID_CANCEL, ContrastDialog::OnClose)
 END_EVENT_TABLE()
 
+static void OnChar(wxKeyEvent & event)
+{
+   // Is this still required?
+   if (event.GetKeyCode() == WXK_TAB) {
+      // pass to next handler
+      event.Skip();
+      return;
+   }
+
+   // ignore any other key
+   event.Skip(false);
+   return;
+}
+
 /* i18n-hint: WCAG2 is the 'Web Content Accessibility Guidelines (WCAG) 2.0', see http://www.w3.org/TR/WCAG20/ */
 ContrastDialog::ContrastDialog(wxWindow * parent, wxWindowID id,
                            const wxString & title,
@@ -187,91 +208,80 @@ ContrastDialog::ContrastDialog(wxWindow * parent, wxWindowID id,
       {
 
          // Headings
-         S.AddFixedText(wxT(""));   // spacer
+         S.AddFixedText( {} );   // spacer
          S.AddFixedText(_("Start"));
          S.AddFixedText(_("End"));
-         S.AddFixedText(wxT(""));   // spacer
+         S.AddFixedText( {} );   // spacer
          S.AddFixedText(_("Volume    "));
 
+         const auto options = NumericTextCtrl::Options{}
+            .AutoPos(true)
+            .MenuEnabled(false)
+            .ReadOnly(true);
+
          //Foreground
          S.AddFixedText(_("&Foreground:"), false);
          if (S.GetMode() == eIsCreating)
          {
             mForegroundStartT = safenew
-               NumericTextCtrl(NumericConverter::TIME, this,
-                         ID_FOREGROUNDSTART_T,
-                         _("hh:mm:ss + hundredths"),
+               NumericTextCtrl(S.GetParent(), ID_FOREGROUNDSTART_T,
+                         NumericConverter::TIME,
+                         NumericConverter::HundredthsFormat(),
                          0.0,
                          mProjectRate,
-                         wxDefaultPosition,
-                         wxDefaultSize,
-                         true);
+                         options);
             mForegroundStartT->SetName(_("Foreground start time"));
-            mForegroundStartT->EnableMenu(false);
-            mForegroundStartT->SetReadOnly(true);
          }
          S.AddWindow(mForegroundStartT);
 
          if (S.GetMode() == eIsCreating)
          {
             mForegroundEndT = safenew
-               NumericTextCtrl(NumericConverter::TIME, this,
-                         ID_FOREGROUNDEND_T,
-                         _("hh:mm:ss + hundredths"),
+               NumericTextCtrl(S.GetParent(), ID_FOREGROUNDEND_T,
+                         NumericConverter::TIME,
+                         NumericConverter::HundredthsFormat(),
                          0.0,
                          mProjectRate,
-                         wxDefaultPosition,
-                         wxDefaultSize,
-                         true);
+                         options);
             mForegroundEndT->SetName(_("Foreground end time"));
-            mForegroundEndT->EnableMenu(false);
-            mForegroundEndT->SetReadOnly(true);
          }
          S.AddWindow(mForegroundEndT);
 
          m_pButton_UseCurrentF = S.Id(ID_BUTTON_USECURRENTF).AddButton(_("&Measure selection"));
-         mForegroundRMSText=S.Id(ID_FOREGROUNDDB_TEXT).AddTextBox(wxT(""), wxT(""), 17);
-         mForegroundRMSText->Connect(wxEVT_KEY_DOWN, wxKeyEventHandler(ContrastDialog::OnChar));
+         mForegroundRMSText=S.Id(ID_FOREGROUNDDB_TEXT).AddTextBox( {}, wxT(""), 17);
+         mForegroundRMSText->Bind(wxEVT_KEY_DOWN, OnChar);
 
          //Background
          S.AddFixedText(_("&Background:"));
          if (S.GetMode() == eIsCreating)
          {
             mBackgroundStartT = safenew
-               NumericTextCtrl(NumericConverter::TIME, this,
-                         ID_BACKGROUNDSTART_T,
-                         _("hh:mm:ss + hundredths"),
+               NumericTextCtrl(S.GetParent(), ID_BACKGROUNDSTART_T,
+                         NumericConverter::TIME,
+                         NumericConverter::HundredthsFormat(),
                          0.0,
                          mProjectRate,
-                         wxDefaultPosition,
-                         wxDefaultSize,
-                         true);
+                         options);
             mBackgroundStartT->SetName(_("Background start time"));
-            mBackgroundStartT->EnableMenu(false);
-            mBackgroundStartT->SetReadOnly(true);
          }
          S.AddWindow(mBackgroundStartT);
 
          if (S.GetMode() == eIsCreating)
          {
             mBackgroundEndT = safenew
-               NumericTextCtrl(NumericConverter::TIME, this,
-                         ID_BACKGROUNDEND_T,
-                         _("hh:mm:ss + hundredths"),
+               NumericTextCtrl(S.GetParent(), ID_BACKGROUNDEND_T,
+                         NumericConverter::TIME,
+                         NumericConverter::HundredthsFormat(),
                          0.0,
                          mProjectRate,
-                         wxDefaultPosition,
-                         wxDefaultSize,
-                         true);
+                         options);
             mBackgroundEndT->SetName(_("Background end time"));
-            mBackgroundEndT->EnableMenu(false);
-            mBackgroundEndT->SetReadOnly(true);
          }
          S.AddWindow(mBackgroundEndT);
 
          m_pButton_UseCurrentB = S.Id(ID_BUTTON_USECURRENTB).AddButton(_("Mea&sure selection"));
-         mBackgroundRMSText = S.Id(ID_BACKGROUNDDB_TEXT).AddTextBox(wxT(""), wxT(""), 17);
-         mBackgroundRMSText->Connect(wxEVT_KEY_DOWN, wxKeyEventHandler(ContrastDialog::OnChar));
+         mBackgroundRMSText = S.Id(ID_BACKGROUNDDB_TEXT).AddTextBox( {}, wxT(""), 17);
+         mBackgroundRMSText->Bind(wxEVT_KEY_DOWN, OnChar);
       }
       S.EndMultiColumn();
    }
@@ -282,13 +292,18 @@ ContrastDialog::ContrastDialog(wxWindow * parent, wxWindowID id,
    {
       S.StartMultiColumn(3, wxCENTER);
       {
-         S.AddFixedText(_("Co&ntrast Result:"));
-         mPassFailText = S.Id(ID_RESULTS_TEXT).AddTextBox(wxT(""), wxT(""), 50);
-         mPassFailText->Connect(wxEVT_KEY_DOWN, wxKeyEventHandler(ContrastDialog::OnChar));
+         wxString label = _("Co&ntrast Result:");
+         S.AddFixedText(label);
+         mPassFailText = S.Id(ID_RESULTS_TEXT).AddTextBox( {}, wxT(""), 50);
+         mPassFailText->SetName(wxStripMenuCodes(label));
+         mPassFailText->Bind(wxEVT_KEY_DOWN, OnChar);
          m_pButton_Reset = S.Id(ID_BUTTON_RESET).AddButton(_("R&eset"));
-         S.AddFixedText(_("&Difference:"));
-         mDiffText = S.Id(ID_RESULTSDB_TEXT).AddTextBox(wxT(""), wxT(""), 50);
-         mDiffText->Connect(wxEVT_KEY_DOWN, wxKeyEventHandler(ContrastDialog::OnChar));
+
+         label = _("&Difference:");
+         S.AddFixedText(label);
+         mDiffText = S.Id(ID_RESULTSDB_TEXT).AddTextBox( {}, wxT(""), 50);
+         mDiffText->SetName(wxStripMenuCodes(label));
+         mDiffText->Bind(wxEVT_KEY_DOWN, OnChar);
          m_pButton_Export = S.Id(ID_BUTTON_EXPORT).AddButton(_("E&xport..."));
       }
       S.EndMultiColumn();
@@ -311,14 +326,6 @@ ContrastDialog::ContrastDialog(wxWindow * parent, wxWindowID id,
    Center();
 }
 
-ContrastDialog::~ContrastDialog()
-{
-   mForegroundRMSText->Disconnect(wxEVT_KEY_DOWN, wxKeyEventHandler(ContrastDialog::OnChar));
-   mBackgroundRMSText->Disconnect(wxEVT_KEY_DOWN, wxKeyEventHandler(ContrastDialog::OnChar));
-   mPassFailText->Disconnect(wxEVT_KEY_DOWN, wxKeyEventHandler(ContrastDialog::OnChar));
-   mDiffText->Disconnect(wxEVT_KEY_DOWN, wxKeyEventHandler(ContrastDialog::OnChar));
-}
-
 void ContrastDialog::OnGetURL(wxCommandEvent & WXUNUSED(event))
 {
    // Original help page is back on-line (March 2016), but the manual should be more reliable.
@@ -337,9 +344,8 @@ void ContrastDialog::OnClose(wxCommandEvent & WXUNUSED(event))
 void ContrastDialog::OnGetForeground(wxCommandEvent & /*event*/)
 {
    AudacityProject *p = GetActiveProject();
-   SelectedTrackListOfKindIterator iter(Track::Wave, p->GetTracks());
 
-   for (Track *t = iter.First(); t; t = iter.Next()) {
+   if( p->GetTracks()->Selected< const WaveTrack >() ) {
       mForegroundStartT->SetValue(p->mViewInfo.selectedRegion.t0());
       mForegroundEndT->SetValue(p->mViewInfo.selectedRegion.t1());
    }
@@ -353,9 +359,8 @@ void ContrastDialog::OnGetForeground(wxCommandEvent & /*event*/)
 void ContrastDialog::OnGetBackground(wxCommandEvent & /*event*/)
 {
    AudacityProject *p = GetActiveProject();
-   SelectedTrackListOfKindIterator iter(Track::Wave, p->GetTracks());
 
-   for (Track *t = iter.First(); t; t = iter.Next()) {
+   if( p->GetTracks()->Selected< const WaveTrack >() ) {
       mBackgroundStartT->SetValue(p->mViewInfo.selectedRegion.t0());
       mBackgroundEndT->SetValue(p->mViewInfo.selectedRegion.t1());
    }
@@ -407,7 +412,7 @@ namespace {
             return wxString::Format(_("%.2f dB RMS"), diffdB);
          else
             /* i18n-hint: dB abbreviates decibels */
-            return wxString::Format(_("Infinite dB difference"));
+            return _("Infinite dB difference");
       }
    }
 
@@ -462,27 +467,27 @@ void ContrastDialog::results()
    if (mForegroundIsDefined) {
       mForegroundRMSText->SetName(_("Measured foreground level"));   // Read by screen-readers
       if(std::isinf(- foregrounddB))
-         mForegroundRMSText->ChangeValue(wxString::Format(_("zero")));
+         mForegroundRMSText->ChangeValue(_("zero"));
       else
          mForegroundRMSText->ChangeValue(wxString::Format(_("%.2f dB"), foregrounddB));   // i18n-hint: short form of 'decibels'        
    }
    else {
       mForegroundRMSText->SetName(_("No foreground measured"));   // Read by screen-readers
       mForegroundRMSText->ChangeValue(wxT(""));
-      mPassFailText->ChangeValue(wxString::Format(_("Foreground not yet measured")));
+      mPassFailText->ChangeValue(_("Foreground not yet measured"));
    }
 
    if (mBackgroundIsDefined) {
       mBackgroundRMSText->SetName(_("Measured background level"));
       if(std::isinf(- backgrounddB))
-         mBackgroundRMSText->ChangeValue(wxString::Format(_("zero")));
+         mBackgroundRMSText->ChangeValue(_("zero"));
       else
          mBackgroundRMSText->ChangeValue(wxString::Format(_("%.2f dB"), backgrounddB));
    }
    else {
       mBackgroundRMSText->SetName(_("No background measured"));
       mBackgroundRMSText->ChangeValue(wxT(""));
-      mPassFailText->ChangeValue(wxString::Format(_("Background not yet measured")));
+      mPassFailText->ChangeValue(_("Background not yet measured"));
    }
 }
 
@@ -501,7 +506,7 @@ void ContrastDialog::OnExport(wxCommandEvent & WXUNUSED(event))
                         wxFD_SAVE | wxRESIZE_BORDER,
                         this);
 
-   if (fName == wxT(""))
+   if (fName.empty())
       return;
 
    wxTextFile f(fName);
@@ -512,7 +517,8 @@ void ContrastDialog::OnExport(wxCommandEvent & WXUNUSED(event))
 #endif
    f.Open();
    if (!f.IsOpened()) {
-      wxMessageBox(_("Couldn't write to file: ") + fName);
+      AudacityMessageBox(
+         wxString::Format( _("Couldn't write to file: %s"), fName) );
       return;
    }
 
@@ -520,7 +526,7 @@ void ContrastDialog::OnExport(wxCommandEvent & WXUNUSED(event))
    /* i18n-hint: WCAG abbreviates Web Content Accessibility Guidelines */
    f.AddLine(_("WCAG 2.0 Success Criteria 1.4.7 Contrast Results"));
    f.AddLine(wxT(""));
-   f.AddLine(wxString::Format(_("Filename = %s."), project->GetFileName().c_str() ));
+   f.AddLine(wxString::Format(_("Filename = %s."), project->GetFileName() ));
    f.AddLine(wxT(""));
    f.AddLine(_("Foreground"));
    float t = (float)mForegroundStartT->GetValue();
@@ -569,7 +575,7 @@ void ContrastDialog::OnExport(wxCommandEvent & WXUNUSED(event))
    int minute = now.GetMinute();
    int second = now.GetSecond();
    sNow = wxString::Format(wxT("%d %s %02d %02dh %02dm %02ds"),
-        dom, monthName.c_str(), year, hour, minute, second);
+        dom, monthName, year, hour, minute, second);
    f.AddLine(sNow);
 
    f.AddLine(wxT("==================================="));
@@ -599,15 +605,3 @@ void ContrastDialog::OnReset(wxCommandEvent & /*event*/)
    mPassFailText->ChangeValue(wxT(""));
    mDiffText->ChangeValue(wxT(""));
 }
-
-void ContrastDialog::OnChar(wxKeyEvent & event)
-{
-   // Is this still required?
-   if (event.GetKeyCode() == WXK_TAB) {
-      event.Skip();
-      return;
-   }
-
-   event.Skip(false);
-   return;
-}
