@@ -14,14 +14,18 @@
 *//*******************************************************************/
 
 #include "../Audacity.h"
-#include "../Experimental.h"
 #include "DtmfGen.h"
 
+#include "../Experimental.h"
+
 #include <wx/intl.h>
+#include <wx/slider.h>
 #include <wx/valgen.h>
 #include <wx/valtext.h>
+#include <wx/stattext.h>
 
 #include "../Prefs.h"
+#include "../Shuttle.h"
 #include "../ShuttleGui.h"
 #include "../widgets/valnum.h"
 
@@ -89,16 +93,16 @@ EffectDtmf::~EffectDtmf()
 {
 }
 
-// IdentInterface implementation
+// ComponentInterface implementation
 
-wxString EffectDtmf::GetSymbol()
+ComponentInterfaceSymbol EffectDtmf::GetSymbol()
 {
    return DTMFTONES_PLUGIN_SYMBOL;
 }
 
 wxString EffectDtmf::GetDescription()
 {
-   return XO("Generates dual-tone multi-frequency (DTMF) tones like those produced by the keypad on telephones");
+   return _("Generates dual-tone multi-frequency (DTMF) tones like those produced by the keypad on telephones");
 }
 
 wxString EffectDtmf::ManualPage()
@@ -106,7 +110,7 @@ wxString EffectDtmf::ManualPage()
    return wxT("DTMF_Tones");
 }
 
-// EffectIdentInterface implementation
+// EffectDefinitionInterface implementation
 
 EffectType EffectDtmf::GetType()
 {
@@ -229,8 +233,14 @@ size_t EffectDtmf::ProcessBlock(float **WXUNUSED(inbuf), float **outbuf, size_t
 
    return processed;
 }
+bool EffectDtmf::DefineParams( ShuttleParams & S ){
+   S.SHUTTLE_PARAM( dtmfSequence, Sequence );
+   S.SHUTTLE_PARAM( dtmfDutyCycle, DutyCycle );
+   S.SHUTTLE_PARAM( dtmfAmplitude, Amplitude );
+   return true;
+}
 
-bool EffectDtmf::GetAutomationParameters(EffectAutomationParameters & parms)
+bool EffectDtmf::GetAutomationParameters(CommandParameters & parms)
 {
    parms.Write(KEY_Sequence, dtmfSequence);
    parms.Write(KEY_DutyCycle, dtmfDutyCycle);
@@ -239,7 +249,7 @@ bool EffectDtmf::GetAutomationParameters(EffectAutomationParameters & parms)
    return true;
 }
 
-bool EffectDtmf::SetAutomationParameters(EffectAutomationParameters & parms)
+bool EffectDtmf::SetAutomationParameters(CommandParameters & parms)
 {
    ReadAndVerifyDouble(DutyCycle);
    ReadAndVerifyDouble(Amplitude);
@@ -320,28 +330,25 @@ void EffectDtmf::PopulateOrExchange(ShuttleGui & S)
       mDtmfSequenceT = S.Id(ID_Sequence).AddTextBox(_("DTMF sequence:"), wxT(""), 10);
       mDtmfSequenceT->SetValidator(vldDtmf);
 
-      FloatingPointValidator<double> vldAmp(3, &dtmfAmplitude, NUM_VAL_NO_TRAILING_ZEROES);
+      FloatingPointValidator<double> vldAmp(3, &dtmfAmplitude, NumValidatorStyle::NO_TRAILING_ZEROES);
       vldAmp.SetRange(MIN_Amplitude, MAX_Amplitude);
       S.Id(ID_Amplitude).AddTextBox(_("Amplitude (0-1):"), wxT(""), 10)->SetValidator(vldAmp);
 
       S.AddPrompt(_("Duration:"));
       mDtmfDurationT = safenew
-         NumericTextCtrl(NumericConverter::TIME,
-                         S.GetParent(),
-                         ID_Duration,
+         NumericTextCtrl(S.GetParent(), ID_Duration,
+                         NumericConverter::TIME,
                          GetDurationFormat(),
                          GetDuration(),
                          mProjectRate,
-                         wxDefaultPosition,
-                         wxDefaultSize,
-                         true);
+                         NumericTextCtrl::Options{}
+                            .AutoPos(true));
       mDtmfDurationT->SetName(_("Duration"));
-      mDtmfDurationT->EnableMenu();
       S.AddWindow(mDtmfDurationT);
 
       S.AddFixedText(_("Tone/silence ratio:"), false);
       S.SetStyle(wxSL_HORIZONTAL | wxEXPAND);
-      mDtmfDutyCycleS = S.Id(ID_DutyCycle).AddSlider(wxT(""),
+      mDtmfDutyCycleS = S.Id(ID_DutyCycle).AddSlider( {},
                                                      dtmfDutyCycle * SCL_DutyCycle,
                                                      MAX_DutyCycle * SCL_DutyCycle, 
                                                      MIN_DutyCycle * SCL_DutyCycle);
@@ -404,7 +411,7 @@ void EffectDtmf::Recalculate()
 {
    // remember that dtmfDutyCycle is in range (0.0-100.0)
 
-   dtmfNTones = (int) dtmfSequence.Length();
+   dtmfNTones = (int) dtmfSequence.length();
 
    if (dtmfNTones==0) {
       // no tones, all zero: don't do anything
@@ -576,10 +583,10 @@ void EffectDtmf::UpdateUI(void)
    mDtmfDutyT->SetLabel(wxString::Format(wxT("%.1f %%"), dtmfDutyCycle));
    mDtmfDutyT->SetName(mDtmfDutyT->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
 
-   mDtmfSilenceT->SetLabel(wxString::Format(wxString(wxT("%.0f ")) + _("ms"), dtmfTone * 1000.0));
+   mDtmfSilenceT->SetLabel(wxString::Format(_("%.0f ms"), dtmfTone * 1000.0));
    mDtmfSilenceT->SetName(mDtmfSilenceT->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
 
-   mDtmfToneT->SetLabel(wxString::Format(wxString(wxT("%0.f ")) + _("ms"), dtmfSilence * 1000.0));
+   mDtmfToneT->SetLabel(wxString::Format(_("%.0f ms"), dtmfSilence * 1000.0));
    mDtmfToneT->SetName(mDtmfToneT->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
 }
 
