@@ -13,18 +13,16 @@
 #define __AUDACITY_EFFECT__
 
 #include "../Audacity.h"
+
+#include "../Experimental.h"
+
 #include "../MemoryX.h"
 #include <set>
 
 #include "../MemoryX.h"
-#include <wx/bmpbuttn.h>
 #include <wx/defs.h>
-#include <wx/dynarray.h>
-#include <wx/intl.h>
-#include <wx/string.h>
-#include <wx/tglbtn.h>
-#include <wx/event.h> // for idle event.
 
+class wxButton;
 class wxCheckBox;
 class wxChoice;
 class wxListBox;
@@ -33,21 +31,23 @@ class wxWindow;
 #include "audacity/ConfigInterface.h"
 #include "audacity/EffectInterface.h"
 
-#include "../Experimental.h"
 #include "../SampleFormat.h"
 #include "../SelectedRegion.h"
-#include "../Shuttle.h"
 #include "../Internat.h"
-#include "../widgets/ProgressDialog.h"
 
 #include "../Track.h"
 
+#include "../widgets/wxPanelWrapper.h" // to inherit
+
+class wxArrayString;
 class ShuttleGui;
+class AudacityCommand;
 
 #define BUILTIN_EFFECT_PREFIX wxT("Built-in Effect: ")
 
 class AudacityProject;
 class LabelTrack;
+class ProgressDialog;
 class SelectedRegion;
 class EffectUIHost;
 class Track;
@@ -76,19 +76,20 @@ class AUDACITY_DLL_API Effect /* not final */ : public wxEvtHandler,
    Effect();
    virtual ~Effect();
 
-   // IdentInterface implementation
+   // ComponentInterface implementation
 
-   wxString GetPath() override;
-   wxString GetSymbol() override;
-   wxString GetName() override;
-   wxString GetVendor() override;
+   PluginPath GetPath() override;
+
+   ComponentInterfaceSymbol GetSymbol() override;
+
+   VendorSymbol GetVendor() override;
    wxString GetVersion() override;
    wxString GetDescription() override;
 
-   // EffectIdentInterface implementation
+   // EffectDefinitionInterface implementation
 
    EffectType GetType() override;
-   wxString GetFamily() override; // returns UNTRANSLATED
+   EffectFamilySymbol GetFamily() override;
    bool IsInteractive() override;
    bool IsDefault() override;
    bool IsLegacy() override;
@@ -130,13 +131,13 @@ class AUDACITY_DLL_API Effect /* not final */ : public wxEvtHandler,
 
    bool ShowInterface(wxWindow *parent, bool forceModal = false) override;
 
-   bool GetAutomationParameters(EffectAutomationParameters & parms) override;
-   bool SetAutomationParameters(EffectAutomationParameters & parms) override;
+   bool GetAutomationParameters(CommandParameters & parms) override;
+   bool SetAutomationParameters(CommandParameters & parms) override;
 
-   bool LoadUserPreset(const wxString & name) override;
-   bool SaveUserPreset(const wxString & name) override;
+   bool LoadUserPreset(const RegistryPath & name) override;
+   bool SaveUserPreset(const RegistryPath & name) override;
 
-   wxArrayString GetFactoryPresets() override;
+   RegistryPaths GetFactoryPresets() override;
    bool LoadFactoryPreset(int id) override;
    bool LoadFactoryDefaults() override;
 
@@ -160,8 +161,8 @@ class AUDACITY_DLL_API Effect /* not final */ : public wxEvtHandler,
 
    double GetDefaultDuration() override;
    double GetDuration() override;
-   wxString GetDurationFormat() override;
-   virtual wxString GetSelectionFormat() /* not override? */; // time format in Selection toolbar
+   NumericFormatSymbol GetDurationFormat() override;
+   virtual NumericFormatSymbol GetSelectionFormat() /* not override? */; // time format in Selection toolbar
    void SetDuration(double duration) override;
 
    bool Apply() override;
@@ -169,48 +170,48 @@ class AUDACITY_DLL_API Effect /* not final */ : public wxEvtHandler,
 
    wxDialog *CreateUI(wxWindow *parent, EffectUIClientInterface *client) override;
 
-   wxString GetUserPresetsGroup(const wxString & name) override;
-   wxString GetCurrentSettingsGroup() override;
-   wxString GetFactoryDefaultsGroup() override;
+   RegistryPath GetUserPresetsGroup(const RegistryPath & name) override;
+   RegistryPath GetCurrentSettingsGroup() override;
+   RegistryPath GetFactoryDefaultsGroup() override;
    virtual wxString GetSavedStateGroup() /* not override? */;
 
    // ConfigClientInterface implementation
 
-   bool HasSharedConfigGroup(const wxString & group) override;
-   bool GetSharedConfigSubgroups(const wxString & group, wxArrayString & subgroups) override;
+   bool HasSharedConfigGroup(const RegistryPath & group) override;
+   bool GetSharedConfigSubgroups(const RegistryPath & group, RegistryPaths &subgroups) override;
 
-   bool GetSharedConfig(const wxString & group, const wxString & key, wxString & value, const wxString & defval = wxEmptyString) override;
-   bool GetSharedConfig(const wxString & group, const wxString & key, int & value, int defval = 0) override;
-   bool GetSharedConfig(const wxString & group, const wxString & key, bool & value, bool defval = false) override;
-   bool GetSharedConfig(const wxString & group, const wxString & key, float & value, float defval = 0.0) override;
-   bool GetSharedConfig(const wxString & group, const wxString & key, double & value, double defval = 0.0) override;
+   bool GetSharedConfig(const RegistryPath & group, const RegistryPath & key, wxString & value, const wxString & defval = {}) override;
+   bool GetSharedConfig(const RegistryPath & group, const RegistryPath & key, int & value, int defval = 0) override;
+   bool GetSharedConfig(const RegistryPath & group, const RegistryPath & key, bool & value, bool defval = false) override;
+   bool GetSharedConfig(const RegistryPath & group, const RegistryPath & key, float & value, float defval = 0.0) override;
+   bool GetSharedConfig(const RegistryPath & group, const RegistryPath & key, double & value, double defval = 0.0) override;
 
-   bool SetSharedConfig(const wxString & group, const wxString & key, const wxString & value) override;
-   bool SetSharedConfig(const wxString & group, const wxString & key, const int & value) override;
-   bool SetSharedConfig(const wxString & group, const wxString & key, const bool & value) override;
-   bool SetSharedConfig(const wxString & group, const wxString & key, const float & value) override;
-   bool SetSharedConfig(const wxString & group, const wxString & key, const double & value) override;
+   bool SetSharedConfig(const RegistryPath & group, const RegistryPath & key, const wxString & value) override;
+   bool SetSharedConfig(const RegistryPath & group, const RegistryPath & key, const int & value) override;
+   bool SetSharedConfig(const RegistryPath & group, const RegistryPath & key, const bool & value) override;
+   bool SetSharedConfig(const RegistryPath & group, const RegistryPath & key, const float & value) override;
+   bool SetSharedConfig(const RegistryPath & group, const RegistryPath & key, const double & value) override;
 
-   bool RemoveSharedConfigSubgroup(const wxString & group) override;
-   bool RemoveSharedConfig(const wxString & group, const wxString & key) override;
+   bool RemoveSharedConfigSubgroup(const RegistryPath & group) override;
+   bool RemoveSharedConfig(const RegistryPath & group, const RegistryPath & key) override;
 
-   bool HasPrivateConfigGroup(const wxString & group) override;
-   bool GetPrivateConfigSubgroups(const wxString & group, wxArrayString & subgroups) override;
+   bool HasPrivateConfigGroup(const RegistryPath & group) override;
+   bool GetPrivateConfigSubgroups(const RegistryPath & group, RegistryPaths &paths) override;
 
-   bool GetPrivateConfig(const wxString & group, const wxString & key, wxString & value, const wxString & defval = wxEmptyString) override;
-   bool GetPrivateConfig(const wxString & group, const wxString & key, int & value, int defval = 0) override;
-   bool GetPrivateConfig(const wxString & group, const wxString & key, bool & value, bool defval = false) override;
-   bool GetPrivateConfig(const wxString & group, const wxString & key, float & value, float defval = 0.0) override;
-   bool GetPrivateConfig(const wxString & group, const wxString & key, double & value, double defval = 0.0) override;
+   bool GetPrivateConfig(const RegistryPath & group, const RegistryPath & key, wxString & value, const wxString & defval = {}) override;
+   bool GetPrivateConfig(const RegistryPath & group, const RegistryPath & key, int & value, int defval = 0) override;
+   bool GetPrivateConfig(const RegistryPath & group, const RegistryPath & key, bool & value, bool defval = false) override;
+   bool GetPrivateConfig(const RegistryPath & group, const RegistryPath & key, float & value, float defval = 0.0) override;
+   bool GetPrivateConfig(const RegistryPath & group, const RegistryPath & key, double & value, double defval = 0.0) override;
 
-   bool SetPrivateConfig(const wxString & group, const wxString & key, const wxString & value) override;
-   bool SetPrivateConfig(const wxString & group, const wxString & key, const int & value) override;
-   bool SetPrivateConfig(const wxString & group, const wxString & key, const bool & value) override;
-   bool SetPrivateConfig(const wxString & group, const wxString & key, const float & value) override;
-   bool SetPrivateConfig(const wxString & group, const wxString & key, const double & value) override;
+   bool SetPrivateConfig(const RegistryPath & group, const RegistryPath & key, const wxString & value) override;
+   bool SetPrivateConfig(const RegistryPath & group, const RegistryPath & key, const int & value) override;
+   bool SetPrivateConfig(const RegistryPath & group, const RegistryPath & key, const bool & value) override;
+   bool SetPrivateConfig(const RegistryPath & group, const RegistryPath & key, const float & value) override;
+   bool SetPrivateConfig(const RegistryPath & group, const RegistryPath & key, const double & value) override;
 
-   bool RemovePrivateConfigSubgroup(const wxString & group) override;
-   bool RemovePrivateConfig(const wxString & group, const wxString & key) override;
+   bool RemovePrivateConfigSubgroup(const RegistryPath & group) override;
+   bool RemovePrivateConfig(const RegistryPath & group, const RegistryPath & key) override;
 
    // Effect implementation
 
@@ -223,7 +224,7 @@ class AUDACITY_DLL_API Effect /* not final */ : public wxEvtHandler,
    virtual bool GetAutomationParameters(wxString & parms);
    virtual bool SetAutomationParameters(const wxString & parms);
 
-   virtual wxArrayString GetUserPresets();
+   virtual RegistryPaths GetUserPresets();
    virtual bool HasCurrentSettings();
    virtual bool HasFactoryDefaults();
    virtual wxString GetPreset(wxWindow * parent, const wxString & parms);
@@ -248,8 +249,7 @@ class AUDACITY_DLL_API Effect /* not final */ : public wxEvtHandler,
                  TrackFactory *factory, SelectedRegion *selectedRegion,
                  bool shouldPrompt = true);
 
-   bool Delegate( Effect &delegate,
-      wxWindow *parent, SelectedRegion *selectedRegion, bool shouldPrompt);
+   bool Delegate( Effect &delegate, wxWindow *parent, bool shouldPrompt);
 
    // Realtime Effect Processing
    /* not virtual */ bool RealtimeAddProcessor(int group, unsigned chans, float rate);
@@ -270,6 +270,8 @@ class AUDACITY_DLL_API Effect /* not final */ : public wxEvtHandler,
                   long style = DefaultMessageBoxStyle,
                   const wxString& titleStr = wxString{});
 
+   static void IncEffectCounter(){ nEffectsDone++;};
+
 //
 // protected virtual methods
 //
@@ -330,18 +332,17 @@ protected:
    // is okay, but don't try to undo).
 
    // Pass a fraction between 0.0 and 1.0
-   bool TotalProgress(double frac);
+   bool TotalProgress(double frac, const wxString & = {});
 
    // Pass a fraction between 0.0 and 1.0, for the current track
    // (when doing one track at a time)
-   bool TrackProgress(int whichTrack, double frac, const wxString & = wxEmptyString);
+   bool TrackProgress(int whichTrack, double frac, const wxString & = {});
 
    // Pass a fraction between 0.0 and 1.0, for the current track group
    // (when doing stereo groups at a time)
-   bool TrackGroupProgress(int whichGroup, double frac, const wxString & = wxEmptyString);
+   bool TrackGroupProgress(int whichGroup, double frac, const wxString & = {});
 
    int GetNumWaveTracks() { return mNumTracks; }
-
    int GetNumWaveGroups() { return mNumGroups; }
 
    // Calculates the start time and selection length in samples
@@ -365,10 +366,13 @@ protected:
    // preview copies of all wave tracks.
    void IncludeNotSelectedPreviewTracks(bool includeNotSelected);
 
-   // Use these two methods to copy the input tracks to mOutputTracks, if
+   // Use this method to copy the input tracks to mOutputTracks, if
    // doing the processing on them, and replacing the originals only on success (and not cancel).
-   void CopyInputTracks(); // trackType = Track::Wave
-   void CopyInputTracks(int trackType);
+   // If not all sync-locked selected, then only selected wave tracks.
+   void CopyInputTracks(bool allSyncLockSelected = false);
+
+   // A global counter of all the successful Effect invocations.
+   static int nEffectsDone;
 
    // For the use of analyzers, which don't need to make output wave tracks,
    // but may need to add label tracks.
@@ -439,7 +443,7 @@ protected:
    void ReplaceProcessedTracks(const bool bGoodResult);
 
    // Use this to append a NEW output track.
-   Track *AddToOutputTracks(std::unique_ptr<Track> &&t);
+   Track *AddToOutputTracks(const std::shared_ptr<Track> &t);
 
 //
 // protected data
@@ -453,8 +457,9 @@ protected:
    double         mProjectRate; // Sample rate of the project - NEW tracks should
                                // be created with this rate...
    double         mSampleRate;
+   SelectedRegion *mpSelectedRegion{};
    TrackFactory   *mFactory;
-   TrackList *inputTracks() const { return mTracks; }
+   const TrackList *inputTracks() const { return mTracks; }
    std::shared_ptr<TrackList> mOutputTracks; // used only if CopyInputTracks() is called.
    double         mT0;
    double         mT1;
@@ -473,9 +478,6 @@ protected:
 
    sampleCount    mSampleCnt;
 
-   // type of the tracks on mOutputTracks
-   int            mOutputTracksType;
-
  // Used only by the base Effect class
  //
  private:
@@ -510,7 +512,7 @@ private:
 
    bool mIsSelection;
    double mDuration;
-   wxString mDurationFormat;
+   NumericFormatSymbol mDurationFormat;
 
    bool mIsPreview;
 
@@ -531,7 +533,7 @@ private:
    size_t mBlockSize;
    unsigned mNumChannels;
 
-   wxArrayInt mGroupProcessor;
+   std::vector<int> mGroupProcessor;
    int mCurrentProcessor;
 
    wxCriticalSection mRealtimeSuspendLock;
@@ -594,6 +596,9 @@ public:
    EffectUIHost(wxWindow *parent,
                 Effect *effect,
                 EffectUIClientInterface *client);
+   EffectUIHost(wxWindow *parent,
+                AudacityCommand *command,
+                EffectUIClientInterface *client);
    virtual ~EffectUIHost();
 
    bool TransferDataToWindow() override;
@@ -630,7 +635,7 @@ private:
    void OnDefaults(wxCommandEvent & evt);
 
    void UpdateControls();
-   wxBitmap CreateBitmap(const char *xpm[], bool up, bool pusher);
+   wxBitmap CreateBitmap(const char * const xpm[], bool up, bool pusher);
    void LoadUserPresets();
 
    void InitializeRealtime();
@@ -641,9 +646,10 @@ private:
    AudacityProject *mProject;
    wxWindow *mParent;
    Effect *mEffect;
+   AudacityCommand * mCommand;
    EffectUIClientInterface *mClient;
 
-   wxArrayString mUserPresets;
+   RegistryPaths mUserPresets;
    bool mInitialized;
    bool mSupportsRealtime;
    bool mIsGUI;
@@ -701,8 +707,8 @@ private:
    wxChoice *mType;
    wxListBox *mPresets;
 
-   wxArrayString mFactoryPresets;
-   wxArrayString mUserPresets;
+   RegistryPaths mFactoryPresets;
+   RegistryPaths mUserPresets;
    wxString mSelection;
 
    DECLARE_EVENT_TABLE()
@@ -766,7 +772,7 @@ inline long TrapLong(long x, long min, long max)
    static const type SCL_ ## name = (scale);
 
 #define ReadParam(type, name) \
-   type name; \
+   type name = DEF_ ## name; \
    if (!parms.ReadAndVerify(KEY_ ## name, &name, DEF_ ## name, MIN_ ## name, MAX_ ## name)) \
       return false;
 
@@ -778,9 +784,15 @@ inline long TrapLong(long x, long min, long max)
    if (!parms.ReadAndVerify(KEY_ ## name, &name, DEF_ ## name)) \
       return false;
 
-#define ReadAndVerifyEnum(name, list) \
+#define ReadAndVerifyEnum(name, list, listSize) \
    int name; \
-   if (!parms.ReadAndVerify(KEY_ ## name, &name, DEF_ ## name, list)) \
+   if (!parms.ReadAndVerify(KEY_ ## name, &name, DEF_ ## name, list, listSize)) \
+      return false;
+
+#define ReadAndVerifyEnumWithObsoletes(name, list, listSize, obsoleteList, nObsolete) \
+   int name; \
+   if (!parms.ReadAndVerify(KEY_ ## name, &name, DEF_ ## name, \
+                            list, listSize, obsoleteList, nObsolete)) \
       return false;
 
 #define ReadAndVerifyInt(name) ReadParam(int, name)
