@@ -38,10 +38,16 @@ a graph for EffectScienFilter.
 #include <math.h>
 #include <float.h>
 
+#include <wx/setup.h> // for wxUSE_* macros
+
 #include <wx/brush.h>
+#include <wx/choice.h>
+#include <wx/dcclient.h>
 #include <wx/dcmemory.h>
 #include <wx/intl.h>
 #include <wx/settings.h>
+#include <wx/slider.h>
+#include <wx/stattext.h>
 #include <wx/utils.h>
 #include <wx/valgen.h>
 
@@ -50,13 +56,14 @@ a graph for EffectScienFilter.
 #include "../PlatformCompatibility.h"
 #include "../Prefs.h"
 #include "../Project.h"
+#include "../Shuttle.h"
 #include "../ShuttleGui.h"
 #include "../Theme.h"
 #include "../WaveTrack.h"
 #include "../widgets/valnum.h"
 #include "../widgets/ErrorDialog.h"
-
-#include "Equalization.h" // For SliderAx
+#include "../widgets/Ruler.h"
+#include "../widgets/WindowAccessible.h"
 
 #if !defined(M_PI)
 #define PI = 3.1415926535897932384626433832795
@@ -83,37 +90,38 @@ enum kTypes
    kButterworth,
    kChebyshevTypeI,
    kChebyshevTypeII,
-   kNumTypes
+   nTypes
 };
 
-static const wxChar *kTypeStrings[] =
+static const EnumValueSymbol kTypeStrings[nTypes] =
 {
    /*i18n-hint: Butterworth is the name of the person after whom the filter type is named.*/
-   XO("Butterworth"),
+   { XO("Butterworth") },
    /*i18n-hint: Chebyshev is the name of the person after whom the filter type is named.*/
-   XO("Chebyshev Type I"),
+   { XO("Chebyshev Type I") },
    /*i18n-hint: Chebyshev is the name of the person after whom the filter type is named.*/
-   XO("Chebyshev Type II")
+   { XO("Chebyshev Type II") }
 };
 
 enum kSubTypes
 {
    kLowPass,
    kHighPass,
-   kNumSubTypes
+   nSubTypes
 };
 
-static const wxChar *kSubTypeStrings[] =
+static const EnumValueSymbol kSubTypeStrings[nSubTypes] =
 {
-   XO("Lowpass"),
-   XO("Highpass")
+   // These are acceptable dual purpose internal/visible names
+   { XO("Lowpass") },
+   { XO("Highpass") }
 };
 
 // Define keys, defaults, minimums, and maximums for the effect parameters
 //
 //     Name       Type     Key                     Def            Min   Max               Scale
-Param( Type,      int,     wxT("FilterType"),       kButterworth,  0,    kNumTypes - 1,    1  );
-Param( Subtype,   int,     wxT("FilterSubtype"),    kLowPass,      0,    kNumSubTypes - 1, 1  );
+Param( Type,      int,     wxT("FilterType"),       kButterworth,  0,    nTypes - 1,    1  );
+Param( Subtype,   int,     wxT("FilterSubtype"),    kLowPass,      0,    nSubTypes - 1, 1  );
 Param( Order,     int,     wxT("Order"),            1,             1,    10,               1  );
 Param( Cutoff,    float,   wxT("Cutoff"),           1000.0,        1.0,  FLT_MAX,          1  );
 Param( Passband,  float,   wxT("PassbandRipple"),   1.0,           0.0,  100.0,            1  );
@@ -177,9 +185,9 @@ EffectScienFilter::~EffectScienFilter()
 {
 }
 
-// IdentInterface implementation
+// ComponentInterface implementation
 
-wxString EffectScienFilter::GetSymbol()
+ComponentInterfaceSymbol EffectScienFilter::GetSymbol()
 {
    return CLASSICFILTERS_PLUGIN_SYMBOL;
 }
@@ -195,7 +203,7 @@ wxString EffectScienFilter::ManualPage()
 }
 
 
-// EffectIdentInterface implementation
+// EffectDefinitionInterface implementation
 
 EffectType EffectScienFilter::GetType()
 {
@@ -217,12 +225,7 @@ unsigned EffectScienFilter::GetAudioOutCount()
 bool EffectScienFilter::ProcessInitialize(sampleCount WXUNUSED(totalLen), ChannelNames WXUNUSED(chanMap))
 {
    for (int iPair = 0; iPair < (mOrder + 1) / 2; iPair++)
-   {
-      mpBiquad[iPair].fPrevIn = 0;
-      mpBiquad[iPair].fPrevPrevIn = 0;
-      mpBiquad[iPair].fPrevOut = 0;
-      mpBiquad[iPair].fPrevPrevOut = 0;
-   }
+      mpBiquad[iPair].Reset();
 
    return true;
 }
@@ -234,17 +237,26 @@ size_t EffectScienFilter::ProcessBlock(float **inBlock, float **outBlock, size_t
    {
       mpBiquad[iPair].pfIn = ibuf;
       mpBiquad[iPair].pfOut = outBlock[0];
-      Biquad_Process(&mpBiquad[iPair], blockLen);
+      mpBiquad[iPair].Process(blockLen);
       ibuf = outBlock[0];
    }
 
    return blockLen;
 }
+bool EffectScienFilter::DefineParams( ShuttleParams & S ){
+   S.SHUTTLE_ENUM_PARAM( mFilterType, Type, kTypeStrings, nTypes );
+   S.SHUTTLE_ENUM_PARAM( mFilterSubtype, Subtype, kSubTypeStrings, nSubTypes );
+   S.SHUTTLE_PARAM( mOrder, Order );
+   S.SHUTTLE_PARAM( mCutoff, Cutoff );
+   S.SHUTTLE_PARAM( mRipple, Passband );
+   S.SHUTTLE_PARAM( mStopbandRipple, Stopband );
+   return true;
+}
 
-bool EffectScienFilter::GetAutomationParameters(EffectAutomationParameters & parms)
+bool EffectScienFilter::GetAutomationParameters(CommandParameters & parms)
 {
-   parms.Write(KEY_Type, kTypeStrings[mFilterType]);
-   parms.Write(KEY_Subtype, kSubTypeStrings[mFilterSubtype]);
+   parms.Write(KEY_Type, kTypeStrings[mFilterType].Internal());
+   parms.Write(KEY_Subtype, kSubTypeStrings[mFilterSubtype].Internal());
    parms.Write(KEY_Order, mOrder);
    parms.WriteFloat(KEY_Cutoff, mCutoff);
    parms.WriteFloat(KEY_Passband, mRipple);
@@ -253,10 +265,10 @@ bool EffectScienFilter::GetAutomationParameters(EffectAutomationParameters & par
    return true;
 }
 
-bool EffectScienFilter::SetAutomationParameters(EffectAutomationParameters & parms)
+bool EffectScienFilter::SetAutomationParameters(CommandParameters & parms)
 {
-   ReadAndVerifyEnum(Type, wxArrayString(kNumTypes, kTypeStrings));
-   ReadAndVerifyEnum(Subtype, wxArrayString(kNumSubTypes, kSubTypeStrings));
+   ReadAndVerifyEnum(Type, kTypeStrings, nTypes);
+   ReadAndVerifyEnum(Subtype, kSubTypeStrings, nSubTypes);
    ReadAndVerifyInt(Order);
    ReadAndVerifyFloat(Cutoff);
    ReadAndVerifyFloat(Passband);
@@ -331,30 +343,28 @@ bool EffectScienFilter::Init()
    int selcount = 0;
    double rate = 0.0;
 
-   TrackListOfKindIterator iter(Track::Wave, inputTracks());
-   WaveTrack *t = (WaveTrack *) iter.First();
+   auto trackRange = inputTracks()->Selected< const WaveTrack >();
 
-   mNyquist = (t ? t->GetRate() : GetActiveProject()->GetRate()) / 2.0;
-
-   while (t)
    {
-      if (t->GetSelected())
+      auto t = *trackRange.begin();
+      mNyquist = (t ? t->GetRate() : GetActiveProject()->GetRate()) / 2.0;
+   }
+
+   for (auto t : trackRange)
+   {
+      if (selcount == 0)
       {
-         if (selcount == 0)
-         {
-            rate = t->GetRate();
-         }
-         else
-         {
-            if (t->GetRate() != rate)
-            {
-               Effect::MessageBox(_("To apply a filter, all selected tracks must have the same sample rate."));
-               return false;
-            }
-         }
-         selcount++;
+         rate = t->GetRate();
       }
-      t = (WaveTrack *) iter.Next();
+      else
+      {
+         if (t->GetRate() != rate)
+         {
+            Effect::MessageBox(_("To apply a filter, all selected tracks must have the same sample rate."));
+            return false;
+         }
+      }
+      selcount++;
    }
 
    return true;
@@ -377,16 +387,15 @@ void EffectScienFilter::PopulateOrExchange(ShuttleGui & S)
 
       S.StartVerticalLay();
       {
-         mdBRuler = safenew RulerPanel(parent, wxID_ANY);
-         mdBRuler->ruler.SetBounds(0, 0, 100, 100); // Ruler can't handle small sizes
-         mdBRuler->ruler.SetOrientation(wxVERTICAL);
-         mdBRuler->ruler.SetRange(30.0, -120.0);
-         mdBRuler->ruler.SetFormat(Ruler::LinearDBFormat);
-         mdBRuler->ruler.SetUnits(_("dB"));
-         mdBRuler->ruler.SetLabelEdges(true);
-         int w;
-         mdBRuler->ruler.GetMaxSize(&w, NULL);
-         mdBRuler->SetSize(wxSize(w, 150));  // height needed for wxGTK
+         mdBRuler = safenew RulerPanel(
+            parent, wxID_ANY, wxVERTICAL,
+            wxSize{ 100, 100 }, // Ruler can't handle small sizes
+            RulerPanel::Range{ 30.0, -120.0 },
+            Ruler::LinearDBFormat,
+            _("dB"),
+            RulerPanel::Options{}
+               .LabelEdges(true)
+         );
 
          S.SetBorder(1);
          S.AddSpace(1, 1);
@@ -396,8 +405,10 @@ void EffectScienFilter::PopulateOrExchange(ShuttleGui & S)
       }
       S.EndVerticalLay();
 
-      mPanel = safenew EffectScienFilterPanel(this, parent);
-      mPanel->SetFreqRange(mLoFreq, mNyquist);
+      mPanel = safenew EffectScienFilterPanel(
+         parent, wxID_ANY,
+         this, mLoFreq, mNyquist
+      );
 
       S.SetBorder(5);
       S.Prop(1);
@@ -430,18 +441,17 @@ void EffectScienFilter::PopulateOrExchange(ShuttleGui & S)
 
       S.AddSpace(1, 1);
 
-      mfreqRuler  = safenew RulerPanel(parent, wxID_ANY);
-      mfreqRuler->ruler.SetBounds(0, 0, 100, 100); // Ruler can't handle small sizes
-      mfreqRuler->ruler.SetOrientation(wxHORIZONTAL);
-      mfreqRuler->ruler.SetLog(true);
-      mfreqRuler->ruler.SetRange(mLoFreq, mNyquist);
-      mfreqRuler->ruler.SetFormat(Ruler::IntFormat);
-      mfreqRuler->ruler.SetUnits(wxT(""));
-      mfreqRuler->ruler.SetFlip(true);
-      mfreqRuler->ruler.SetLabelEdges(true);
-      int h;
-      mfreqRuler->ruler.GetMaxSize(NULL, &h);
-      mfreqRuler->SetMinSize(wxSize(-1, h));
+      mfreqRuler  = safenew RulerPanel(
+         parent, wxID_ANY, wxHORIZONTAL,
+         wxSize{ 100, 100 }, // Ruler can't handle small sizes
+         RulerPanel::Range{ mLoFreq, mNyquist },
+         Ruler::IntFormat,
+         wxT(""),
+         RulerPanel::Options{}
+            .Log(true)
+            .Flip(true)
+            .LabelEdges(true)
+      );
 
       S.Prop(1);
       S.AddWindow(mfreqRuler, wxEXPAND | wxALIGN_LEFT | wxRIGHT);
@@ -456,25 +466,21 @@ void EffectScienFilter::PopulateOrExchange(ShuttleGui & S)
       S.SetSizerProportion(0);
       S.StartMultiColumn(8, wxALIGN_CENTER);
       {
-         wxASSERT(kNumTypes == WXSIZEOF(kTypeStrings));
+         wxASSERT(nTypes == WXSIZEOF(kTypeStrings));
 
-         wxArrayString typeChoices;
-         for (int i = 0; i < kNumTypes; i++)
-         {
-            typeChoices.Add(wxGetTranslation(kTypeStrings[i]));
-         }
-
-         mFilterTypeCtl = S.Id(ID_Type).AddChoice(_("&Filter Type:"), wxT(""), &typeChoices);
+         auto typeChoices = LocalizedStrings(kTypeStrings, nTypes);
+         mFilterTypeCtl = S.Id(ID_Type)
+            .AddChoice(_("&Filter Type:"), typeChoices);
          mFilterTypeCtl->SetValidator(wxGenericValidator(&mFilterType));
          S.SetSizeHints(-1, -1);
 
-         wxArrayString orders;
+         wxArrayStringEx orders;
          for (int i = 1; i <= 10; i++)
          {
-            orders.Add(wxString::Format(wxT("%d"), i));
+            orders.push_back(wxString::Format(wxT("%d"), i));
          }
          /*i18n-hint: 'Order' means the complexity of the filter, and is a number between 1 and 10.*/
-         mFilterOrderCtl = S.Id(ID_Order).AddChoice(_("O&rder:"), wxT(""), &orders);
+         mFilterOrderCtl = S.Id(ID_Order).AddChoice(_("O&rder:"), orders);
          mFilterOrderCtl->SetValidator(wxGenericValidator(&mOrderIndex));
          S.SetSizeHints(-1, -1);
          S.AddSpace(1, 1);
@@ -488,15 +494,11 @@ void EffectScienFilter::PopulateOrExchange(ShuttleGui & S)
          mRippleCtl->SetValidator(vldRipple);
          mRippleCtlU = S.AddVariableText(_("dB"), false, wxALL | wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL);
 
-         wxASSERT(kNumSubTypes == WXSIZEOF(kSubTypeStrings));
+         wxASSERT(nSubTypes == WXSIZEOF(kSubTypeStrings));
 
-         wxArrayString subTypeChoices;
-         for (int i = 0; i < kNumSubTypes; i++)
-         {
-            subTypeChoices.Add(wxGetTranslation(kSubTypeStrings[i]));
-         }
-
-         mFilterSubTypeCtl = S.Id(ID_SubType).AddChoice(_("&Subtype:"), wxT(""), &subTypeChoices);
+         auto subTypeChoices = LocalizedStrings(kSubTypeStrings, nSubTypes);
+         mFilterSubTypeCtl = S.Id(ID_SubType)
+            .AddChoice(_("&Subtype:"), subTypeChoices);
          mFilterSubTypeCtl->SetValidator(wxGenericValidator(&mFilterSubtype));
          S.SetSizeHints(-1, -1);
       
@@ -671,8 +673,8 @@ bool EffectScienFilter::CalcFilter()
             fDCPoleDistSqr = fZPoleX + 1;    // dist from Nyquist
          for (int iPair = 1; iPair <= mOrder/2; iPair++)
          {
-            float fSPoleX = fC * cos (PI - iPair * PI / mOrder);
-            float fSPoleY = fC * sin (PI - iPair * PI / mOrder);
+            fSPoleX = fC * cos (PI - iPair * PI / mOrder);
+            fSPoleY = fC * sin (PI - iPair * PI / mOrder);
             BilinTransform (fSPoleX, fSPoleY, &fZPoleX, &fZPoleY);
             mpBiquad[iPair].fNumerCoeffs [0] = 1;
             if (mFilterSubtype == kLowPass)		// LOWPASS
@@ -1012,8 +1014,10 @@ BEGIN_EVENT_TABLE(EffectScienFilterPanel, wxPanelWrapper)
     EVT_SIZE(EffectScienFilterPanel::OnSize)
 END_EVENT_TABLE()
 
-EffectScienFilterPanel::EffectScienFilterPanel(EffectScienFilter *effect, wxWindow *parent)
-:  wxPanelWrapper(parent, wxID_ANY, wxDefaultPosition, wxSize(400, 200))
+EffectScienFilterPanel::EffectScienFilterPanel(
+   wxWindow *parent, wxWindowID winid,
+   EffectScienFilter *effect, double lo, double hi)
+:  wxPanelWrapper(parent, winid, wxDefaultPosition, wxSize(400, 200))
 {
    mEffect = effect;
    mParent = parent;
@@ -1025,6 +1029,8 @@ EffectScienFilterPanel::EffectScienFilterPanel(EffectScienFilter *effect, wxWind
    mHiFreq = 0.0;
    mDbMin = 0.0;
    mDbMax = 0.0;
+
+   SetFreqRange(lo, hi);
 }
 
 EffectScienFilterPanel::~EffectScienFilterPanel()
@@ -1070,7 +1076,7 @@ void EffectScienFilterPanel::OnPaint(wxPaintEvent & WXUNUSED(evt))
    {
       mWidth = width;
       mHeight = height;
-      mBitmap = std::make_unique<wxBitmap>(mWidth, mHeight);
+      mBitmap = std::make_unique<wxBitmap>(mWidth, mHeight,24);
    }
 
    wxBrush bkgndBrush(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
@@ -1104,7 +1110,7 @@ void EffectScienFilterPanel::OnPaint(wxPaintEvent & WXUNUSED(evt))
    mEnvRect.Deflate(2, 2);
 
    // Pure blue x-axis line
-   memDC.SetPen(wxPen(theTheme.Colour(clrGraphLines), 1, wxSOLID));
+   memDC.SetPen(wxPen(theTheme.Colour(clrGraphLines), 1, wxPENSTYLE_SOLID));
    int center = (int) (mEnvRect.height * mDbMax / (mDbMax - mDbMin) + 0.5);
    AColor::Line(memDC,
                 mEnvRect.GetLeft(), mEnvRect.y + center,
@@ -1112,7 +1118,7 @@ void EffectScienFilterPanel::OnPaint(wxPaintEvent & WXUNUSED(evt))
 
    //Now draw the actual response that you will get.
    //mFilterFunc has a linear scale, window has a log one so we have to fiddle about
-   memDC.SetPen(wxPen(theTheme.Colour(clrResponseLines), 3, wxSOLID));
+   memDC.SetPen(wxPen(theTheme.Colour(clrResponseLines), 3, wxPENSTYLE_SOLID));
    double scale = (double) mEnvRect.height / (mDbMax - mDbMin);    // pixels per dB
    double yF;                                                     // gain at this freq
 
