@@ -24,17 +24,15 @@
 // WARNING:  This is NOT 64-bit safe
 // *******************************************************************
 
-#include "../../Audacity.h"
+#include "../../Audacity.h" // for USE_* macros
+#include "VSTEffect.h"
 
 #if 0
 #if defined(BUILDING_AUDACITY)
-#include "../../Audacity.h"
 #include "../../PlatformCompatibility.h"
 
 // Make the main function private
-#define MODULEMAIN_SCOPE static
 #else
-#define MODULEMAIN_SCOPE
 #define USE_VST 1
 #endif
 #endif
@@ -44,6 +42,7 @@
 #include <limits.h>
 #include <stdio.h>
 
+#include <wx/setup.h> // for wxUSE_* macros
 #include <wx/dynlib.h>
 #include <wx/app.h>
 #include <wx/defs.h>
@@ -54,7 +53,6 @@
 #include <wx/dcclient.h>
 #include <wx/file.h>
 #include <wx/filename.h>
-#include <wx/frame.h>
 #include <wx/imaglist.h>
 #include <wx/listctrl.h>
 #include <wx/log.h>
@@ -94,10 +92,41 @@
 #include "../../xml/XMLFileReader.h"
 #include "../../xml/XMLWriter.h"
 
+#if wxUSE_ACCESSIBILITY
+#include "../../widgets/WindowAccessible.h"
+#endif
+
 #include "audacity/ConfigInterface.h"
 
-#include "VSTEffect.h"
 #include "../../MemoryX.h"
+#include <cstring>
+
+// Put this inclusion last.  On Linux it makes some unfortunate pollution of
+// preprocessor macro name space that interferes with other headers.
+#if defined(__WXOSX__)
+#include "VSTControlOSX.h"
+#elif defined(__WXMSW__)
+#include "VSTControlMSW.h"
+#elif defined(__WXGTK__)
+#include "VSTControlGTK.h"
+#endif
+
+static float reinterpretAsFloat(uint32_t x)
+{
+    static_assert(sizeof(float) == sizeof(uint32_t), "Cannot reinterpret uint32_t to float since sizes are different.");
+    float f;
+    std::memcpy(&f, &x, sizeof(float));
+    return f;
+}
+
+static uint32_t reinterpretAsUint32(float f)
+{
+    static_assert(sizeof(float) == sizeof(uint32_t), "Cannot reinterpret float to uint32_t since sizes are different.");
+
+    uint32_t x;
+    std::memcpy(&x, &f, sizeof(uint32_t));
+    return x;
+}
 
 // NOTE:  To debug the subprocess, use wxLogDebug and, on Windows, Debugview
 //        from TechNet (Sysinternals).
@@ -194,7 +223,7 @@ enum InfoKeys
 ///
 ///////////////////////////////////////////////////////////////////////////////
 class VSTSubProcess final : public wxProcess,
-                      public EffectIdentInterface
+                      public EffectDefinitionInterface
 {
 public:
    VSTSubProcess()
@@ -204,24 +233,19 @@ public:
 
    // EffectClientInterface implementation
 
-   wxString GetPath() override
+   PluginPath GetPath() override
    {
       return mPath;
    }
 
-   wxString GetSymbol() override
+   ComponentInterfaceSymbol GetSymbol() override
    {
       return mName;
    }
 
-   wxString GetName() override
+   VendorSymbol GetVendor() override
    {
-      return GetSymbol();
-   }
-
-   wxString GetVendor() override
-   {
-      return mVendor;
+      return { mVendor };
    }
 
    wxString GetVersion() override
@@ -234,7 +258,7 @@ public:
       return mDescription;
    }
 
-   wxString GetFamily() override
+   EffectFamilySymbol GetFamily() override
    {
       return VSTPLUGINTYPE;
    }
@@ -301,25 +325,20 @@ VSTEffectsModule::~VSTEffectsModule()
 }
 
 // ============================================================================
-// IdentInterface implementation
+// ComponentInterface implementation
 // ============================================================================
 
-wxString VSTEffectsModule::GetPath()
+PluginPath VSTEffectsModule::GetPath()
 {
    return mPath;
 }
 
-wxString VSTEffectsModule::GetSymbol()
+ComponentInterfaceSymbol VSTEffectsModule::GetSymbol()
 {
    return XO("VST Effects");
 }
 
-wxString VSTEffectsModule::GetName()
-{
-   return GetSymbol();
-}
-
-wxString VSTEffectsModule::GetVendor()
+VendorSymbol VSTEffectsModule::GetVendor()
 {
    return XO("The Audacity Team");
 }
@@ -351,14 +370,12 @@ void VSTEffectsModule::Terminate()
    return;
 }
 
-wxArrayString VSTEffectsModule::FileExtensions()
+FileExtensions VSTEffectsModule::GetFileExtensions()
 {
-   static const wxString ext[] = { _T("vst") };
-   static const wxArrayString result{ sizeof(ext)/sizeof(*ext), ext };
-   return result;
+   return {{ _T("vst") }};
 }
 
-wxString VSTEffectsModule::InstallPath()
+FilePath VSTEffectsModule::InstallPath()
 {
    // Not yet ready for VST drag-and-drop...
    // return FileNames::PlugInDir();
@@ -372,10 +389,10 @@ bool VSTEffectsModule::AutoRegisterPlugins(PluginManagerInterface & WXUNUSED(pm)
    return true;
 }
 
-wxArrayString VSTEffectsModule::FindPluginPaths(PluginManagerInterface & pm)
+PluginPaths VSTEffectsModule::FindPluginPaths(PluginManagerInterface & pm)
 {
-   wxArrayString pathList;
-   wxArrayString files;
+   FilePaths pathList;
+   FilePaths files;
 
    // Check for the VST_PATH environment variable
    wxString vstpath = wxString::FromUTF8(getenv("VST_PATH"));
@@ -384,7 +401,7 @@ wxArrayString VSTEffectsModule::FindPluginPaths(PluginManagerInterface & pm)
       wxStringTokenizer tok(vstpath);
       while (tok.HasMoreTokens())
       {
-         pathList.Add(tok.GetNextToken());
+         pathList.push_back(tok.GetNextToken());
       }
    }
 
@@ -392,20 +409,20 @@ wxArrayString VSTEffectsModule::FindPluginPaths(PluginManagerInterface & pm)
 #define VSTPATH wxT("/Library/Audio/Plug-Ins/VST")
 
    // Look in ~/Library/Audio/Plug-Ins/VST and /Library/Audio/Plug-Ins/VST
-   pathList.Add(wxGetHomeDir() + wxFILE_SEP_PATH + VSTPATH);
-   pathList.Add(VSTPATH);
+   pathList.push_back(wxGetHomeDir() + wxFILE_SEP_PATH + VSTPATH);
+   pathList.push_back(VSTPATH);
 
    // Recursively search all paths for Info.plist files.  This will identify all
    // bundles.
    pm.FindFilesInPathList(wxT("Info.plist"), pathList, files, true);
 
    // Remove the 'Contents/Info.plist' portion of the names
-   for (size_t i = 0; i < files.GetCount(); i++)
+   for (size_t i = 0; i < files.size(); i++)
    {
       files[i] = wxPathOnly(wxPathOnly(files[i]));
       if (!files[i].EndsWith(wxT(".vst")))
       {
-         files.RemoveAt(i--);
+         files.erase( files.begin() + i-- );
       }
    }
 
@@ -429,7 +446,7 @@ wxArrayString VSTEffectsModule::FindPluginPaths(PluginManagerInterface & pm)
       tpath[len] = 0;
       dpath[0] = 0;
       ExpandEnvironmentStrings(tpath, dpath, WXSIZEOF(dpath));
-      pathList.Add(dpath);
+      pathList.push_back(dpath);
    }
 
    // Then try HKEY_LOCAL_MACHINE registry key
@@ -446,7 +463,7 @@ wxArrayString VSTEffectsModule::FindPluginPaths(PluginManagerInterface & pm)
       tpath[len] = 0;
       dpath[0] = 0;
       ExpandEnvironmentStrings(tpath, dpath, WXSIZEOF(dpath));
-      pathList.Add(dpath);
+      pathList.push_back(dpath);
    }
 
    // Add the default path last
@@ -454,7 +471,7 @@ wxArrayString VSTEffectsModule::FindPluginPaths(PluginManagerInterface & pm)
    ExpandEnvironmentStrings(wxT("%ProgramFiles%\\Steinberg\\VSTPlugins"),
                             dpath,
                             WXSIZEOF(dpath));
-   pathList.Add(dpath);
+   pathList.push_back(dpath);
 
    // Recursively scan for all DLLs
    pm.FindFilesInPathList(wxT("*.dll"), pathList, files, true);
@@ -462,15 +479,15 @@ wxArrayString VSTEffectsModule::FindPluginPaths(PluginManagerInterface & pm)
 #else
 
    // Nothing specified in the VST_PATH environment variable...provide defaults
-   if (vstpath.IsEmpty())
+   if (vstpath.empty())
    {
       // We add this "non-default" one
-      pathList.Add(wxT(LIBDIR) wxT("/vst"));
+      pathList.push_back(wxT(LIBDIR) wxT("/vst"));
 
       // These are the defaults used by other hosts
-      pathList.Add(wxT("/usr/lib/vst"));
-      pathList.Add(wxT("/usr/local/lib/vst"));
-      pathList.Add(wxGetHomeDir() + wxFILE_SEP_PATH + wxT(".vst"));
+      pathList.push_back(wxT("/usr/lib/vst"));
+      pathList.push_back(wxT("/usr/local/lib/vst"));
+      pathList.push_back(wxGetHomeDir() + wxFILE_SEP_PATH + wxT(".vst"));
    }
 
    // Recursively scan for all shared objects
@@ -478,18 +495,18 @@ wxArrayString VSTEffectsModule::FindPluginPaths(PluginManagerInterface & pm)
 
 #endif
 
-   return files;
+   return { files.begin(), files.end() };
 }
 
 unsigned VSTEffectsModule::DiscoverPluginsAtPath(
-   const wxString & path, wxString &errMsg,
+   const PluginPath & path, wxString &errMsg,
    const RegistrationCallback &callback)
 {
    bool error = false;
    unsigned nFound = 0;
    errMsg.clear();
    // TODO:  Fix this for external usage
-   const wxString &cmdpath = PlatformCompatibility::GetExecutablePath();
+   const auto &cmdpath = PlatformCompatibility::GetExecutablePath();
 
    wxString effectIDs = wxT("0;");
    wxStringTokenizer effectTzr(effectIDs, wxT(";"));
@@ -498,7 +515,6 @@ unsigned VSTEffectsModule::DiscoverPluginsAtPath(
    size_t idCnt = 0;
    size_t idNdx = 0;
 
-   bool valid = false;
    bool cont = true;
 
    while (effectTzr.HasMoreTokens() && cont)
@@ -521,7 +537,6 @@ unsigned VSTEffectsModule::DiscoverPluginsAtPath(
       {
          wxLogMessage(_("VST plugin registration failed for %s\n"), path);
          error = true;
-         valid = false;
       }
 
       wxString output;
@@ -557,7 +572,8 @@ unsigned VSTEffectsModule::DiscoverPluginsAtPath(
                if (idCnt > 3)
                {
                   progress.create( _("Scanning Shell VST"),
-                        wxString::Format(_("Registering %d of %d: %-64.64s"), 0, idCnt, proc.GetName()),
+                        wxString::Format(_("Registering %d of %d: %-64.64s"), 0, idCnt,
+                                         proc.GetSymbol().Translation()),
                         static_cast<int>(idCnt),
                         nullptr,
                         wxPD_APP_MODAL |
@@ -608,12 +624,12 @@ unsigned VSTEffectsModule::DiscoverPluginsAtPath(
             break;
 
             case kKeyInteractive:
-               proc.mInteractive = val.IsSameAs(wxT("1"));
+               proc.mInteractive = val == wxT("1");
                keycount++;
             break;
 
             case kKeyAutomatable:
-               proc.mAutomatable = val.IsSameAs(wxT("1"));
+               proc.mAutomatable = val == wxT("1");
                keycount++;
             break;
 
@@ -631,12 +647,12 @@ unsigned VSTEffectsModule::DiscoverPluginsAtPath(
                {
                   idNdx++;
                   cont = progress->Update(idNdx,
-                                          wxString::Format(_("Registering %d of %d: %-64.64s"), idNdx, idCnt, proc.GetName()));
+                     wxString::Format(_("Registering %d of %d: %-64.64s"), idNdx, idCnt,
+                        proc.GetSymbol().Translation() ));
                }
 
                if (!skip && cont)
                {
-                  valid = true;
                   if (callback)
                      callback( this, &proc );
                   ++nFound;
@@ -658,7 +674,7 @@ unsigned VSTEffectsModule::DiscoverPluginsAtPath(
    return nFound;
 }
 
-bool VSTEffectsModule::IsPluginValid(const wxString & path, bool bFast)
+bool VSTEffectsModule::IsPluginValid(const PluginPath & path, bool bFast)
 {
    if( bFast )
       return true;
@@ -666,7 +682,7 @@ bool VSTEffectsModule::IsPluginValid(const wxString & path, bool bFast)
    return wxFileName::FileExists(realPath) || wxFileName::DirExists(realPath);
 }
 
-IdentInterface *VSTEffectsModule::CreateInstance(const wxString & path)
+ComponentInterface *VSTEffectsModule::CreateInstance(const PluginPath & path)
 {
    // Acquires a resource for the application.
    // For us, the ID is simply the path to the effect
@@ -674,7 +690,7 @@ IdentInterface *VSTEffectsModule::CreateInstance(const wxString & path)
    return safenew VSTEffect(path);
 }
 
-void VSTEffectsModule::DeleteInstance(IdentInterface *instance)
+void VSTEffectsModule::DeleteInstance(ComponentInterface *instance)
 {
    std::unique_ptr < VSTEffect > {
       dynamic_cast<VSTEffect *>(instance)
@@ -697,14 +713,14 @@ void VSTEffectsModule::Check(const wxChar *path)
    VSTEffect effect(path);
    if (effect.SetHost(NULL))
    {
-      wxArrayInt effectIDs = effect.GetEffectIDs();
+      auto effectIDs = effect.GetEffectIDs();
       wxString out;
 
-      if (effectIDs.GetCount() > 0)
+      if (effectIDs.size() > 0)
       {
          wxString subids;
 
-         for (size_t i = 0, cnt = effectIDs.GetCount(); i < cnt; i++)
+         for (size_t i = 0, cnt = effectIDs.size(); i < cnt; i++)
          {
             subids += wxString::Format(wxT("%d;"), effectIDs[i]);
          }
@@ -715,8 +731,9 @@ void VSTEffectsModule::Check(const wxChar *path)
       {
          out += wxString::Format(wxT("%s%d=%s\n"), OUTPUTKEY, kKeyBegin, wxEmptyString);
          out += wxString::Format(wxT("%s%d=%s\n"), OUTPUTKEY, kKeyPath, effect.GetPath());
-         out += wxString::Format(wxT("%s%d=%s\n"), OUTPUTKEY, kKeyName, effect.GetName());
-         out += wxString::Format(wxT("%s%d=%s\n"), OUTPUTKEY, kKeyVendor, effect.GetVendor());
+         out += wxString::Format(wxT("%s%d=%s\n"), OUTPUTKEY, kKeyName, effect.GetSymbol().Internal());
+         out += wxString::Format(wxT("%s%d=%s\n"), OUTPUTKEY, kKeyVendor,
+                                 effect.GetVendor().Internal());
          out += wxString::Format(wxT("%s%d=%s\n"), OUTPUTKEY, kKeyVersion, effect.GetVersion());
          out += wxString::Format(wxT("%s%d=%s\n"), OUTPUTKEY, kKeyDescription, effect.GetDescription());
          out += wxString::Format(wxT("%s%d=%d\n"), OUTPUTKEY, kKeyEffectType, effect.GetType());
@@ -1102,16 +1119,16 @@ void VSTEffect::BundleDeleter::operator() (void* p) const
       CFRelease(static_cast<CFBundleRef>(p));
 }
 
-void VSTEffect::ResourceDeleter::operator() (void *p) const
+void VSTEffect::ResourceHandle::reset()
 {
-   if (mpHandle) {
-      int resource = (int)p;
-      CFBundleCloseBundleResourceMap(mpHandle->get(), resource);
-   }
+   if (mpHandle)
+      CFBundleCloseBundleResourceMap(mpHandle, mNum);
+   mpHandle = nullptr;
+   mNum = 0;
 }
 #endif
 
-VSTEffect::VSTEffect(const wxString & path, VSTEffect *master)
+VSTEffect::VSTEffect(const PluginPath & path, VSTEffect *master)
 :  mPath(path),
    mMaster(master)
 {
@@ -1141,7 +1158,7 @@ VSTEffect::VSTEffect(const wxString & path, VSTEffect *master)
    memset(&mTimeInfo, 0, sizeof(mTimeInfo));
    mTimeInfo.samplePos = 0.0;
    mTimeInfo.sampleRate = 44100.0;  // this is a bogus value, but it's only for the display
-   mTimeInfo.nanoSeconds = wxGetLocalTimeMillis().ToDouble();
+   mTimeInfo.nanoSeconds = wxGetUTCTimeMillis().ToDouble();
    mTimeInfo.tempo = 120.0;
    mTimeInfo.timeSigNumerator = 4;
    mTimeInfo.timeSigDenominator = 4;
@@ -1170,27 +1187,22 @@ VSTEffect::~VSTEffect()
 }
 
 // ============================================================================
-// IdentInterface Implementation
+// ComponentInterface Implementation
 // ============================================================================
 
-wxString VSTEffect::GetPath()
+PluginPath VSTEffect::GetPath()
 {
    return mPath;
 }
 
-wxString VSTEffect::GetSymbol()
+ComponentInterfaceSymbol VSTEffect::GetSymbol()
 {
    return mName;
 }
 
-wxString VSTEffect::GetName()
+VendorSymbol VSTEffect::GetVendor()
 {
-   return GetSymbol();
-}
-
-wxString VSTEffect::GetVendor()
-{
-   return mVendor;
+   return { mVendor };
 }
 
 wxString VSTEffect::GetVersion()
@@ -1222,14 +1234,14 @@ wxString VSTEffect::GetDescription()
 }
 
 // ============================================================================
-// EffectIdentInterface Implementation
+// EffectDefinitionInterface Implementation
 // ============================================================================
 
 EffectType VSTEffect::GetType()
 {
    if (mAudioIns == 0 && mAudioOuts == 0 && mMidiIns == 0 && mMidiOuts == 0)
    {
-      return EffectTypeNone;
+      return EffectTypeTool;
    }
 
    if (mAudioIns == 0 && mMidiIns == 0)
@@ -1246,7 +1258,7 @@ EffectType VSTEffect::GetType()
 }
 
 
-wxString VSTEffect::GetFamily()
+EffectFamilySymbol VSTEffect::GetFamily()
 {
    return VSTPLUGINTYPE;
 }
@@ -1383,7 +1395,7 @@ bool VSTEffect::ProcessInitialize(sampleCount WXUNUSED(totalLen), ChannelNames W
    // Initialize time info
    memset(&mTimeInfo, 0, sizeof(mTimeInfo));
    mTimeInfo.sampleRate = mSampleRate;
-   mTimeInfo.nanoSeconds = wxGetLocalTimeMillis().ToDouble();
+   mTimeInfo.nanoSeconds = wxGetUTCTimeMillis().ToDouble();
    mTimeInfo.tempo = 120.0;
    mTimeInfo.timeSigNumerator = 4;
    mTimeInfo.timeSigDenominator = 4;
@@ -1448,7 +1460,7 @@ bool VSTEffect::RealtimeInitialize()
 
 bool VSTEffect::RealtimeAddProcessor(unsigned numChannels, float sampleRate)
 {
-   mSlaves.push_back(make_movable<VSTEffect>(mPath, this));
+   mSlaves.push_back(std::make_unique<VSTEffect>(mPath, this));
    VSTEffect *const slave = mSlaves.back().get();
 
    slave->SetBlockSize(mBlockSize);
@@ -1621,12 +1633,12 @@ bool VSTEffect::ShowInterface(wxWindow *parent, bool forceModal)
    return res;
 }
 
-bool VSTEffect::GetAutomationParameters(EffectAutomationParameters & parms)
+bool VSTEffect::GetAutomationParameters(CommandParameters & parms)
 {
    for (int i = 0; i < mAEffect->numParams; i++)
    {
       wxString name = GetString(effGetParamName, i);
-      if (name.IsEmpty())
+      if (name.empty())
       {
          name.Printf(wxT("parm_%d"), i);
       }
@@ -1641,13 +1653,13 @@ bool VSTEffect::GetAutomationParameters(EffectAutomationParameters & parms)
    return true;
 }
 
-bool VSTEffect::SetAutomationParameters(EffectAutomationParameters & parms)
+bool VSTEffect::SetAutomationParameters(CommandParameters & parms)
 {
    callDispatcher(effBeginSetProgram, 0, 0, NULL, 0.0);
    for (int i = 0; i < mAEffect->numParams; i++)
    {
       wxString name = GetString(effGetParamName, i);
-      if (name.IsEmpty())
+      if (name.empty())
       {
          name.Printf(wxT("parm_%d"), i);
       }
@@ -1671,7 +1683,7 @@ bool VSTEffect::SetAutomationParameters(EffectAutomationParameters & parms)
 }
 
 
-bool VSTEffect::LoadUserPreset(const wxString & name)
+bool VSTEffect::LoadUserPreset(const RegistryPath & name)
 {
    if (!LoadParameters(name))
    {
@@ -1683,14 +1695,14 @@ bool VSTEffect::LoadUserPreset(const wxString & name)
    return true;
 }
 
-bool VSTEffect::SaveUserPreset(const wxString & name)
+bool VSTEffect::SaveUserPreset(const RegistryPath & name)
 {
    return SaveParameters(name);
 }
 
-wxArrayString VSTEffect::GetFactoryPresets()
+RegistryPaths VSTEffect::GetFactoryPresets()
 {
-   wxArrayString progs; 
+   RegistryPaths progs;
 
    // Some plugins, like Guitar Rig 5, only report 128 programs while they have hundreds.  While
    // I was able to come up with a hack in the Guitar Rig case to gather all of the program names
@@ -1699,7 +1711,7 @@ wxArrayString VSTEffect::GetFactoryPresets()
    {
       for (int i = 0; i < mAEffect->numPrograms; i++)
       {
-         progs.Add(GetString(effGetProgramNameIndexed, i));
+         progs.push_back(GetString(effGetProgramNameIndexed, i));
       }
    }
 
@@ -1847,7 +1859,7 @@ void VSTEffect::ExportPresets()
                        NULL);
 
    // User canceled...
-   if (path.IsEmpty())
+   if (path.empty())
    {
       return;
    }
@@ -1899,7 +1911,7 @@ void VSTEffect::ImportPresets()
                        mParent);
 
    // User canceled...
-   if (path.IsEmpty())
+   if (path.empty())
    {
       return;
    }
@@ -2054,10 +2066,8 @@ bool VSTEffect::Load()
    mBundleRef = std::move(bundleRef);
 
    // Open the resource map ... some plugins (like GRM Tools) need this.
-   mResource = ResourceHandle {
-      reinterpret_cast<char*>(
-         CFBundleOpenBundleResourceMap(mBundleRef.get())),
-      ResourceDeleter{&mBundleRef}
+   mResource = ResourceHandle{
+      mBundleRef.get(), CFBundleOpenBundleResourceMap(mBundleRef.get())
    };
 
 #elif defined(__WXMSW__)
@@ -2260,7 +2270,7 @@ void VSTEffect::Unload()
    if (mModule)
    {
 #if defined(__WXMAC__)
-      mResource = ResourceHandle{};
+      mResource.reset();
       mBundleRef.reset();
 #endif
 
@@ -2269,9 +2279,9 @@ void VSTEffect::Unload()
    }
 }
 
-wxArrayInt VSTEffect::GetEffectIDs()
+std::vector<int> VSTEffect::GetEffectIDs()
 {
-   wxArrayInt effectIDs;
+   std::vector<int> effectIDs;
 
    // Are we a shell?
    if (mVstVersion >= 2 && (VstPlugCategory) callDispatcher(effGetPlugCategory, 0, 0, NULL, 0) == kPlugCategShell)
@@ -2282,7 +2292,7 @@ wxArrayInt VSTEffect::GetEffectIDs()
       effectID = (int) callDispatcher(effShellGetNextPlugin, 0, 0, &name, 0);
       while (effectID)
       {
-         effectIDs.Add(effectID);
+         effectIDs.push_back(effectID);
          effectID = (int) callDispatcher(effShellGetNextPlugin, 0, 0, &name, 0);
       }
    }
@@ -2290,7 +2300,7 @@ wxArrayInt VSTEffect::GetEffectIDs()
    return effectIDs;
 }
 
-bool VSTEffect::LoadParameters(const wxString & group)
+bool VSTEffect::LoadParameters(const RegistryPath & group)
 {
    wxString value;
 
@@ -2325,7 +2335,7 @@ bool VSTEffect::LoadParameters(const wxString & group)
       return false;
    }
 
-   EffectAutomationParameters eap;
+   CommandParameters eap;
    if (!eap.SetParameters(parms))
    {
       return false;
@@ -2334,7 +2344,7 @@ bool VSTEffect::LoadParameters(const wxString & group)
    return SetAutomationParameters(eap);
 }
 
-bool VSTEffect::SaveParameters(const wxString & group)
+bool VSTEffect::SaveParameters(const RegistryPath & group)
 {
    mHost->SetPrivateConfig(group, wxT("UniqueID"), mAEffect->uniqueID);
    mHost->SetPrivateConfig(group, wxT("Version"), mAEffect->version);
@@ -2353,7 +2363,7 @@ bool VSTEffect::SaveParameters(const wxString & group)
       return true;
    }
 
-   EffectAutomationParameters eap;
+   CommandParameters eap;
    if (!GetAutomationParameters(eap))
    {
       return false;
@@ -2407,7 +2417,7 @@ void VSTEffect::NeedEditIdle(bool state)
 
 VstTimeInfo *VSTEffect::GetTimeInfo()
 {
-   mTimeInfo.nanoSeconds = wxGetLocalTimeMillis().ToDouble();
+   mTimeInfo.nanoSeconds = wxGetUTCTimeMillis().ToDouble();
    return &mTimeInfo;
 }
 
@@ -2755,6 +2765,24 @@ void VSTEffect::RemoveHandler()
 {
 }
 
+static void OnSize(wxSizeEvent & evt)
+{
+   evt.Skip();
+
+   // Once the parent dialog reaches its final size as indicated by
+   // a non-default minimum size, we set the maximum size to match.
+   // This is a bit of a hack to prevent VSTs GUI windows from resizing
+   // there's no real reason to allow it.  But, there should be a better
+   // way of handling it.
+   wxWindow *w = (wxWindow *) evt.GetEventObject();
+   wxSize sz = w->GetMinSize();
+
+   if (sz != wxDefaultSize)
+   {
+      w->SetMaxSize(sz);
+   }
+}
+
 void VSTEffect::BuildFancy()
 {
    // Turn the power on...some effects need this when the editor is open
@@ -2782,7 +2810,7 @@ void VSTEffect::BuildFancy()
 
    NeedEditIdle(true);
 
-   mDialog->Connect(wxEVT_SIZE, wxSizeEventHandler(VSTEffect::OnSize));
+   mDialog->Bind(wxEVT_SIZE, OnSize);
 
 #ifdef __WXMAC__
 #ifdef __WX_EVTLOOP_BUSY_WAITING__
@@ -2836,17 +2864,14 @@ void VSTEffect::BuildPlain()
             wxControl *item = safenew wxStaticText(scroller, 0, _("Duration:"));
             gridSizer->Add(item, 0, wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT | wxALL, 5);
             mDuration = safenew
-               NumericTextCtrl(NumericConverter::TIME,
-               scroller,
-               ID_Duration,
-               mHost->GetDurationFormat(),
-               mHost->GetDuration(),
-               mSampleRate,
-               wxDefaultPosition,
-               wxDefaultSize,
-               true);
+               NumericTextCtrl(scroller, ID_Duration,
+                  NumericConverter::TIME,
+                  mHost->GetDurationFormat(),
+                  mHost->GetDuration(),
+                  mSampleRate,
+                  NumericTextCtrl::Options{}
+                     .AutoPos(true));
             mDuration->SetName(_("Duration"));
-            mDuration->EnableMenu();
             gridSizer->Add(mDuration, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
             gridSizer->Add(1, 1, 0);
             gridSizer->Add(1, 1, 0);
@@ -2884,7 +2909,7 @@ void VSTEffect::BuildPlain()
                wxALIGN_RIGHT | wxST_NO_AUTORESIZE);
             gridSizer->Add(mNames[i], 0, wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT | wxALL, 5);
 
-            mSliders[i] = safenew wxSlider(scroller,
+            mSliders[i] = safenew wxSliderWrapper(scroller,
                ID_Sliders + i,
                0,
                0,
@@ -2892,6 +2917,10 @@ void VSTEffect::BuildPlain()
                wxDefaultPosition,
                wxSize(200, -1));
             gridSizer->Add(mSliders[i], 0, wxALIGN_CENTER_VERTICAL | wxEXPAND | wxALL, 5);
+#if wxUSE_ACCESSIBILITY
+            // so that name can be set on a standard control
+            mSliders[i]->SetAccessible(safenew WindowAccessible(mSliders[i]));
+#endif
 
             mDisplays[i] = safenew wxStaticText(scroller,
                wxID_ANY,
@@ -2952,7 +2981,7 @@ void VSTEffect::RefreshParameters(int skip)
       name = text;
 
       text = GetString(effGetParamDisplay, i);
-      if (text.IsEmpty())
+      if (text.empty())
       {
          text.Printf(wxT("%.5g"),callGetParameter(i));
       }
@@ -2960,7 +2989,7 @@ void VSTEffect::RefreshParameters(int skip)
       name += wxT(' ') + text;
 
       text = GetString(effGetParamDisplay, i);
-      if (!text.IsEmpty())
+      if (!text.empty())
       {
          text.Printf(wxT("%-8s"), GetString(effGetParamLabel, i));
          mLabels[i]->SetLabel(wxString::Format(wxT("%8s"), text));
@@ -2971,24 +3000,6 @@ void VSTEffect::RefreshParameters(int skip)
    }
 }
 
-void VSTEffect::OnSize(wxSizeEvent & evt)
-{
-   evt.Skip();
-
-   // Once the parent dialog reaches it's final size as indicated by
-   // a non-default minimum size, we set the maximum size to match.
-   // This is a bit of a hack to prevent VSTs GUI windows from resizing
-   // there's no real reason to allow it.  But, there should be a better
-   // way of handling it.
-   wxWindow *w = (wxWindow *) evt.GetEventObject();
-   wxSize sz = w->GetMinSize();
-
-   if (sz != wxDefaultSize)
-   {
-      w->SetMaxSize(sz);
-   }
-}
-
 void VSTEffect::OnSizeWindow(wxCommandEvent & evt)
 {
    if (!mControl)
@@ -3315,7 +3326,7 @@ bool VSTEffect::LoadFXProgram(unsigned char **bptr, ssize_t & len, int index, bo
       for (int i = 0; i < numParams; i++)
       {
          uint32_t ival = wxUINT32_SWAP_ON_LE(iptr[14 + i]);
-         float val = *((float *) &ival);
+         float val = reinterpretAsFloat(ival);
          if (val < 0.0 || val > 1.0)
          {
             return false;
@@ -3336,7 +3347,7 @@ bool VSTEffect::LoadFXProgram(unsigned char **bptr, ssize_t & len, int index, bo
          for (int i = 0; i < numParams; i++)
          {
             wxUint32 val = wxUINT32_SWAP_ON_LE(iptr[14 + i]);
-            callSetParameter(i, *((float *) &val));
+            callSetParameter(i, reinterpretAsFloat(val));
          }
          callDispatcher(effEndSetProgram, 0, 0, NULL, 0.0);
       }
@@ -3443,8 +3454,8 @@ void VSTEffect::SaveFXB(const wxFileName & fn)
 
    wxMemoryBuffer buf;
    wxInt32 subType;
-   void *chunkPtr;
-   int chunkSize;
+   void *chunkPtr = nullptr;
+   int chunkSize = 0;
    int dataSize = 148;
    wxInt32 tab[8];
    int curProg = 0 ; //mProgram->GetCurrentSelection();
@@ -3595,7 +3606,7 @@ void VSTEffect::SaveFXProgram(wxMemoryBuffer & buf, int index)
       for (int i = 0; i < mAEffect->numParams; i++)
       {
          float val = callGetParameter(i);
-         wxUint32 ival = wxUINT32_SWAP_ON_LE(*((wxUint32 *) &val));
+         wxUint32 ival = wxUINT32_SWAP_ON_LE(reinterpretAsUint32(val));
          buf.AppendData(&ival, sizeof(ival));
       }
    }
@@ -3613,7 +3624,8 @@ void VSTEffect::SaveXML(const wxFileName & fn)
    xmlFile.WriteAttr(wxT("version"), wxT("2"));
 
    xmlFile.StartTag(wxT("effect"));
-   xmlFile.WriteAttr(wxT("name"), GetName());
+   // Use internal name only in persistent information
+   xmlFile.WriteAttr(wxT("name"), GetSymbol().Internal());
    xmlFile.WriteAttr(wxT("uniqueID"), mAEffect->uniqueID);
    xmlFile.WriteAttr(wxT("version"), mAEffect->version);
    xmlFile.WriteAttr(wxT("numParams"), mAEffect->numParams);
@@ -3725,10 +3737,10 @@ bool VSTEffect::HandleXMLTag(const wxChar *tag, const wxChar **attrs)
                return false;
             }
 
-            if (value != GetName())
+            if (value != GetSymbol().Internal())
             {
                wxString msg;
-               msg.Printf(_("This parameter file was saved from %s.  Continue?"), value);
+               msg.Printf(_("This parameter file was saved from %s. Continue?"), value);
                int result = AudacityMessageBox(msg, wxT("Confirm"), wxYES_NO, mParent);
                if (result == wxNO)
                {
