@@ -14,9 +14,10 @@
 
 *//*******************************************************************/
 
-#include "../../Audacity.h"
+#include "../../Audacity.h" // for USE_* macros
 
 #if USE_AUDIO_UNITS
+#include "AudioUnitEffect.h"
 
 #include <wx/defs.h>
 #include <wx/button.h>
@@ -41,7 +42,6 @@
 #include "../../widgets/valnum.h"
 #include "../../widgets/wxPanelWrapper.h"
 
-#include "AudioUnitEffect.h"
 #include "../../Internat.h"
 
 struct CFReleaser
@@ -99,29 +99,25 @@ AudioUnitEffectsModule::AudioUnitEffectsModule(ModuleManagerInterface *moduleMan
 
 AudioUnitEffectsModule::~AudioUnitEffectsModule()
 {
-   mPath.Clear();
+   mPath.clear();
 }
 
 // ============================================================================
-// IdentInterface implementation
+// ComponentInterface implementation
 // ============================================================================
 
-wxString AudioUnitEffectsModule::GetPath()
+PluginPath AudioUnitEffectsModule::GetPath()
 {
    return mPath;
 }
 
-wxString AudioUnitEffectsModule::GetSymbol()
-{
-   return wxT("Audio Unit Effects");
-}
-
-wxString AudioUnitEffectsModule::GetName()
+ComponentInterfaceSymbol AudioUnitEffectsModule::GetSymbol()
 {
+   /* i18n-hint: Audio Unit is the name of an Apple audio software protocol */
    return XO("Audio Unit Effects");
 }
 
-wxString AudioUnitEffectsModule::GetVendor()
+VendorSymbol AudioUnitEffectsModule::GetVendor()
 {
    return XO("The Audacity Team");
 }
@@ -141,11 +137,9 @@ wxString AudioUnitEffectsModule::GetDescription()
 // ModuleInterface implementation
 // ============================================================================
 
-wxArrayString AudioUnitEffectsModule::FileExtensions()
+FileExtensions AudioUnitEffectsModule::GetFileExtensions()
 {
-   static const wxString ext[] = { _T("au") };
-   static const wxArrayString result{ sizeof(ext)/sizeof(*ext), ext };
-   return result;
+   return {{ _T("au") }};
 }
 
 bool AudioUnitEffectsModule::Initialize()
@@ -166,9 +160,9 @@ bool AudioUnitEffectsModule::AutoRegisterPlugins(PluginManagerInterface & pm)
    return true;
 }
 
-wxArrayString AudioUnitEffectsModule::FindPluginPaths(PluginManagerInterface & pm)
+PluginPaths AudioUnitEffectsModule::FindPluginPaths(PluginManagerInterface & pm)
 {
-   wxArrayString effects;
+   PluginPaths effects;
 
    LoadAudioUnitsOfType(kAudioUnitType_Effect, effects);
    LoadAudioUnitsOfType(kAudioUnitType_Generator, effects);
@@ -180,7 +174,7 @@ wxArrayString AudioUnitEffectsModule::FindPluginPaths(PluginManagerInterface & p
 }
 
 unsigned AudioUnitEffectsModule::DiscoverPluginsAtPath(
-   const wxString & path, wxString &errMsg,
+   const PluginPath & path, wxString &errMsg,
    const RegistrationCallback &callback)
 {
    errMsg.clear();
@@ -208,7 +202,7 @@ unsigned AudioUnitEffectsModule::DiscoverPluginsAtPath(
 }
 
 bool AudioUnitEffectsModule::IsPluginValid(
-   const wxString & path, bool bFast)
+   const PluginPath & path, bool bFast)
 {
    if( bFast )
       return true;
@@ -216,7 +210,7 @@ bool AudioUnitEffectsModule::IsPluginValid(
    return FindAudioUnit(path, name) != NULL;
 }
 
-IdentInterface *AudioUnitEffectsModule::CreateInstance(const wxString & path)
+ComponentInterface *AudioUnitEffectsModule::CreateInstance(const PluginPath & path)
 {
    // Acquires a resource for the application.
    wxString name;
@@ -230,7 +224,7 @@ IdentInterface *AudioUnitEffectsModule::CreateInstance(const wxString & path)
    return safenew AudioUnitEffect(path, name, component);
 }
 
-void AudioUnitEffectsModule::DeleteInstance(IdentInterface *instance)
+void AudioUnitEffectsModule::DeleteInstance(ComponentInterface *instance)
 {
    std::unique_ptr < AudioUnitEffect > {
       dynamic_cast<AudioUnitEffect *>(instance)
@@ -242,7 +236,7 @@ void AudioUnitEffectsModule::DeleteInstance(IdentInterface *instance)
 // ============================================================================
 
 void AudioUnitEffectsModule::LoadAudioUnitsOfType(OSType inAUType,
-                                                  wxArrayString & effects)
+                                                  PluginPaths & effects)
 {
    AudioComponentDescription desc;
    AudioComponent component;
@@ -270,7 +264,7 @@ void AudioUnitEffectsModule::LoadAudioUnitsOfType(OSType inAUType,
          {
             wxString name = wxCFStringRef::AsString(cfName);
       
-            effects.Add(wxString::Format(wxT("%-4.4s/%-4.4s/%-4.4s/%s"),
+            effects.push_back(wxString::Format(wxT("%-4.4s/%-4.4s/%-4.4s/%s"),
                         FromOSType(found.componentManufacturer),
                         FromOSType(found.componentType),
                         FromOSType(found.componentSubType),
@@ -282,7 +276,7 @@ void AudioUnitEffectsModule::LoadAudioUnitsOfType(OSType inAUType,
    }
 }
 
-AudioComponent AudioUnitEffectsModule::FindAudioUnit(const wxString & path,
+AudioComponent AudioUnitEffectsModule::FindAudioUnit(const PluginPath & path,
                                                      wxString & name)
 {
    wxStringTokenizer tokens(path, wxT("/"));
@@ -344,8 +338,6 @@ private:
    bool mUseLatency;
    wxString mUIType;
 
-   wxArrayString mUITypes;
-
    DECLARE_EVENT_TABLE()
 };
 
@@ -358,10 +350,6 @@ AudioUnitEffectOptionsDialog::AudioUnitEffectOptionsDialog(wxWindow * parent, Ef
 {
    mHost = host;
 
-   mUITypes.Add(_("Full"));
-   mUITypes.Add(_("Generic"));
-   mUITypes.Add(_("Basic"));
-
    mHost->GetSharedConfig(wxT("Options"), wxT("UseLatency"), mUseLatency, true);
    mHost->GetSharedConfig(wxT("Options"), wxT("UIType"), mUIType, wxT("Full"));
 
@@ -406,14 +394,14 @@ void AudioUnitEffectOptionsDialog::PopulateOrExchange(ShuttleGui & S)
             S.AddVariableText(wxString() +
                _("Select \"Full\" to use the graphical interface if supplied by the Audio Unit.") +
                _(" Select \"Generic\" to use the system supplied generic interface.") +
-               _(" Select \"Basic\" for a basic text-only interface. ") +
+               _(" Select \"Basic\" for a basic text-only interface.") +
                _(" Reopen the effect for this to take effect."))->Wrap(650);
 
             S.StartHorizontalLay(wxALIGN_LEFT);
             {
                S.TieChoice(_("Select &interface"),
                            mUIType,
-                           &mUITypes);
+                           { _("Full"), _("Generic"), _("Basic") });
             }
             S.EndHorizontalLay();
          }
@@ -526,13 +514,13 @@ void AudioUnitEffectExportDialog::PopulateOrExchange(ShuttleGui & S)
 
    S.AddStandardButtons();
 
-   wxArrayString presets;
+   RegistryPaths presets;
 
    mEffect->mHost->GetPrivateConfigSubgroups(mEffect->mHost->GetUserPresetsGroup(wxEmptyString), presets);
 
-   presets.Sort();
+   std::sort( presets.begin(), presets.end() );
 
-   for (size_t i = 0, cnt = presets.GetCount(); i < cnt; i++)
+   for (size_t i = 0, cnt = presets.size(); i < cnt; i++)
    {
       mList->InsertItem(i, presets[i]);
    }
@@ -700,7 +688,7 @@ void AudioUnitEffectImportDialog::PopulateOrExchange(ShuttleGui & S)
 
    S.AddStandardButtons();
 
-   wxArrayString presets;
+   FilePaths presets;
 
    // Make sure the user preset directory exists
    wxString path;
@@ -726,7 +714,7 @@ void AudioUnitEffectImportDialog::PopulateOrExchange(ShuttleGui & S)
    
    presets.Sort();
 
-   for (size_t i = 0, cnt = presets.GetCount(); i < cnt; i++)
+   for (size_t i = 0, cnt = presets.size(); i < cnt; i++)
    {
       fn = presets[i];
       mList->InsertItem(i, fn.GetName());
@@ -831,7 +819,7 @@ void AudioUnitEffectImportDialog::OnOk(wxCommandEvent & WXUNUSED(evt))
 //
 ///////////////////////////////////////////////////////////////////////////////
 
-AudioUnitEffect::AudioUnitEffect(const wxString & path,
+AudioUnitEffect::AudioUnitEffect(const PluginPath & path,
                                  const wxString & name,
                                  AudioComponent component,
                                  AudioUnitEffect *master)
@@ -876,27 +864,22 @@ AudioUnitEffect::~AudioUnitEffect()
 }
 
 // ============================================================================
-// IdentInterface implementation
+// ComponentInterface implementation
 // ============================================================================
 
-wxString AudioUnitEffect::GetPath()
+PluginPath AudioUnitEffect::GetPath()
 {
    return mPath;
 }
 
-wxString AudioUnitEffect::GetSymbol()
+ComponentInterfaceSymbol AudioUnitEffect::GetSymbol()
 {
    return mName;
 }
 
-wxString AudioUnitEffect::GetName()
+VendorSymbol AudioUnitEffect::GetVendor()
 {
-   return GetSymbol();
-}
-
-wxString AudioUnitEffect::GetVendor()
-{
-   return mVendor;
+   return { mVendor };
 }
 
 wxString AudioUnitEffect::GetVersion()
@@ -918,7 +901,7 @@ wxString AudioUnitEffect::GetDescription()
 }
 
 // ============================================================================
-// EffectIdentInterface implementation
+// EffectComponentInterface implementation
 // ============================================================================
 
 EffectType AudioUnitEffect::GetType()
@@ -941,7 +924,7 @@ EffectType AudioUnitEffect::GetType()
    return EffectTypeProcess;
 }
 
-wxString AudioUnitEffect::GetFamily()
+EffectFamilySymbol AudioUnitEffect::GetFamily()
 {
    return AUDIOUNITEFFECTS_FAMILY;
 }
@@ -1355,7 +1338,7 @@ bool AudioUnitEffect::RealtimeInitialize()
 
 bool AudioUnitEffect::RealtimeAddProcessor(unsigned numChannels, float sampleRate)
 {
-   auto slave = make_movable<AudioUnitEffect>(mPath, mName, mComponent, this);
+   auto slave = std::make_unique<AudioUnitEffect>(mPath, mName, mComponent, this);
    if (!slave->SetHost(NULL))
       return false;
 
@@ -1472,7 +1455,7 @@ bool AudioUnitEffect::ShowInterface(wxWindow *parent, bool forceModal)
    return res;
 }
 
-bool AudioUnitEffect::GetAutomationParameters(EffectAutomationParameters & parms)
+bool AudioUnitEffect::GetAutomationParameters(CommandParameters & parms)
 {
    OSStatus result;
    UInt32 dataSize;
@@ -1524,7 +1507,7 @@ bool AudioUnitEffect::GetAutomationParameters(EffectAutomationParameters & parms
          name = wxCFStringRef::AsString(info.cfNameString);
       }
 
-      if (name.IsEmpty())
+      if (name.empty())
       {
          continue;
       }
@@ -1545,7 +1528,7 @@ bool AudioUnitEffect::GetAutomationParameters(EffectAutomationParameters & parms
    return true;
 }
 
-bool AudioUnitEffect::SetAutomationParameters(EffectAutomationParameters & parms)
+bool AudioUnitEffect::SetAutomationParameters(CommandParameters & parms)
 {
    OSStatus result;
    UInt32 dataSize;
@@ -1597,7 +1580,7 @@ bool AudioUnitEffect::SetAutomationParameters(EffectAutomationParameters & parms
          name = wxCFStringRef::AsString(info.cfNameString);
       }
 
-      if (name.IsEmpty())
+      if (name.empty())
       {
          continue;
       }
@@ -1632,12 +1615,12 @@ bool AudioUnitEffect::SetAutomationParameters(EffectAutomationParameters & parms
    return true;
 }
 
-bool AudioUnitEffect::LoadUserPreset(const wxString & name)
+bool AudioUnitEffect::LoadUserPreset(const RegistryPath & name)
 {
    return LoadParameters(name);
 }
 
-bool AudioUnitEffect::SaveUserPreset(const wxString & name)
+bool AudioUnitEffect::SaveUserPreset(const RegistryPath & name)
 {
    return SaveParameters(name);
 }
@@ -1692,10 +1675,10 @@ bool AudioUnitEffect::LoadFactoryDefaults()
    return LoadParameters(mHost->GetFactoryDefaultsGroup());
 }
 
-wxArrayString AudioUnitEffect::GetFactoryPresets()
+RegistryPaths AudioUnitEffect::GetFactoryPresets()
 {
    OSStatus result;
-   wxArrayString presets;
+   RegistryPaths presets;
 
    // Retrieve the list of factory presets
    CFArrayRef array{};
@@ -1712,7 +1695,7 @@ wxArrayString AudioUnitEffect::GetFactoryPresets()
       for (CFIndex i = 0, cnt = CFArrayGetCount(array); i < cnt; i++)
       {
          AUPreset *preset = (AUPreset *) CFArrayGetValueAtIndex(array, i);
-         presets.Add(wxCFStringRef::AsString(preset->presetName));
+         presets.push_back(wxCFStringRef::AsString(preset->presetName));
       }
    }
                         
@@ -1882,7 +1865,7 @@ void AudioUnitEffect::ShowOptions()
 // AudioUnitEffect Implementation
 // ============================================================================
 
-bool AudioUnitEffect::LoadParameters(const wxString & group)
+bool AudioUnitEffect::LoadParameters(const RegistryPath & group)
 {
    wxString parms;
    if (!mHost->GetPrivateConfig(group, wxT("Parameters"), parms, wxEmptyString))
@@ -1890,7 +1873,7 @@ bool AudioUnitEffect::LoadParameters(const wxString & group)
       return false;
    }
 
-   EffectAutomationParameters eap;
+   CommandParameters eap;
    if (!eap.SetParameters(parms))
    {
       return false;
@@ -1899,9 +1882,9 @@ bool AudioUnitEffect::LoadParameters(const wxString & group)
    return SetAutomationParameters(eap);
 }
 
-bool AudioUnitEffect::SaveParameters(const wxString & group)
+bool AudioUnitEffect::SaveParameters(const RegistryPath & group)
 {
-   EffectAutomationParameters eap;
+   CommandParameters eap;
    if (!GetAutomationParameters(eap))
    {
       return false;
@@ -1965,7 +1948,9 @@ bool AudioUnitEffect::SetRateAndChannels()
                                  sizeof(Float64));
    if (result != noErr)
    {
-      wxPrintf("%ls Didn't accept sample rate on global\n", GetName().wx_str());
+      wxPrintf("%ls Didn't accept sample rate on global\n",
+               // Exposing internal name only in debug printf
+               GetSymbol().Internal().wx_str());
       return false;
    }
 
@@ -1979,7 +1964,9 @@ bool AudioUnitEffect::SetRateAndChannels()
                                     sizeof(Float64));
       if (result != noErr)
       {
-         wxPrintf("%ls Didn't accept sample rate on input\n", GetName().wx_str());
+         wxPrintf("%ls Didn't accept sample rate on input\n",
+               // Exposing internal name only in debug printf
+               GetSymbol().Internal().wx_str());
          return false;
       }
 
@@ -1991,7 +1978,9 @@ bool AudioUnitEffect::SetRateAndChannels()
                                     sizeof(AudioStreamBasicDescription));
       if (result != noErr)
       {
-         wxPrintf("%ls didn't accept stream format on input\n", GetName().wx_str());
+         wxPrintf("%ls didn't accept stream format on input\n",
+               // Exposing internal name only in debug printf
+               GetSymbol().Internal().wx_str());
          return false;
       }
    }
@@ -2006,7 +1995,9 @@ bool AudioUnitEffect::SetRateAndChannels()
                                     sizeof(Float64));
       if (result != noErr)
       {
-         wxPrintf("%ls Didn't accept sample rate on output\n", GetName().wx_str());
+         wxPrintf("%ls Didn't accept sample rate on output\n",
+               // Exposing internal name only in debug printf
+               GetSymbol().Internal().wx_str());
          return false;
       }
    
@@ -2020,7 +2011,9 @@ bool AudioUnitEffect::SetRateAndChannels()
    
       if (result != noErr)
       {
-         wxPrintf("%ls didn't accept stream format on output\n", GetName().wx_str());
+         wxPrintf("%ls didn't accept stream format on output\n",
+               // Exposing internal name only in debug printf
+               GetSymbol().Internal().wx_str());
          return false;
       }
    }
