@@ -8,15 +8,15 @@
 
 **********************************************************************/
 
-#include "../../Audacity.h"
+#include "../../Audacity.h" // for USE_* macros
 
 #if defined(USE_VAMP)
+#include "LoadVamp.h"
 
 #include <wx/filename.h>
 
 #include "../EffectManager.h"
 #include "VampEffect.h"
-#include "LoadVamp.h"
 
 #include <iostream>
 #include <map>
@@ -67,25 +67,20 @@ VampEffectsModule::~VampEffectsModule()
 }
 
 // ============================================================================
-// IdentInterface implementation
+// ComponentInterface implementation
 // ============================================================================
 
-wxString VampEffectsModule::GetPath()
+PluginPath VampEffectsModule::GetPath()
 {
    return mPath;
 }
 
-wxString VampEffectsModule::GetSymbol()
-{
-   return wxT("Vamp Effects");
-}
-
-wxString VampEffectsModule::GetName()
+ComponentInterfaceSymbol VampEffectsModule::GetSymbol()
 {
    return XO("Vamp Effects");
 }
 
-wxString VampEffectsModule::GetVendor()
+VendorSymbol VampEffectsModule::GetVendor()
 {
    return XO("The Audacity Team");
 }
@@ -117,14 +112,19 @@ void VampEffectsModule::Terminate()
    return;
 }
 
+FileExtensions VampEffectsModule::GetFileExtensions()
+{
+   return {};
+}
+
 bool VampEffectsModule::AutoRegisterPlugins(PluginManagerInterface & WXUNUSED(pm))
 {
    return false;
 }
 
-wxArrayString VampEffectsModule::FindPluginPaths(PluginManagerInterface & WXUNUSED(pm))
+PluginPaths VampEffectsModule::FindPluginPaths(PluginManagerInterface & WXUNUSED(pm))
 {
-   wxArrayString names;
+   PluginPaths names;
 
    PluginLoader *loader = PluginLoader::getInstance();
 
@@ -190,7 +190,7 @@ wxArrayString VampEffectsModule::FindPluginPaths(PluginManagerInterface & WXUNUS
          }
 
          wxString path = wxString::FromUTF8(i->c_str()) + wxT("/") + name;
-         names.Add(path);
+         names.push_back(path);
 
          ++output;
       }
@@ -200,7 +200,7 @@ wxArrayString VampEffectsModule::FindPluginPaths(PluginManagerInterface & WXUNUS
 }
 
 unsigned VampEffectsModule::DiscoverPluginsAtPath(
-   const wxString & path, wxString &errMsg,
+   const PluginPath & path, wxString &errMsg,
    const RegistrationCallback &callback)
 {
    errMsg.clear();
@@ -221,7 +221,7 @@ unsigned VampEffectsModule::DiscoverPluginsAtPath(
    return 0;
 }
 
-bool VampEffectsModule::IsPluginValid(const wxString & path, bool bFast)
+bool VampEffectsModule::IsPluginValid(const PluginPath & path, bool bFast)
 {
    int output;
    bool hasParameters;
@@ -232,7 +232,7 @@ bool VampEffectsModule::IsPluginValid(const wxString & path, bool bFast)
    return bool(vp);
 }
 
-IdentInterface *VampEffectsModule::CreateInstance(const wxString & path)
+ComponentInterface *VampEffectsModule::CreateInstance(const PluginPath & path)
 {
    // Acquires a resource for the application.
    int output;
@@ -248,7 +248,7 @@ IdentInterface *VampEffectsModule::CreateInstance(const wxString & path)
    return NULL;
 }
 
-void VampEffectsModule::DeleteInstance(IdentInterface *instance)
+void VampEffectsModule::DeleteInstance(ComponentInterface *instance)
 {
    std::unique_ptr < VampEffect > {
       dynamic_cast<VampEffect *>(instance)
@@ -257,7 +257,7 @@ void VampEffectsModule::DeleteInstance(IdentInterface *instance)
 
 // VampEffectsModule implementation
 
-std::unique_ptr<Vamp::Plugin> VampEffectsModule::FindPlugin(const wxString & path,
+std::unique_ptr<Vamp::Plugin> VampEffectsModule::FindPlugin(const PluginPath & path,
                                       int & output,
                                       bool & hasParameters)
 {
