@@ -8,14 +8,18 @@
 
 **********************************************************************/
 
-#include "../../Audacity.h"
+#include "../../Audacity.h" // for USE_* macros
 
 #if defined(USE_VAMP)
 
+#include <memory>
+
 #include "audacity/ModuleInterface.h"
 #include "audacity/EffectInterface.h"
 #include "audacity/PluginInterface.h"
 
+#include <vamp-hostsdk/PluginLoader.h>
+
 ///////////////////////////////////////////////////////////////////////////////
 //
 // VampEffectsModule
@@ -28,12 +32,11 @@ public:
    VampEffectsModule(ModuleManagerInterface *moduleManager, const wxString *path);
    virtual ~VampEffectsModule();
 
-   // IdentInterface implementation
+   // ComponentInterface implementation
 
-   wxString GetPath() override;
-   wxString GetSymbol() override;
-   wxString GetName() override;
-   wxString GetVendor() override;
+   PluginPath GetPath() override;
+   ComponentInterfaceSymbol GetSymbol() override;
+   VendorSymbol GetVendor() override;
    wxString GetVersion() override;
    wxString GetDescription() override;
 
@@ -42,31 +45,31 @@ public:
    bool Initialize() override;
    void Terminate() override;
 
-   wxArrayString FileExtensions() override { return {}; }
-   wxString InstallPath() override { return {}; }
+   FileExtensions GetFileExtensions() override;
+   FilePath InstallPath() override { return {}; }
 
    bool AutoRegisterPlugins(PluginManagerInterface & pm) override;
-   wxArrayString FindPluginPaths(PluginManagerInterface & pm) override;
+   PluginPaths FindPluginPaths(PluginManagerInterface & pm) override;
    unsigned DiscoverPluginsAtPath(
-      const wxString & path, wxString &errMsg,
+      const PluginPath & path, wxString &errMsg,
       const RegistrationCallback &callback)
          override;
 
-   bool IsPluginValid(const wxString & path, bool bFast) override;
+   bool IsPluginValid(const PluginPath & path, bool bFast) override;
 
-   IdentInterface *CreateInstance(const wxString & path) override;
-   void DeleteInstance(IdentInterface *instance) override;
+   ComponentInterface *CreateInstance(const PluginPath & path) override;
+   void DeleteInstance(ComponentInterface *instance) override;
 
 private:
    // VampEffectModule implementation
 
-   std::unique_ptr<Vamp::Plugin> FindPlugin(const wxString & wpath,
+   std::unique_ptr<Vamp::Plugin> FindPlugin(const PluginPath & wpath,
                             int & output,
                             bool & hasParameters);
 
 private:
    ModuleManagerInterface *mModMan;
-   wxString mPath;
+   PluginPath mPath;
 };
 
 #endif
