@@ -8,7 +8,7 @@
 
 **********************************************************************/
 
-#include "../Audacity.h"
+#include "../Audacity.h" // for USE_* macros
 #include "ExportPCM.h"
 
 #include <wx/defs.h>
@@ -18,7 +18,6 @@
 #include <wx/filename.h>
 #include <wx/intl.h>
 #include <wx/timer.h>
-#include <wx/msgdlg.h>
 #include <wx/progdlg.h>
 #include <wx/string.h>
 #include <wx/textctrl.h>
@@ -36,6 +35,8 @@
 #include "../Tags.h"
 #include "../Track.h"
 #include "../ondemand/ODManager.h"
+#include "../widgets/ErrorDialog.h"
+#include "../widgets/ProgressDialog.h"
 
 #include "Export.h"
 
@@ -59,8 +60,11 @@ struct
 }
 static const kFormats[] =
 {
+#if defined(__WXMAC__)
    { SF_FORMAT_AIFF | SF_FORMAT_PCM_16,   wxT("AIFF"),   XO("AIFF (Apple) signed 16-bit PCM")    },
+#endif
    { SF_FORMAT_WAV | SF_FORMAT_PCM_16,    wxT("WAV"),    XO("WAV (Microsoft) signed 16-bit PCM") },
+   { SF_FORMAT_WAV | SF_FORMAT_PCM_24,    wxT("WAV24"),  XO("WAV (Microsoft) signed 24-bit PCM") },
    { SF_FORMAT_WAV | SF_FORMAT_FLOAT,     wxT("WAVFLT"), XO("WAV (Microsoft) 32-bit float PCM")  },
 // { SF_FORMAT_WAV | SF_FORMAT_GSM610,    wxT("GSM610"), XO("GSM 6.10 WAV (mobile)")             },
 };
@@ -113,13 +117,13 @@ private:
 
 private:
 
-   wxArrayString mHeaderNames;
-   wxArrayString mEncodingNames;
+   wxArrayStringEx mHeaderNames;
+   wxArrayStringEx mEncodingNames;
    wxChoice *mHeaderChoice;
    wxChoice *mEncodingChoice;
    int mHeaderFromChoice;
    int mEncodingFromChoice;
-   wxArrayInt mEncodingFormats;
+   std::vector<int> mEncodingFormats;
 
    DECLARE_EVENT_TABLE()
 };
@@ -133,7 +137,7 @@ ExportPCMOptions::ExportPCMOptions(wxWindow *parent, int selformat)
 {
    int format;
 
-   if (selformat < 0 || selformat >= WXSIZEOF(kFormats))
+   if (selformat < 0 || static_cast<unsigned int>(selformat) >= WXSIZEOF(kFormats))
    {
       format = ReadExportFormatPref();
    }
@@ -144,7 +148,7 @@ ExportPCMOptions::ExportPCMOptions(wxWindow *parent, int selformat)
 
    mHeaderFromChoice = 0;
    for (int i = 0, num = sf_num_headers(); i < num; i++) {
-      mHeaderNames.Add(sf_header_index_name(i));
+      mHeaderNames.push_back(sf_header_index_name(i));
       if ((format & SF_FORMAT_TYPEMASK) == (int)sf_header_index_to_type(i))
          mHeaderFromChoice = i;
    }
@@ -156,8 +160,8 @@ ExportPCMOptions::ExportPCMOptions(wxWindow *parent, int selformat)
       bool valid = ValidatePair(fmt);
       if (valid)
       {
-         mEncodingNames.Add(sf_encoding_index_name(i));
-         mEncodingFormats.Add(enc);
+         mEncodingNames.push_back(sf_encoding_index_name(i));
+         mEncodingFormats.push_back(enc);
          if ((format & SF_FORMAT_SUBMASK) == (int)sf_encoding_index_to_subtype(i))
             mEncodingFromChoice = sel;
          else
@@ -188,12 +192,12 @@ void ExportPCMOptions::PopulateOrExchange(ShuttleGui & S)
             S.SetStretchyCol(1);
             mHeaderChoice = S.Id(ID_HEADER_CHOICE)
                .AddChoice(_("Header:"),
-                          mHeaderNames[mHeaderFromChoice],
-                          &mHeaderNames);
+                          mHeaderNames,
+                          mHeaderFromChoice);
             mEncodingChoice = S.Id(ID_ENCODING_CHOICE)
                .AddChoice(_("Encoding:"),
-                          mEncodingNames[mEncodingFromChoice],
-                          &mEncodingNames);
+                          mEncodingNames,
+                          mEncodingFromChoice);
          }
          S.EndMultiColumn();
       }
@@ -232,39 +236,39 @@ void ExportPCMOptions::OnHeaderChoice(wxCommandEvent & WXUNUSED(evt))
    if( format == SF_FORMAT_AIFF )
       format = SF_FORMAT_AIFF | SF_FORMAT_PCM_16;
 
-   mEncodingNames.Clear();
+   mEncodingNames.clear();
    mEncodingChoice->Clear();
-   mEncodingFormats.Clear();
+   mEncodingFormats.clear();
    int sel = wxNOT_FOUND;
    int i,j;
 
    int sfnum = sf_num_simple_formats();
-   wxArrayInt sfs;
+   std::vector<int> sfs;
 
    for (i = 0; i < sfnum; i++)
    {
       SF_FORMAT_INFO *fi = sf_simple_format(i);
-      sfs.Add(fi->format);
+      sfs.push_back(fi->format);
    }
 
    int num = sf_num_encodings();
    for (i = 0; i < num; i++)
    {
-      int enc = sf_encoding_index_to_subtype(i);
-      int fmt = format | enc;
+      int encSubtype = sf_encoding_index_to_subtype(i);
+      int fmt = format | encSubtype;
       bool valid  = ValidatePair(fmt);
       if (valid)
       {
          const auto name = sf_encoding_index_name(i);
-         mEncodingNames.Add(name);
+         mEncodingNames.push_back(name);
          mEncodingChoice->Append(name);
-         mEncodingFormats.Add(enc);
+         mEncodingFormats.push_back(encSubtype);
          for (j = 0; j < sfnum; j++)
          {
             int enc = sfs[j];
             if ((sel == wxNOT_FOUND) && (fmt == enc))
             {
-               sel = mEncodingFormats.GetCount()-1;
+               sel = mEncodingFormats.size() - 1;
                break;
             }
          }
@@ -277,6 +281,14 @@ void ExportPCMOptions::OnHeaderChoice(wxCommandEvent & WXUNUSED(evt))
    ValidatePair(GetFormat());
 
    TransferDataFromWindow();
+
+   // Send the event indicating a file suffix change.
+   // We pass the entire header string, which starts with the suffix.
+   wxCommandEvent event(AUDACITY_FILE_SUFFIX_EVENT, GetId());
+   event.SetEventObject(this);
+   event.SetString(mHeaderChoice->GetString(mHeaderChoice->GetSelection()));
+   ProcessWindowEvent(event);
+
 }
 
 int ExportPCMOptions::GetFormat()
@@ -319,6 +331,7 @@ public:
 
    wxWindow *OptionsCreate(wxWindow *parent, int format) override;
    ProgressResult Export(AudacityProject *project,
+               std::unique_ptr<ProgressDialog> &pDialog,
                unsigned channels,
                const wxString &fName,
                bool selectedOnly,
@@ -328,11 +341,11 @@ public:
                const Tags *metadata = NULL,
                int subformat = 0) override;
    // optional
-   wxString GetExtension(int index);
+   FileExtension GetExtension(int index) override;
    bool CheckFileName(wxFileName &filename, int format) override;
 
 private:
-
+   void ReportTooBigError(wxWindow * pParent);
    ArrayOf<char> AdjustString(const wxString & wxStr, int sf_format);
    bool AddStrings(AudacityProject *project, SNDFILE *sf, const Tags *tags, int sf_format);
    bool AddID3Chunk(wxString fName, const Tags *tags, int sf_format);
@@ -372,24 +385,55 @@ ExportPCM::ExportPCM()
    SetFormat(wxT("LIBSNDFILE"), format);
    SetCanMetaData(true, format);
    SetDescription(_("Other uncompressed files"), format);
-   wxArrayString allext = sf_get_all_extensions();
+   auto allext = sf_get_all_extensions();
    wxString wavext = sf_header_extension(SF_FORMAT_WAV);   // get WAV ext.
 #if defined(wxMSW)
    // On Windows make sure WAV is at the beginning of the list of all possible
    // extensions for this format
-   allext.Remove(wavext);
-   allext.Insert(wavext, 0);
+   allext.erase( std::find( allext.begin(), allext.end(), wavext ) );
+   allext.insert(allext.begin(), wavext);
 #endif
    SetExtensions(allext, format);
    SetMaxChannels(255, format);
 }
 
+// Bug 2057
+// This function is not yet called anywhere.
+// It is provided to get the translation strings.
+// We can hook it in properly whilst translation happens.
+void ExportPCM::ReportTooBigError(wxWindow * pParent)
+{
+   //Temporary translation hack, to say 'WAV or AIFF' rather than 'WAV'
+   wxString message = 
+      _("You have attempted to Export a WAV file which would be greater than 4GB.\n"
+      "Audacity cannot do this, the Export was abandoned.");
+   if( message == 
+      "You have attempted to Export a WAV file which would be greater than 4GB.\n"
+      "Audacity cannot do this, the Export was abandoned.")
+   {
+      message.Replace( "WAV", "WAV or AIFF");
+   }
+
+   ShowErrorDialog(pParent, _("Error Exporting"), message,
+                  wxT("Size_limits_for_WAV_and_AIFF_files"));
+
+// This alternative error dialog was to cover the possibility we could not 
+// compute the size in advance.
+#if 0
+   ShowErrorDialog(pParent, _("Error Exporting"),
+                  _("Your exported WAV file has been truncated as Audacity cannot export WAV\n"
+                    "files bigger than 4GB."),
+                  wxT("Size_limits_for_WAV_files"));
+#endif
+}
+
 /**
  *
  * @param subformat Control whether we are doing a "preset" export to a popular
  * file type, or giving the user full control over libsndfile.
  */
 ProgressResult ExportPCM::Export(AudacityProject *project,
+                       std::unique_ptr<ProgressDialog> &pDialog,
                        unsigned numChannels,
                        const wxString &fName,
                        bool selectionOnly,
@@ -403,7 +447,7 @@ ProgressResult ExportPCM::Export(AudacityProject *project,
    const TrackList   *tracks = project->GetTracks();
    int sf_format;
 
-   if (subformat < 0 || subformat >= WXSIZEOF(kFormats))
+   if (subformat < 0 || static_cast<unsigned int>(subformat) >= WXSIZEOF(kFormats))
    {
       sf_format = ReadExportFormatPref();
    }
@@ -440,7 +484,7 @@ ProgressResult ExportPCM::Export(AudacityProject *project,
       if (!sf_format_check(&info))
          info.format = (info.format & SF_FORMAT_TYPEMASK);
       if (!sf_format_check(&info)) {
-         wxMessageBox(_("Cannot export audio in this format."));
+         AudacityMessageBox(_("Cannot export audio in this format."));
          return ProgressResult::Cancelled;
       }
 
@@ -454,8 +498,8 @@ ProgressResult ExportPCM::Export(AudacityProject *project,
       }
 
       if (!sf) {
-         wxMessageBox(wxString::Format(_("Cannot export audio to %s"),
-                                       fName.c_str()));
+         AudacityMessageBox(wxString::Format(_("Cannot export audio to %s"),
+                                       fName));
          return ProgressResult::Cancelled;
       }
       // Retrieve tags if not given a set
@@ -477,6 +521,16 @@ ProgressResult ExportPCM::Export(AudacityProject *project,
       else
          format = int16Sample;
 
+      float sampleCount = (float)(t1-t0)*rate*info.channels;
+      // floatSamples are always 4 byte, even if processor uses more.
+      float byteCount = sampleCount * ((format==int16Sample)?2:4);
+      // Test for 4 Gibibytes, rather than 4 Gigabytes
+      if( byteCount > 4.295e9)
+      {
+         ReportTooBigError( wxTheApp->GetTopWindow() );
+         return ProgressResult::Failed;
+      }
+
       size_t maxBlockLen = 44100 * 5;
 
       const WaveTrackConstArray waveTracks =
@@ -489,16 +543,17 @@ ProgressResult ExportPCM::Export(AudacityProject *project,
                                   info.channels, maxBlockLen, true,
                                   rate, format, true, mixerSpec);
 
-         ProgressDialog progress(wxFileName(fName).GetName(),
-                                 selectionOnly ?
-                                 wxString::Format(_("Exporting the selected audio as %s"),
-                                                  formatStr.c_str()) :
-                                 wxString::Format(_("Exporting the entire project as %s"),
-                                                  formatStr.c_str()));
+         InitProgress( pDialog, wxFileName(fName).GetName(),
+            selectionOnly
+               ? wxString::Format(_("Exporting the selected audio as %s"),
+                  formatStr)
+               : wxString::Format(_("Exporting the audio as %s"),
+                  formatStr) );
+         auto &progress = *pDialog;
 
          while (updateResult == ProgressResult::Success) {
             sf_count_t samplesWritten;
-            auto numSamples = mixer->Process(maxBlockLen);
+            size_t numSamples = mixer->Process(maxBlockLen);
 
             if (numSamples == 0)
                break;
@@ -510,16 +565,16 @@ ProgressResult ExportPCM::Export(AudacityProject *project,
             else
                samplesWritten = SFCall<sf_count_t>(sf_writef_float, sf.get(), (float *)mixed, numSamples);
 
-            if (samplesWritten != numSamples) {
+            if (static_cast<size_t>(samplesWritten) != numSamples) {
                char buffer2[1000];
                sf_error_str(sf.get(), buffer2, 1000);
-               wxMessageBox(wxString::Format(
+               AudacityMessageBox(wxString::Format(
                                              /* i18n-hint: %s will be the error message from libsndfile, which
                                               * is usually something unhelpful (and untranslated) like "system
                                               * error" */
                                              _("Error while writing %s file (disk full?).\nLibsndfile says \"%s\""),
-                                             formatStr.c_str(),
-                                             wxString::FromAscii(buffer2).c_str()));
+                                             formatStr,
+                                             wxString::FromAscii(buffer2)));
                updateResult = ProgressResult::Cancelled;
                break;
             }
@@ -529,18 +584,34 @@ ProgressResult ExportPCM::Export(AudacityProject *project,
       }
       
       // Install the WAV metata in a "LIST" chunk at the end of the file
-      if ((sf_format & SF_FORMAT_TYPEMASK) == SF_FORMAT_WAV ||
-          (sf_format & SF_FORMAT_TYPEMASK) == SF_FORMAT_WAVEX) {
-         if (!AddStrings(project, sf.get(), metadata, sf_format)) {
+      if (updateResult == ProgressResult::Success ||
+          updateResult == ProgressResult::Stopped) {
+         if ((sf_format & SF_FORMAT_TYPEMASK) == SF_FORMAT_WAV ||
+             (sf_format & SF_FORMAT_TYPEMASK) == SF_FORMAT_WAVEX) {
+            if (!AddStrings(project, sf.get(), metadata, sf_format)) {
+               // TODO: more precise message
+               AudacityMessageBox(_("Unable to export"));
+               return ProgressResult::Cancelled;
+            }
+         }
+         if (0 != sf.close()) {
+            // TODO: more precise message
+            AudacityMessageBox(_("Unable to export"));
             return ProgressResult::Cancelled;
          }
       }
    }
 
-   if (((sf_format & SF_FORMAT_TYPEMASK) == SF_FORMAT_AIFF) ||
-       ((sf_format & SF_FORMAT_TYPEMASK) == SF_FORMAT_WAV))
-      if (!AddID3Chunk(fName, metadata, sf_format) )
-         return ProgressResult::Cancelled;
+   if (updateResult == ProgressResult::Success ||
+       updateResult == ProgressResult::Stopped)
+      if (((sf_format & SF_FORMAT_TYPEMASK) == SF_FORMAT_AIFF) ||
+          ((sf_format & SF_FORMAT_TYPEMASK) == SF_FORMAT_WAV))
+         // Note: file has closed, and gets reopened and closed again here:
+         if (!AddID3Chunk(fName, metadata, sf_format) ) {
+            // TODO: more precise message
+            AudacityMessageBox(_("Unable to export"));
+            return ProgressResult::Cancelled;
+         }
 
    return updateResult;
 }
@@ -700,10 +771,12 @@ bool ExportPCM::AddStrings(AudacityProject * WXUNUSED(project), SNDFILE *sf, con
    return true;
 }
 
+#ifdef USE_LIBID3TAG
 struct id3_tag_deleter {
    void operator () (id3_tag *p) const { if (p) id3_tag_delete(p); }
 };
 using id3_tag_holder = std::unique_ptr<id3_tag, id3_tag_deleter>;
+#endif
 
 bool ExportPCM::AddID3Chunk(wxString fName, const Tags *tags, int sf_format)
 {
@@ -834,6 +907,9 @@ bool ExportPCM::AddID3Chunk(wxString fName, const Tags *tags, int sf_format)
       if (4 != f.Write(&sz, 4))
          return false;
 
+      if (!f.Flush())
+         return false;
+
       if (!f.Close())
          return false;
    }
@@ -847,7 +923,7 @@ wxWindow *ExportPCM::OptionsCreate(wxWindow *parent, int format)
 {
    wxASSERT(parent); // to justify safenew
    // default, full user control
-   if (format < 0 || format >= WXSIZEOF(kFormats))
+   if (format < 0 || static_cast<unsigned int>(format) >= WXSIZEOF(kFormats))
    {
       return safenew ExportPCMOptions(parent, format);
    }
@@ -855,7 +931,7 @@ wxWindow *ExportPCM::OptionsCreate(wxWindow *parent, int format)
    return ExportPlugin::OptionsCreate(parent, format);
 }
 
-wxString ExportPCM::GetExtension(int index)
+FileExtension ExportPCM::GetExtension(int index)
 {
    if (index == WXSIZEOF(kFormats)) {
       // get extension libsndfile thinks is correct for currently selected format
@@ -885,7 +961,7 @@ bool ExportPCM::CheckFileName(wxFileName &filename, int format)
    return ExportPlugin::CheckFileName(filename, format);
 }
 
-movable_ptr<ExportPlugin> New_ExportPCM()
+std::unique_ptr<ExportPlugin> New_ExportPCM()
 {
-   return make_movable<ExportPCM>();
+   return std::make_unique<ExportPCM>();
 }
