@@ -16,6 +16,7 @@
 *//*******************************************************************/
 
 #include "../Audacity.h"
+#include "DirectoriesPrefs.h"
 
 #include <math.h>
 
@@ -36,7 +37,6 @@
 #include "../Internat.h"
 #include "../ShuttleGui.h"
 #include "../widgets/ErrorDialog.h"
-#include "DirectoriesPrefs.h"
 
 enum {
    TempDirID = 1000,
@@ -48,9 +48,9 @@ BEGIN_EVENT_TABLE(DirectoriesPrefs, PrefsPanel)
    EVT_BUTTON(ChooseButtonID, DirectoriesPrefs::OnChooseTempDir)
 END_EVENT_TABLE()
 
-DirectoriesPrefs::DirectoriesPrefs(wxWindow * parent)
+DirectoriesPrefs::DirectoriesPrefs(wxWindow * parent, wxWindowID winid)
 /* i18n-hint:  Directories, also called folders, in computer file systems */
-:  PrefsPanel(parent, _("Directories")),
+:  PrefsPanel(parent, winid, _("Directories")),
    mFreeSpace(NULL),
    mTempDir(NULL)
 {
@@ -61,6 +61,22 @@ DirectoriesPrefs::~DirectoriesPrefs()
 {
 }
 
+
+ComponentInterfaceSymbol DirectoriesPrefs::GetSymbol()
+{
+   return DIRECTORIES_PREFS_PLUGIN_SYMBOL;
+}
+
+wxString DirectoriesPrefs::GetDescription()
+{
+   return _("Preferences for Directories");
+}
+
+wxString DirectoriesPrefs::HelpPageName()
+{
+   return "Directories_Preferences";
+}
+
 /// Creates the dialog and its contents.
 void DirectoriesPrefs::Populate()
 {
@@ -83,7 +99,7 @@ void DirectoriesPrefs::PopulateOrExchange(ShuttleGui & S)
 
    S.StartStatic(_("Temporary files directory"));
    {
-      S.StartMultiColumn(3, wxEXPAND);
+      S.StartMultiColumn(2, wxEXPAND);
       {
          S.SetStretchyCol(1);
 
@@ -92,13 +108,16 @@ void DirectoriesPrefs::PopulateOrExchange(ShuttleGui & S)
                                  wxT("/Directories/TempDir"),
                                  wxT(""),
                                  30);
-         S.Id(ChooseButtonID);
-         S.AddButton(_("C&hoose..."));
-
-         S.AddFixedText(_("Free Space:"));
-         mFreeSpace = S.AddVariableText( {} );
       }
       S.EndMultiColumn();
+      S.StartHorizontalLay(wxEXPAND);
+      {
+         S.Prop(0).AddFixedText(_("Free Space:"));
+         mFreeSpace = S.Prop(0).AddVariableText( {} );
+         S.Prop(10).AddSpace( 10 );
+         S.Id(ChooseButtonID).Prop(0).AddButton(_("C&hoose..."));
+      }
+
    }
    S.EndStatic();
 
@@ -142,7 +161,7 @@ void DirectoriesPrefs::OnChooseTempDir(wxCommandEvent & e)
                     _("Choose a location to place the temporary directory"),
                     oldTempDir );
    int retval = dlog.ShowModal();
-   if (retval != wxID_CANCEL && dlog.GetPath() != wxT("")) {
+   if (retval != wxID_CANCEL && !dlog.GetPath().empty()) {
       wxFileName tmpDirPath;
       tmpDirPath.AssignDir(dlog.GetPath());
 
@@ -159,15 +178,15 @@ void DirectoriesPrefs::OnChooseTempDir(wxCommandEvent & e)
 #else
       newDirName = wxT(".audacity_temp");
 #endif
-      wxArrayString dirsInPath = tmpDirPath.GetDirs();
+      auto dirsInPath = tmpDirPath.GetDirs();
 
       // If the default temp dir or user's pref dir don't end in '/' they cause
       // wxFileName's == operator to construct a wxFileName representing a file
       // (that doesn't exist) -- hence the constructor calls
       if (tmpDirPath != wxFileName(wxGetApp().defaultTempDir, wxT("")) &&
             tmpDirPath != wxFileName(mTempDir->GetValue(), wxT("")) &&
-            (dirsInPath.GetCount() == 0 ||
-             dirsInPath[dirsInPath.GetCount()-1] != newDirName))
+            (dirsInPath.size() == 0 ||
+             dirsInPath[dirsInPath.size()-1] != newDirName))
       {
          tmpDirPath.AppendDir(newDirName);
       }
@@ -268,13 +287,8 @@ bool DirectoriesPrefs::Commit()
    return true;
 }
 
-wxString DirectoriesPrefs::HelpPageName()
-{
-   return "Directories_Preferences";
-}
-
-PrefsPanel *DirectoriesPrefsFactory::Create(wxWindow *parent)
+PrefsPanel *DirectoriesPrefsFactory::operator () (wxWindow *parent, wxWindowID winid)
 {
    wxASSERT(parent); // to justify safenew
-   return safenew DirectoriesPrefs(parent);
+   return safenew DirectoriesPrefs(parent, winid);
 }
