@@ -12,42 +12,41 @@
 #ifndef __AUDACITY_KEY_CONFIG_PREFS__
 #define __AUDACITY_KEY_CONFIG_PREFS__
 
-#include "../Experimental.h"
-
+class CommandManager;
 class ShuttleGui;
 
 #include <wx/defs.h>
-#include <wx/imaglist.h>
-#include <wx/listctrl.h>
-#include <wx/radiobut.h>
-#include <wx/srchctrl.h>
-#include <wx/string.h>
-#include <wx/textctrl.h>
-#include <wx/timer.h>
-
-#include "../commands/CommandManager.h"
-#include "../widgets/KeyView.h"
+#include <wx/timer.h> // member variable
 
 #include "PrefsPanel.h"
 
+class wxRadioButton;
 class wxStaticText;
+class wxTextCtrl;
+class KeyView;
+struct NormalizedKeyString;
+enum ViewByType : int;
+
+#define KEY_CONFIG_PREFS_PLUGIN_SYMBOL ComponentInterfaceSymbol{ XO("Key Config") }
 
 class KeyConfigPrefs final : public PrefsPanel
 {
 public:
-   KeyConfigPrefs(wxWindow * parent, const wxString &name);
-   ~KeyConfigPrefs();
+   KeyConfigPrefs(wxWindow * parent, wxWindowID winid, const CommandID &name);
+   ComponentInterfaceSymbol GetSymbol() override;
+   wxString GetDescription() override;
+
    bool Commit() override;
    void Cancel() override;
    wxString HelpPageName() override;
+   void PopulateOrExchange(ShuttleGui & S) override;
 
 private:
    void Populate();
-   void PopulateOrExchange(ShuttleGui & S);
    void RefreshBindings(bool bSort);
-   void FilterKeys( wxArrayString & arr );
-   wxString NameFromKey(const wxString & key);
-   void SetKeyForSelected(const wxString & key);
+   void FilterKeys( std::vector<NormalizedKeyString> & arr );
+   CommandID NameFromKey(const NormalizedKeyString & key);
+   void SetKeyForSelected(const NormalizedKeyString & key);
 
    void OnViewBy(wxCommandEvent & e);
    void OnDefaults(wxCommandEvent & e);
@@ -84,24 +83,25 @@ private:
    CommandManager *mManager;
    int mCommandSelected;
 
-   wxArrayString mNames;
-   wxArrayString mDefaultKeys; // The full set.
-   wxArrayString mStandardDefaultKeys; // The reduced set.
-   wxArrayString mKeys;
-   wxArrayString mNewKeys; // Used for work in progress.
+   CommandIDs mNames;
+   std::vector<NormalizedKeyString> mDefaultKeys; // The full set.
+   std::vector<NormalizedKeyString> mStandardDefaultKeys; // The reduced set.
+   std::vector<NormalizedKeyString> mKeys;
+   std::vector<NormalizedKeyString> mNewKeys; // Used for work in progress.
 
    DECLARE_EVENT_TABLE()
 };
 
 
+/// A PrefsPanelFactory that creates one KeyConfigPrefs panel.
 class KeyConfigPrefsFactory final : public PrefsPanelFactory
 {
 public:
-   KeyConfigPrefsFactory(const wxString &name = wxString{})
+   KeyConfigPrefsFactory(const CommandID &name = {})
       : mName{ name } {}
-   PrefsPanel *Create(wxWindow *parent) override;
-   
+   PrefsPanel *operator () (wxWindow *parent, wxWindowID winid) override;
+
 private:
-   wxString mName;
+   CommandID mName;
 };
 #endif
