@@ -17,6 +17,7 @@ handling.
 *//*******************************************************************/
 
 #include "../Audacity.h"
+#include "ProjectsPrefs.h"
 
 #include <wx/defs.h>
 #include <wx/textctrl.h>
@@ -24,13 +25,12 @@ handling.
 #include "../Prefs.h"
 #include "../ShuttleGui.h"
 
-#include "ProjectsPrefs.h"
 #include "../Internat.h"
 
 ////////////////////////////////////////////////////////////////////////////////
 
-ProjectsPrefs::ProjectsPrefs(wxWindow * parent)
-:   PrefsPanel(parent,
+ProjectsPrefs::ProjectsPrefs(wxWindow * parent, wxWindowID winid)
+:   PrefsPanel(parent, winid,
    /* i18n-hint: (noun) i.e Audacity projects. */
                _("Projects"))
 {
@@ -41,6 +41,21 @@ ProjectsPrefs::~ProjectsPrefs()
 {
 }
 
+ComponentInterfaceSymbol ProjectsPrefs::GetSymbol()
+{
+   return PROJECTS_PREFS_PLUGIN_SYMBOL;
+}
+
+wxString ProjectsPrefs::GetDescription()
+{
+   return _("Preferences for Projects");
+}
+
+wxString ProjectsPrefs::HelpPageName()
+{
+   return "Projects_Preferences";
+}
+
 /// Creates the dialog and its contents.
 void ProjectsPrefs::Populate()
 {
@@ -84,13 +99,8 @@ bool ProjectsPrefs::Commit()
    return true;
 }
 
-wxString ProjectsPrefs::HelpPageName()
-{
-   return "Projects_Preferences";
-}
-
-PrefsPanel *ProjectsPrefsFactory::Create(wxWindow *parent)
+PrefsPanel *ProjectsPrefsFactory::operator () (wxWindow *parent, wxWindowID winid)
 {
    wxASSERT(parent); // to justify safenew
-   return safenew ProjectsPrefs(parent);
+   return safenew ProjectsPrefs(parent, winid);
 }
