@@ -15,22 +15,27 @@
 #define __AUDACITY_THEME_PREFS__
 
 #include <wx/defs.h>
-#include <wx/window.h>
 
 #include "PrefsPanel.h"
 
 class ShuttleGui;
 
+#define THEME_PREFS_PLUGIN_SYMBOL ComponentInterfaceSymbol{ XO("Theme") }
+
 class ThemePrefs final : public PrefsPanel
 {
  public:
-   ThemePrefs(wxWindow * parent);
+   ThemePrefs(wxWindow * parent, wxWindowID winid);
    ~ThemePrefs(void);
+   ComponentInterfaceSymbol GetSymbol() override;
+   wxString GetDescription() override;
+
    bool Commit() override;
+   wxString HelpPageName() override;
 
  private:
    void Populate();
-   void PopulateOrExchange(ShuttleGui & S);
+   void PopulateOrExchange(ShuttleGui & S) override;
    void OnLoadThemeComponents(wxCommandEvent & e);
    void OnSaveThemeComponents(wxCommandEvent & e);
    void OnLoadThemeCache(wxCommandEvent & e);
@@ -41,9 +46,10 @@ class ThemePrefs final : public PrefsPanel
    DECLARE_EVENT_TABLE()
 };
 
+/// A PrefsPanelFactory that creates one ThemePrefs panel.
 class ThemePrefsFactory final : public PrefsPanelFactory
 {
 public:
-   PrefsPanel *Create(wxWindow *parent) override;
+   PrefsPanel *operator () (wxWindow *parent, wxWindowID winid) override;
 };
 #endif
