@@ -15,10 +15,12 @@ Paul Licameli
 
 #include "../Audacity.h"
 #include "WaveformPrefs.h"
+
 #include "GUIPrefs.h"
 #include "GUISettings.h"
 
 #include <wx/checkbox.h>
+#include <wx/choice.h>
 
 #include "../Project.h"
 
@@ -26,9 +28,9 @@ Paul Licameli
 #include "../ShuttleGui.h"
 #include "../WaveTrack.h"
 
-WaveformPrefs::WaveformPrefs(wxWindow * parent, WaveTrack *wt)
+WaveformPrefs::WaveformPrefs(wxWindow * parent, wxWindowID winid, WaveTrack *wt)
 /* i18n-hint: A waveform is a visual representation of vibration */
-: PrefsPanel(parent, _("Waveforms"))
+: PrefsPanel(parent, winid, _("Waveforms"))
 , mWt(wt)
 , mPopulating(false)
 {
@@ -50,6 +52,21 @@ WaveformPrefs::~WaveformPrefs()
 {
 }
 
+ComponentInterfaceSymbol WaveformPrefs::GetSymbol()
+{
+   return WAVEFORM_PREFS_PLUGIN_SYMBOL;
+}
+
+wxString WaveformPrefs::GetDescription()
+{
+   return _("Preferences for Waveforms");
+}
+
+wxString WaveformPrefs::HelpPageName()
+{
+   return "Waveform_Preferences";
+}
+
 enum {
    ID_DEFAULTS = 10001,
 
@@ -59,8 +76,6 @@ enum {
 
 void WaveformPrefs::Populate()
 {
-   mScaleChoices = WaveformSettings::GetScaleNames();
-
    // Reuse the same choices and codes as for Interface prefs
    GUIPrefs::GetRangeChoices(&mRangeChoices, &mRangeCodes);
 
@@ -92,14 +107,13 @@ void WaveformPrefs::PopulateOrExchange(ShuttleGui & S)
          {
             mScaleChoice =
                S.Id(ID_SCALE).TieChoice(_("S&cale") + wxString(wxT(":")),
-                  *(int*)&mTempSettings.scaleType,
-                  &mScaleChoices);
+                  mTempSettings.scaleType,
+                  WaveformSettings::GetScaleNames());
 
             mRangeChoice =
                S.Id(ID_RANGE).TieChoice(_("Waveform dB &range") + wxString(wxT(":")),
-               *(int*)&mTempSettings.dBRange,
-               &mRangeChoices);
-            S.SetSizeHints(mRangeChoices);
+               mTempSettings.dBRange,
+               mRangeChoices);
          }
          S.EndTwoColumn();
       }
@@ -141,12 +155,6 @@ bool WaveformPrefs::Commit()
 {
    const bool isOpenPage = this->IsShown();
 
-   const auto partner =
-      mWt ?
-            // Assume linked track is wave or null
-            static_cast<WaveTrack*>(mWt->GetLink())
-          : nullptr;
-
    ShuttleGui S(this, eIsGettingFromDialog);
    PopulateOrExchange(S);
 
@@ -154,18 +162,13 @@ bool WaveformPrefs::Commit()
    WaveformSettings::Globals::Get().SavePrefs();
 
    if (mWt) {
-      if (mDefaulted) {
-         mWt->SetWaveformSettings({});
-         if (partner)
-            partner->SetWaveformSettings({});
-      }
-      else {
-         WaveformSettings *pSettings =
-            &mWt->GetIndependentWaveformSettings();
-         *pSettings = mTempSettings;
-         if (partner) {
-            pSettings = &partner->GetIndependentWaveformSettings();
-            *pSettings = mTempSettings;
+      for (auto channel : TrackList::Channels(mWt)) {
+         if (mDefaulted)
+            channel->SetWaveformSettings({});
+         else {
+            WaveformSettings &settings =
+               channel->GetIndependentWaveformSettings();
+            settings = mTempSettings;
          }
       }
    }
@@ -180,9 +183,8 @@ bool WaveformPrefs::Commit()
    mTempSettings.ConvertToEnumeratedDBRange();
 
    if (mWt && isOpenPage) {
-      mWt->SetDisplay(WaveTrack::Waveform);
-      if (partner)
-         partner->SetDisplay(WaveTrack::Waveform);
+      for (auto channel : TrackList::Channels(mWt))
+         channel->SetDisplay(WaveTrack::Waveform);
    }
 
    if (isOpenPage) {
@@ -250,8 +252,8 @@ WaveformPrefsFactory::WaveformPrefsFactory(WaveTrack *wt)
 {
 }
 
-PrefsPanel *WaveformPrefsFactory::Create(wxWindow *parent)
+PrefsPanel *WaveformPrefsFactory::operator () (wxWindow *parent, wxWindowID winid)
 {
    wxASSERT(parent); // to justify safenew
-   return safenew WaveformPrefs(parent, mWt);
+   return safenew WaveformPrefs(parent, winid, mWt);
 }
