@@ -18,7 +18,6 @@
 
 #include "ToolBar.h"
 #include "../Theme.h"
-#include "../Experimental.h"
 
 class wxCommandEvent;
 class wxDC;
@@ -56,8 +55,10 @@ public:
 
 private:
 
-   AButton *AddButton(teBmps eEnabledUp, teBmps eEnabledDown, teBmps eDisabled,
-                      int id, const wxChar *label, bool toggle = false);
+   static AButton *AddButton(
+      ScrubbingToolBar *pBar,
+      teBmps eEnabledUp, teBmps eEnabledDown, teBmps eDisabled,
+      int id, const wxChar *label, bool toggle = false);
 
    void MakeButtons();
 
@@ -69,7 +70,7 @@ private:
 
 public:
 
-   DECLARE_CLASS(EditToolBar)
+   DECLARE_CLASS(ScrubbingToolBar)
    DECLARE_EVENT_TABLE()
 };
 
