@@ -27,10 +27,14 @@ with changes in the SelectionBar.
 
 
 #include "../Audacity.h"
+#include "SelectionBar.h"
+#include "SelectionBarListener.h"
 
 // For compilers that support precompilation, includes "wx/wx.h".
 #include <wx/wxprec.h>
 
+#include <wx/setup.h> // for wxUSE_* macros
+
 #ifndef WX_PRECOMP
 #include <wx/button.h>
 #include <wx/checkbox.h>
@@ -45,9 +49,6 @@ with changes in the SelectionBar.
 #include <wx/statline.h>
 
 
-#include "SelectionBarListener.h"
-#include "SelectionBar.h"
-
 #include "../widgets/AButton.h"
 #include "../AudioIO.h"
 #include "../AColor.h"
@@ -57,6 +58,10 @@ with changes in the SelectionBar.
 #include "../widgets/NumericTextCtrl.h"
 #include "../AllThemeResources.h"
 
+#if wxUSE_ACCESSIBILITY
+#include "../widgets/WindowAccessible.h"
+#endif
+
 IMPLEMENT_CLASS(SelectionBar, ToolBar);
 
 const static wxChar *numbers[] =
@@ -73,29 +78,11 @@ enum {
 
    ChoiceID,
 
-   StartTitleID,
-   LengthTitleID,
-   CenterTitleID,
-   EndTitleID,
-   AudioTitleID,
-
-   LeftID,
-   CentralNameID,
-   RightID,
-
-   StartEndRadioID,
-   StartLengthRadioID,
-   LengthEndRadioID,
-   LengthCenterRadioID,
-
-   SelTBFirstButton,
-   SelTBMenuID = SelTBFirstButton,
-
-   id2,
    StartTimeID,
    LengthTimeID,
    CenterTimeID,
    EndTimeID,
+
    AudioTimeID,
 };
 
@@ -109,29 +96,20 @@ BEGIN_EVENT_TABLE(SelectionBar, ToolBar)
    EVT_CHOICE(ChoiceID, SelectionBar::OnChoice )
    EVT_COMBOBOX(RateID, SelectionBar::OnRate)
    EVT_TEXT(RateID, SelectionBar::OnRate)
-   EVT_RADIOBUTTON(StartEndRadioID, SelectionBar::OnFieldChoice )
-   EVT_RADIOBUTTON(StartLengthRadioID, SelectionBar::OnFieldChoice )
-   EVT_RADIOBUTTON(LengthEndRadioID, SelectionBar::OnFieldChoice )
-   EVT_RADIOBUTTON(LengthCenterRadioID, SelectionBar::OnFieldChoice )
 
-   EVT_COMMAND_RANGE( SelTBMenuID,
-                      SelTBMenuID,
-                      wxEVT_COMMAND_BUTTON_CLICKED,
-                      SelectionBar::OnButton )
-// EVT_COMMAND( OnMenuId, wxEVT_COMMAND_BUTTON_CLICKED, SelectionBar::OnButton )
    EVT_COMMAND(wxID_ANY, EVT_TIMETEXTCTRL_UPDATED, SelectionBar::OnUpdate)
    EVT_COMMAND(wxID_ANY, EVT_CAPTURE_KEY, SelectionBar::OnCaptureKey)
 END_EVENT_TABLE()
 
 SelectionBar::SelectionBar()
 : ToolBar(SelectionBarID, _("Selection"), wxT("Selection")),
-  mListener(NULL), mRate(0.0), 
+  mListener(NULL), mRate(0.0),
   mStart(0.0), mEnd(0.0), mLength(0.0), mCenter(0.0), mAudio(0.0),
-  mStartTime(NULL), mEndTime(NULL), mLengthTime(NULL), mCenterTime(NULL), 
-  mAudioTime(NULL),
-  mChoice(NULL),
   mDrive1( StartTimeID), mDrive2( EndTimeID ),
-  mSelectionMode(0)
+  mSelectionMode(0),
+  mStartTime(NULL), mCenterTime(NULL), mLengthTime(NULL), mEndTime(NULL),
+  mAudioTime(NULL),
+  mChoice(NULL)
 {
    // Make sure we have a valid rate as the NumericTextCtrl()s
    // created in Populate()
@@ -139,6 +117,7 @@ SelectionBar::SelectionBar()
    // will occur.
    // Refer to bug #462 for a scenario where the division-by-zero causes
    // Audacity to fail.
+   // We expect mRate to be set from the project later.
    mRate = (double) gPrefs->Read(wxT("/SamplingRate/DefaultProjectSampleRate"),
       AudioIO::GetOptimalSupportedSampleRate());
 
@@ -155,60 +134,22 @@ void SelectionBar::Create(wxWindow * parent)
    ToolBar::Create(parent);
 }
 
-// Add Radio Button function is not used anymore.
-// But maybe we will need it again in the future.
-//
-// Can't set textcolour of radio buttons.
-// so instead if we want to them, we make the text empty and add in a wxStaticText
-// and we can set the colour of that.
-// Slight regression relative ot Audacity, in that this text is not 
-// clickable/active.  You have to click on the actual button.
-// And you can't tab between and hear the labels with voice over.
-// So VI users should use blend themes (which is the default).
-// Should not be a hardship for them, as themes make little difference 
-// for them, except Hi-Contrast, which should be used with blend thems
-// and a windows theme that is close enough to actually blend.
-
-wxRadioButton * SelectionBar::AddRadioButton( const wxString & Name, 
-   int id, wxSizer *pSizer, long style )
-{
-   bool bUseNativeRadioButton = theTheme.IsUsingSystemTextColour();
-   wxRadioButton * pBtn;
-   // Safenew because the button is being create dinto this window.
-   pBtn = safenew wxRadioButton(this, id,bUseNativeRadioButton ? Name : wxT(""),
-      wxDefaultPosition, wxDefaultSize, style);
-   pBtn->SetName(Name);
-   pBtn->SetForegroundColour( theTheme.Colour( clrTrackPanelText ));
-
-   pSizer->Add(pBtn, 0, wxALIGN_CENTER_VERTICAL | wxRIGHT, 5);
-   // Hacky code to return a second optional value via the variable mProxy.
-   // If not NULL, we made a NEW proxy label
-   mProxy = NULL;
-   if( !bUseNativeRadioButton )
-   {
-      mProxy = safenew wxStaticText(this, -1, Name);
-      mProxy->SetForegroundColour( theTheme.Colour( clrTrackPanelText ) );
-      pSizer->Add(mProxy, 0, wxRIGHT | wxALIGN_CENTER_VERTICAL, 5);
-   }
-   return pBtn;
-}
 
 auStaticText * SelectionBar::AddTitle( const wxString & Title, wxSizer * pSizer ){
    auStaticText * pTitle = safenew auStaticText(this, Title );
    pTitle->SetBackgroundColour( theTheme.Colour( clrMedium ));
    pTitle->SetForegroundColour( theTheme.Colour( clrTrackPanelText ) );
-   pSizer->Add( pTitle,0, wxALIGN_CENTER_VERTICAL | wxRIGHT,  (Title.Length() == 1 ) ? 0:5);
+   pSizer->Add( pTitle,0, wxALIGN_CENTER_VERTICAL | wxRIGHT,  (Title.length() == 1 ) ? 0:5);
    return pTitle;
 }
 
 
 NumericTextCtrl * SelectionBar::AddTime( const wxString Name, int id, wxSizer * pSizer ){
-   wxString formatName = mListener ? mListener->AS_GetSelectionFormat() 
-      : wxString(wxEmptyString);
-   NumericTextCtrl * pCtrl = safenew NumericTextCtrl(
-      NumericConverter::TIME, this, id, formatName, 0.0, mRate);
+   auto formatName = mListener ? mListener->AS_GetSelectionFormat()
+      : NumericFormatSymbol{};
+   auto pCtrl = safenew NumericTextCtrl(
+      this, id, NumericConverter::TIME, formatName, 0.0, mRate);
    pCtrl->SetName(Name);
-   pCtrl->EnableMenu();
    pSizer->Add(pCtrl, 0, wxALIGN_TOP | wxRIGHT, 5);
    return pCtrl;
 }
@@ -249,54 +190,49 @@ void SelectionBar::Populate()
    // on each sizer as to what spacings it creates.
    Add((mainSizer = safenew wxFlexGridSizer(SIZER_COLS, 1, 1)), 0, wxALIGN_TOP | wxLEFT | wxTOP, 5);
 
-   //
    // Top row (mostly labels)
-   //
-
    wxColour clrText =  theTheme.Colour( clrTrackPanelText );
    wxColour clrText2 = *wxBLUE;
    AddTitle( _("Project Rate (Hz)"), mainSizer );
    AddVLine( mainSizer );
    AddTitle( _("Snap-To"), mainSizer );
-#ifdef OPTIONS_BUTTON
-   // Not enough room to say 'Selection Options".  There is a tooltip instead.
-   AddTitle( wxT(""), mainSizer );
-#endif
    AddVLine( mainSizer );
    AddTitle( _("Audio Position"), mainSizer );
    AddVLine( mainSizer );
 
    {
-   const wxString choices[4] = {
-      _("Start and End of Selection"),
-      _("Start and Length of Selection"),
-      _("Length and End of Selection"),
-      _("Length and Center of Selection"),
-   };
-   // The choice control now has a name.
-   // If this is less good for VI users, can go back to no name.
-   mChoice = safenew wxChoice
-      (this, ChoiceID, wxDefaultPosition, wxDefaultSize, 4, choices,
-       0, wxDefaultValidator, "Selection Type Chooser");
-   //mChoice->SetName(wxT("\a"));     // stop Jaws screen reader using nearby text for name when name is empty
-   mChoice->SetSelection(0);
-#ifdef __WXGTK__
-   // Combo boxes are taller on Linux, and if we don't do the following, the selection toolbar will
-   // be three units high.
-   wxSize sz = mChoice->GetBestSize();
-   sz.SetHeight( sz.y-4);
-   mChoice->SetMinSize( sz );
+      const wxString choices[4] = {
+         _("Start and End of Selection"),
+         _("Start and Length of Selection"),
+         _("Length and End of Selection"),
+         _("Length and Center of Selection"),
+      };
+      mChoice = safenew wxChoice
+         (this, ChoiceID, wxDefaultPosition, wxDefaultSize, 4, choices,
+          0, wxDefaultValidator, _("Show"));
+      mChoice->SetSelection(0);
+#if wxUSE_ACCESSIBILITY
+      // so that name can be set on a standard control
+      mChoice->SetAccessible(safenew WindowAccessible(mChoice));
 #endif
-   mainSizer->Add(mChoice, 0, wxALIGN_TOP | wxEXPAND | wxRIGHT, 6);
+#ifdef __WXGTK__
+      // Combo boxes are taller on Linux, and if we don't do the following, the selection toolbar will
+      // be three units high.
+      wxSize sz = mChoice->GetBestSize();
+      sz.SetHeight( sz.y-4);
+      mChoice->SetMinSize( sz );
+#endif
+      mainSizer->Add(mChoice, 0, wxALIGN_TOP | wxEXPAND | wxRIGHT, 6);
    }
 
-   //
    // Botton row, (mostly time controls)
-   //
-
    mRateBox = safenew wxComboBox(this, RateID,
                              wxT(""),
                              wxDefaultPosition, wxSize(80, -1));
+#if wxUSE_ACCESSIBILITY
+   // so that name can be set on a standard control
+   mRateBox->SetAccessible(safenew WindowAccessible(mRateBox));
+#endif
    mRateBox->SetName(_("Project Rate (Hz)"));
    //mRateBox->SetForegroundColour( clrText2 );
    wxTextValidator vld(wxFILTER_INCLUDE_CHAR_LIST);
@@ -315,7 +251,7 @@ void SelectionBar::Populate()
    // the control that gets the focus events.  So we have to find the
    // textctrl.
    wxWindowList kids = mRateBox->GetChildren();
-   for (unsigned int i = 0; i < kids.GetCount(); i++) {
+   for (unsigned int i = 0; i < kids.size(); i++) {
       wxClassInfo *ci = kids[i]->GetClassInfo();
       if (ci->IsKindOf(CLASSINFO(wxTextCtrl))) {
          mRateText = kids[i];
@@ -324,13 +260,11 @@ void SelectionBar::Populate()
    }
 #endif
 
-   mRateText->Connect(wxEVT_SET_FOCUS,
-                      wxFocusEventHandler(SelectionBar::OnFocus),
-                      NULL,
+   mRateText->Bind(wxEVT_SET_FOCUS,
+                      &SelectionBar::OnFocus,
                       this);
-   mRateText->Connect(wxEVT_KILL_FOCUS,
-                      wxFocusEventHandler(SelectionBar::OnFocus),
-                      NULL,
+   mRateText->Bind(wxEVT_KILL_FOCUS,
+                      &SelectionBar::OnFocus,
                       this);
 
 #ifdef __WXGTK__
@@ -358,17 +292,19 @@ void SelectionBar::Populate()
 
    mainSizer->Add(mSnapTo,
                   0, wxALIGN_TOP | wxRIGHT, 5);
+#if wxUSE_ACCESSIBILITY
+   // so that name can be set on a standard control
+   mSnapTo->SetAccessible(safenew WindowAccessible(mSnapTo));
+#endif
    mSnapTo->SetName(_("Snap To"));
    //mSnapTo->SetForegroundColour( clrText2 );
    mSnapTo->SetSelection(mListener ? mListener->AS_GetSnapTo() : SNAP_OFF);
 
-   mSnapTo->Connect(wxEVT_SET_FOCUS,
-                    wxFocusEventHandler(SelectionBar::OnFocus),
-                    NULL,
+   mSnapTo->Bind(wxEVT_SET_FOCUS,
+                    &SelectionBar::OnFocus,
                     this);
-   mSnapTo->Connect(wxEVT_KILL_FOCUS,
-                    wxFocusEventHandler(SelectionBar::OnFocus),
-                    NULL,
+   mSnapTo->Bind(wxEVT_KILL_FOCUS,
+                    &SelectionBar::OnFocus,
                     this);
 
    AddVLine( mainSizer );
@@ -392,7 +328,6 @@ void SelectionBar::Populate()
       mCenterTime = AddTime(_("Center"), CenterTimeID, hSizer.get() );
       mEndTime    = AddTime(_("End"), EndTimeID, hSizer.get() );
       mainSizer->Add(hSizer.release(), 0, wxALIGN_TOP | wxRIGHT, 0);
-
    }
 
    mChoice->MoveBeforeInTabOrder( mStartTime );
@@ -408,7 +343,12 @@ void SelectionBar::Populate()
 
 void SelectionBar::UpdatePrefs()
 {
-   mRate = (double) gPrefs->Read(wxT("/SamplingRate/DefaultProjectSampleRate"), AudioIO::GetOptimalSupportedSampleRate());
+   // The project rate is no longer driven from here.
+   // When preferences change, the Project learns about it too.
+   // If necessary we can drive the SelectionBar mRate via the Project
+   // calling our SetRate().
+   // As of 13-Sep-2018, changes to the sample rate pref will only affect 
+   // creation of new projects, not the smaple rate in existing ones.
 
    wxCommandEvent e;
    e.SetInt(mStartTime->GetFormatIndex());
@@ -418,7 +358,6 @@ void SelectionBar::UpdatePrefs()
    SetLabel(_("Selection"));
 
    RegenerateTooltips();
-
    // Give base class a chance
    ToolBar::UpdatePrefs();
 }
@@ -434,8 +373,13 @@ void SelectionBar::SetListener(SelectionBarListener *l)
 void SelectionBar::RegenerateTooltips()
 {
 #if wxUSE_TOOLTIPS
-   wxString formatName = mListener ? mListener->AS_GetSelectionFormat() : wxString(wxEmptyString);
-   mSnapTo->SetToolTip(wxString::Format(_("Snap Clicks/Selections to %s"), formatName));
+   auto formatName =
+      mListener
+         ? mListener->AS_GetSelectionFormat()
+         : NumericFormatSymbol{};
+   mSnapTo->SetToolTip(
+      wxString::Format(
+         _("Snap Clicks/Selections to %s"), formatName.Translation() ));
 #endif
 }
 
@@ -467,6 +411,18 @@ void SelectionBar::ModifySelection(int newDriver, bool done)
    if( newDriver == CenterTimeID )
       mCenter = mCenterTime->GetValue();
 
+   // There are four controls, and two constraints, which are:
+   //    mid = (start+end)/2
+   //    length = (end-start)
+   // Therefore we can select any two controls as 'drivers' of
+   // the other two.
+   // Here we compute 'i' which combines the identity of the two 
+   // driving controls, to use it as an index.
+   // The order of the two drivers generally does not matter much,
+   // except that we have want:
+   //    start < end
+   // and preserve that by adjusting the least dominant driving
+   // control.
    int i = mDrive1 + 4 * mDrive2;
    switch(i){
    case StartTimeID + 4 * EndTimeID:
@@ -531,36 +487,6 @@ void SelectionBar::OnChangedTime(wxCommandEvent & event)
    ModifySelection(event.GetId(), event.GetInt() != 0);
 }
 
-
-void SelectionBar::OnTitleClicked(int newDriver)
-{
-   // Ensure newDriver is the most recent driver.
-   if( newDriver != mDrive2 )
-      SetDrivers( mDrive2, newDriver);
-}
-
-// These functions give the IDs of the associated control, NOT the ID of the title.
-void SelectionBar::OnStartTitleClicked(wxMouseEvent & WXUNUSED(event)){ OnTitleClicked( StartTimeID );};
-void SelectionBar::OnLengthTitleClicked(wxMouseEvent & WXUNUSED(event)){ OnTitleClicked( LengthTimeID );};
-void SelectionBar::OnCenterTitleClicked(wxMouseEvent & WXUNUSED(event)){ OnTitleClicked( CenterTimeID );};
-void SelectionBar::OnEndTitleClicked(wxMouseEvent & WXUNUSED(event)){ OnTitleClicked( EndTimeID );};
-
-void SelectionBar::OnModeDecClicked(wxMouseEvent & WXUNUSED(event)){
-   SetSelectionMode( (mSelectionMode +3)%4 );
-   SelectionModeUpdated();
-}
-
-void SelectionBar::OnModeIncClicked(wxMouseEvent & WXUNUSED(event)){
-   SetSelectionMode( (mSelectionMode +1)%4 );
-   SelectionModeUpdated();
-}
-
-void SelectionBar::OnChooserTitleClicked(wxMouseEvent & WXUNUSED(event)){
-   wxCommandEvent evt;
-   OnButton( evt );
-}
-
-
 // Called when one of the format drop downs is changed.
 void SelectionBar::OnUpdate(wxCommandEvent &evt)
 {
@@ -575,11 +501,11 @@ void SelectionBar::OnUpdate(wxCommandEvent &evt)
 
    evt.Skip(false);
 
-   wxString format;
-
    // Save format name before recreating the controls so they resize properly
-   format = mStartTime->GetBuiltinName(index);
-   mListener->AS_SetSelectionFormat(format);
+   {
+      auto format = mStartTime->GetBuiltinName(index);
+      mListener->AS_SetSelectionFormat(format);
+   }
 
    RegenerateTooltips();
 
@@ -595,7 +521,7 @@ void SelectionBar::OnUpdate(wxCommandEvent &evt)
 
    ValuesToControls();
 
-   format = mStartTime->GetBuiltinFormat(index);
+   auto format = mStartTime->GetBuiltinFormat(index);
    for( i=0;i<5;i++)
       (*Ctrls[i])->SetFormatString( format );
 
@@ -623,54 +549,13 @@ void SelectionBar::SetDrivers( int driver1, int driver2 )
       wxString Format = ( (id!=mDrive1) && (id!=mDrive2 ) ) ? _("%s - driven") : "%s";
       wxString Title= wxString::Format( Format, Temp );
       // i18n-hint: %s1 is replaced e.g by 'Length', %s2 e.g by 'Center'.
-      wxString VoiceOverText = wxString::Format(_("Selection %s.  %s won't change."), Temp, Text[fixed]);
-      // i18n-hint: %s is replaced e.g by 'Length'.  This is a tooltip on a numerical control.
-      //wxString Tooltip = wxString::Format( _(" With %s fixed.  (Use context menu to change format.) "),  Text[fixed] );
+      wxString VoiceOverText = wxString::Format(_("Selection %s. %s won't change."), Temp, Text[fixed]);
       if( *Ctrls[i] ){
          (*Ctrls[i])->SetName( Temp );
-         //(*Ctrls[i])->SetToolTip( Tooltip );
       }
    }
 }
 
-
-// There currently is only one clickable AButton
-// so we just do what it needs.
-void SelectionBar::OnButton(wxCommandEvent & WXUNUSED(event))
-{
-   AudacityProject *p = GetActiveProject();
-   if (!p) return;
-
-   AButton * pBtn = mButtons[ SelTBMenuID-SelTBFirstButton];// use event.GetId();
-
-   auto cleanup = finally( [&] { pBtn->InteractionOver();}
-   );
-   SetFocus();
-
-   wxMenu Menu;
-   Menu.AppendRadioItem( 0, _("Start - End") );
-   Menu.AppendRadioItem( 1, _("Start - Length") );
-   Menu.AppendRadioItem( 2, _("Length - End") );
-   Menu.AppendRadioItem( 3, _("Length - Center") );
-   Menu.Check( mSelectionMode, true );
-   // Pop it up where the mouse is.
-   pBtn->PopupMenu(&Menu);//, wxPoint(0, 0));
-
-   // only one radio button should be checked.
-   for( int i=0;i<4;i++)
-      if( Menu.IsChecked(i))
-         SetSelectionMode( i );
-
-   SelectionModeUpdated();
-}
-
-void SelectionBar::OnFieldChoice(wxCommandEvent &event)
-{
-   int id = event.GetId();
-   SetSelectionMode( id - StartEndRadioID );
-   SelectionModeUpdated();
-}
-
 void SelectionBar::OnChoice(wxCommandEvent & WXUNUSED(event))
 {
    int mode = mChoice->GetSelection();
@@ -692,7 +577,9 @@ void SelectionBar::SelectionModeUpdated()
    Updated();
 }
 
-
+// We used to have 8 modes which showed different combinations of the
+// length, start, end, center controls.
+// Mode 7 for example showed all four at the same time.
 void SelectionBar::SetSelectionMode(int mode)
 {
    // Only modes 0 to 3 are now supported,
@@ -722,10 +609,10 @@ void SelectionBar::SetSelectionMode(int mode)
 // Our mode determines which controls are visible.
 void SelectionBar::ShowHideControls(int mode)
 {
-   // These 
+   // The bits in these say which controls are visible.
    int masks[8]= { 
-      9, 5, 12, 6, // 2 items
-      13, 7, 11,// 3 items
+      9, 5, 12, 6, // 2 items shown
+      13, 7, 11,// 3 items shown
       15};
    int mask = masks[mode];
 
@@ -746,6 +633,7 @@ void SelectionBar::ValuesToControls()
          (*Ctrls[i])->SetValue( Values[i] );
 }
 
+// A time has been set.  Update the control values.
 void SelectionBar::SetTimes(double start, double end, double audio)
 {
    mStart = start;
@@ -757,23 +645,12 @@ void SelectionBar::SetTimes(double start, double end, double audio)
    ValuesToControls();
 }
 
-void SelectionBar::SetField(const wxChar *msg, int fieldNum)
-{
-   if (fieldNum < 0 || fieldNum >= 10)
-      return;
-
-   if (mField[fieldNum] != msg) {
-      mField[fieldNum] = msg;
-      Refresh(false);
-   }
-}
-
 void SelectionBar::SetSnapTo(int snap)
 {
    mSnapTo->SetSelection(snap);
 }
 
-void SelectionBar::SetSelectionFormat(const wxString & format)
+void SelectionBar::SetSelectionFormat(const NumericFormatSymbol & format)
 {
    mStartTime->SetFormatString(mStartTime->GetBuiltinFormat(format));
 
@@ -788,6 +665,7 @@ void SelectionBar::SetRate(double rate)
       // if the rate is actually being changed
       mRate = rate;   // update the stored rate
       mRateBox->SetValue(wxString::Format(wxT("%d"), (int)rate));
+
       // update the TimeTextCtrls if they exist
       NumericTextCtrl ** Ctrls[5] = { &mStartTime, &mEndTime, &mLengthTime, &mCenterTime, &mAudioTime };
       int i;
