@@ -10,16 +10,23 @@ Paul Licameli split from TrackPanel.cpp
 
 #include "../../Audacity.h"
 #include "TrackControls.h"
+
 #include "TrackButtonHandles.h"
 #include "TrackSelectHandle.h"
 #include "../../HitTestResult.h"
 #include "../../RefreshCode.h"
-#include "../../MixerBoard.h"
+#include "../../Menus.h"
 #include "../../Project.h"
 #include "../../TrackPanel.h" // for TrackInfo
 #include "../../TrackPanelMouseEvent.h"
 #include "../../Track.h"
 #include <wx/textdlg.h>
+#include "../../commands/CommandType.h"
+#include "../../commands/Command.h"
+#include "../../commands/CommandManager.h"
+#include "../../ShuttleGui.h"
+#include "../../widgets/PopupMenuTable.h"
+
 
 TrackControls::TrackControls( std::shared_ptr<Track> pTrack )
    : mwTrack{ pTrack }
@@ -30,7 +37,7 @@ TrackControls::~TrackControls()
 {
 }
 
-std::shared_ptr<Track> TrackControls::FindTrack()
+std::shared_ptr<Track> TrackControls::DoFindTrack()
 {
    return mwTrack.lock();
 }
@@ -62,6 +69,10 @@ std::vector<UIHandlePtr> TrackControls::HitTest
       mMinimizeHandle, state, rect, this)))
       results.push_back(result);
 
+   if (NULL != (result = SelectButtonHandle::HitTest(
+      mSelectButtonHandle, state, rect, this)))
+      results.push_back(result);
+
    if (results.empty()) {
       if (NULL != (result = TrackSelectHandle::HitAnywhere(
          mSelectHandle, pTrack)))
@@ -125,33 +136,71 @@ BEGIN_POPUP_MENU(TrackMenuTable)
    POPUP_MENU_ITEM(OnSetNameID, _("&Name..."), OnSetName)
    POPUP_MENU_SEPARATOR()
    POPUP_MENU_ITEM(
-      // It is not correct to use KeyStringDisplay here -- wxWidgets will apply
-      // its equivalent to the key names passed to menu functions.
+      // It is not correct to use NormalizedKeyString::Display here --
+      // wxWidgets will apply its equivalent to the key names passed to menu
+      // functions.
       OnMoveUpID,
       _("Move Track &Up") + wxT("\t") +
          (GetActiveProject()->GetCommandManager()->
-          GetKeyFromName(wxT("TrackMoveUp"))),
+          GetKeyFromName(wxT("TrackMoveUp")).Raw()),
       OnMoveTrack)
    POPUP_MENU_ITEM(
       OnMoveDownID,
       _("Move Track &Down") + wxT("\t") +
          (GetActiveProject()->GetCommandManager()->
-          GetKeyFromName(wxT("TrackMoveDown"))),
+          GetKeyFromName(wxT("TrackMoveDown")).Raw()),
       OnMoveTrack)
    POPUP_MENU_ITEM(
       OnMoveTopID,
       _("Move Track to &Top") + wxT("\t") +
          (GetActiveProject()->GetCommandManager()->
-          GetKeyFromName(wxT("TrackMoveTop"))),
+          GetKeyFromName(wxT("TrackMoveTop")).Raw()),
       OnMoveTrack)
    POPUP_MENU_ITEM(
       OnMoveBottomID,
       _("Move Track to &Bottom") + wxT("\t") +
          (GetActiveProject()->GetCommandManager()->
-          GetKeyFromName(wxT("TrackMoveBottom"))),
+          GetKeyFromName(wxT("TrackMoveBottom")).Raw()),
       OnMoveTrack)
 END_POPUP_MENU()
 
+
+
+
+#define SET_TRACK_NAME_PLUGIN_SYMBOL ComponentInterfaceSymbol{ XO("Set Track Name") }
+
+// An example of using an AudacityCommand simply to create a dialog.
+// We can add additional functions later, if we want to make it
+// available to scripting.
+// However there is no reason to, as SetTrackStatus is already provided.
+class SetTrackNameCommand : public AudacityCommand
+{
+public:
+   // ComponentInterface overrides
+   ComponentInterfaceSymbol GetSymbol() override
+   {return SET_TRACK_NAME_PLUGIN_SYMBOL;};
+   //wxString GetDescription() override {return _("Sets the track name.");};
+   //bool DefineParams( ShuttleParams & S ) override;
+   void PopulateOrExchange(ShuttleGui & S) override;
+   //bool Apply(const CommandContext & context) override;
+
+   // Provide an override, if we want the help button.
+   // wxString ManualPage() override {return wxT("");};
+public:
+   wxString mName;
+};
+
+void SetTrackNameCommand::PopulateOrExchange(ShuttleGui & S)
+{
+   S.AddSpace(0, 5);
+
+   S.StartMultiColumn(2, wxALIGN_CENTER);
+   {
+      S.TieTextBox(_("Name:"),mName);
+   }
+   S.EndMultiColumn();
+}
+
 void TrackMenuTable::OnSetName(wxCommandEvent &)
 {
    Track *const pTrack = mpData->pTrack;
@@ -159,21 +208,16 @@ void TrackMenuTable::OnSetName(wxCommandEvent &)
    {
       AudacityProject *const proj = ::GetActiveProject();
       const wxString oldName = pTrack->GetName();
-      const wxString newName =
-         wxGetTextFromUser(_("Change track name to:"),
-         _("Track Name"), oldName);
-      if (newName != wxT("")) // wxGetTextFromUser returns empty string on Cancel.
-      {
-         pTrack->SetName(newName);
-         // if we have a linked channel this name should change as well
-         // (otherwise sort by name and time will crash).
-         if (pTrack->GetLinked())
-            pTrack->GetLink()->SetName(newName);
 
-         MixerBoard *const pMixerBoard = proj->GetMixerBoard();
-         auto pt = dynamic_cast<PlayableTrack*>(pTrack);
-         if (pt && pMixerBoard)
-            pMixerBoard->UpdateName(pt);
+      SetTrackNameCommand Command;
+      Command.mName = oldName;
+      // Bug 1837 : We need an OK/Cancel result if we are to enter a blank string.
+      bool bResult = Command.PromptUser( proj );
+      if (bResult) 
+      {
+         wxString newName = Command.mName;
+         for (auto channel : TrackList::Channels(pTrack))
+            channel->SetName(newName);
 
          proj->PushState(wxString::Format(_("Renamed '%s' to '%s'"),
             oldName,
@@ -188,21 +232,21 @@ void TrackMenuTable::OnSetName(wxCommandEvent &)
 void TrackMenuTable::OnMoveTrack(wxCommandEvent &event)
 {
    AudacityProject *const project = GetActiveProject();
-   AudacityProject::MoveChoice choice;
+   TrackActions::MoveChoice choice;
    switch (event.GetId()) {
    default:
       wxASSERT(false);
    case OnMoveUpID:
-      choice = AudacityProject::OnMoveUpID; break;
+      choice = TrackActions::OnMoveUpID; break;
    case OnMoveDownID:
-      choice = AudacityProject::OnMoveDownID; break;
+      choice = TrackActions::OnMoveDownID; break;
    case OnMoveTopID:
-      choice = AudacityProject::OnMoveTopID; break;
+      choice = TrackActions::OnMoveTopID; break;
    case OnMoveBottomID:
-      choice = AudacityProject::OnMoveBottomID; break;
+      choice = TrackActions::OnMoveBottomID; break;
    }
 
-   project->MoveTrack(mpData->pTrack, choice);
+   TrackActions::DoMoveTrack(*project, mpData->pTrack, choice);
 
    // MoveTrack already refreshed TrackPanel, which means repaint will happen.
    // This is a harmless redundancy:
