@@ -17,6 +17,8 @@ Paul Licameli split from TrackPanel.cpp
 class Track;
 class wxDC;
 
+const int kGuard = 5; // 5 pixels to reduce risk of VZooming accidentally
+
 class TrackVRulerControls /* not final */ : public CommonTrackPanelCell
 {
 public:
@@ -25,8 +27,6 @@ public:
 
    virtual ~TrackVRulerControls() = 0;
 
-   std::shared_ptr<Track> FindTrack() override;
-
    // Define a default hit test method, just for message and cursor
    std::vector<UIHandlePtr> HitTest
       (const TrackPanelMouseState &state,
@@ -37,6 +37,8 @@ public:
         int zoomStart, int zoomEnd);
 
 protected:
+   std::shared_ptr<Track> DoFindTrack() override;
+
    Track *GetTrack() const;
 
    std::weak_ptr<Track> mwTrack;
